;------------------------------------------------------------
; Schakelklok V2
;
; Hardware loopt op 11.0592 MHz extern PLL4, uitvoering 46K22
;
; Hardware : poort A	A0 = Fotocel		I
;			A1 =
;                       A2 = ServiceLED         O
;			A3 = PhotoLED		O
;                       A4 = Toggle50		O
;			A5 =
;
;            poort B	B0 = Uitgang T1		O
;			B1 = Uitgang T2		O
;			B2 = Uitgang T3		O
;			B3 = Uitgang T4		O
;			B4 = Uitgang T5		O
;			B5 = Uitgang T6		O
;                       B6 = Uitgang T7		O
;			B7 = Uitgang T8		O
;
;            poort C	C0 = D/C signaal OLED	O
;                       C1 = W signaal OLED	O
;                       C2 =
;                       C3 =
;			C4 =
;                       C5 = RES signaal OLED	O
;                       C6 = RS232W		O
;                       C7 = RS232R             I
;
;            poort D	D0 = Data 0 OLED	I/O
;			D1 = Data 1 OLED	I/O
;			D2 = Data 2 OLED	I/O
;			D3 = Data 3 OLED	I/O
;			D4 = Data 4 OLED	I/O
;			D5 = Data 5 OLED	I/O
;			D6 = Data 6 OLED	I/O
;			D7 = Data 7 OLED	I/O
;
;	     poort E	E0 = Rotaswitch   	I
;			E1 = Rotaswitch         I
;                       E2 = Rota drukknop      I
;
; W.Tak juni 2014
;
;
;
; Versie 2.0  maart 2018
;
; Aangepast voor 18F46K22, Midas display en UBlox ontvanger
;
;------------------------------------------------------------

#INCLUDE  	<P18F46K22.INC>

	CONFIG	FOSC=HSMP, PLLCFG=ON	;Extern kristal, geen watchdog
	CONFIG  PBADEN=OFF, LVP=OFF     ;PORTB digital I/O, low voltage programming off
	CONFIG	WDTEN=SWON, CPD=OFF     ;Watchdog via software, geen EEPROM protectie
	CONFIG  WDTPS=8192		;32 sec
	CONFIG	PWRTEN=ON

DELAY_H_1	EQU	0X10		;Delay in 0.5 msec. loop
DELAY_L_1	EQU	0X72            ;1072 at 10.0592 x 4 MHz

DELAY_H_10	EQU	0XA0            ;Delay in 5 msec. loop
DELAY_L_10	EQU	0X71            ;A071 at 10.0592 x 4 MHz

INT_VAL_H	EQU     0X79		;7902 is 50 msec. bij 22.1184 MHz; af te regelen in de
INT_VAL_L	EQU     0X02            ;schakeling met eigen kristal en Vcc

GPS_ERR_MAX	EQU	0X19		;Max. 25 opeenvolgende GPS missers

STATUS_TIME	EQU	0X5A		;Tijd zichtbaarheid statusscreen

DEF_BLOCK_H	EQU	0X02		;Default tijd voor blokkade (258 = 10 minuten)
DEF_BLOCK_L	EQU     0X58

#DEFINE I_D		LATC,0          ;0 = instructie; 1 = data
#DEFINE WRITE		LATC,1
#DEFINE RESET		LATC,5          ;0 = reset

#DEFINE SERV_LED	LATA,2		;Service LED intern
#DEFINE PHOTO_LED	LATA,3		;LED bij fotocel
#DEFINE	T50		LATA,4

#DEFINE JUMPER		PORTA,5		;Jumper 10 minuten vertraging

#DEFINE RS_IN		PORTC,7		;Ingang RS232

#DEFINE DS1		PORTE,0 	;Rotary encoder
#DEFINE DS2		PORTE,1
#DEFINE ROT_SW		PORTE,2

;FLAGS byte

#DEFINE GPS_STABLE	FLAGS,0		;1 als 10 correcte opeenvolgende GPS strings geweest zijn
#DEFINE INT		FLAGS,1 	;Wordt elke 50 msec. geset in interrupt routine
#DEFINE RS232_ERR	FLAGS,2         ;Time out in RS232
#DEFINE DATA_VALID	FLAGS,3 	;1 als GPS string is goedgekeurd
#DEFINE CH_SUM_OK	FLAGS,4 	;1 als checksum OK is
#DEFINE FIRST_GPS	FLAGS,5		;1 na de eerste string
#DEFINE CHECKIT		FLAGS,6		;1 als de tijden afgestemd moeten worden
#DEFINE SCREEN_FLAG	FLAGS,7         ;0 is scherm1, 1 is scherm2

#DEFINE PHOTOCELL	FLAGS2,0	;0 is donker, 1 is licht
#DEFINE NO_GPS		FLAGS2,1        ;Geen GPS
#DEFINE SHOW		FLAGS2,2        ;Forceert weergave in TEXT routine
#DEFINE DISA_ROT_SW	FLAGS2,3	;Disablet verder kijken naar ROT_SW in interruptroutine
#DEFINE PRESS_IN_INT	FLAGS2,4	;1 als gedrukt op de rotary in interrupt
#DEFINE STATUS_ACT	FLAGS2,5	;1 als statusscreen actief is
#DEFINE ERA_STAT	FLAGS2,6 	;1 om statusscreen te wissen
#DEFINE D_ERR_FLAG	FLAGS2,7        ;Datumfout

#DEFINE SPEC_DAY	FLAGS3,0	;1 bij zater-,zon- of feestdag
#DEFINE DS_CU		FLAGS3,1        ;Rotaryflags
#DEFINE DS_CD		FLAGS3,2
#DEFINE DS_CHANGED	FLAGS3,3
#DEFINE ENA_INPS	FLAGS3,4	;1 als aanvraag inputscreen mag
#DEFINE BLOCKED		FLAGS3,5	;1 als geen A/D conversie is toegestaan
#DEFINE S_U_FLAG	FLAGS3,6	;1 gedurende opstart
#DEFINE DATE_WRONG	FLAGS3,7	;1 bij datumfout

#DEFINE D_E_SHOWN	FLAGS4,0	;1 als de GPS error is getoond
#DEFINE BUT_FORCED	FLAGS4,1	;1 als time out op button is verstreken
#DEFINE DATE_COR	FLAGS4,2	;1 als de datum moet worden verhoogd

#define dump_data	latc,3
#define dump_clock	latc,4

;Start variabelen declaratie

	CBLOCK 0X0

		FLAGS
		FLAGS2
		FLAGS3
		FLAGS4

		GEN_CNT
		LOOP_CNT
		TEMP
		TEMP2
		TEMP_H
		TEMP_L
		
		WREG_LOC
		TEMP_LOC
		
		DELAY_LOC		;Aantal vertragingen in delay routines
		DELAY_TYPE		;1 of 2 voor 0.5 of 5 msec.
		DELAY_H                 ;Timers in delay routines
		DELAY_L
		CNT_DELAY_H
		CNT_DELAY_L
		MAIN_DELAY
		DTS_TIMER
		L_CNT

		WD_TIMER_RS232H 	;WD RS232
		WD_TIMER_RS232L

		CHAR_CNT 		;Telt karakters in een string
		
		INT_CNT 		;Teller voor de 50 msec. interrupt

		GPS_STATUS		;Status veld van GPRMC

                HR_10_ASC		;Gelezen uit de GPS module
		HR_1_ASC
		MIN_10_ASC
		MIN_1_ASC
		SEC_10_ASC
		SEC_1_ASC

		DAY_10_ASC
		DAY_1_ASC
		MONTH_10_ASC
		MONTH_1_ASC
		YEAR_10_ASC
		YEAR_1_ASC
		
		HR_BCD			;GPS tijd in BCD
		MIN_BCD
		SEC_BCD

		DAY_BCD  		;GPS datum in BCD
		MONTH_BCD
		YEAR_BCD

		GPS_HR_HEX		;GPS tijd in hex
		GPS_MIN_HEX
		GPS_SEC_HEX

		G_HR_HEX 		;Rekenvelden GPS getal
		G_MIN_HEX
		G_SEC_HEX

		G_VAL_U			;Het 24 bits GPS getal
		G_VAL_H
		G_VAL_L

                PIC_HR_HEX 		;PIC tijd in HEX
                PIC_MIN_HEX
		PIC_SEC_HEX
		
		PIC_HR_BCD 		;PIC tijd in BCD
		PIC_MIN_BCD
		PIC_SEC_BCD
		
		P_HR_HEX 		;Rekenvelden PIC getal
		P_MIN_HEX
		P_SEC_HEX
		
		P_VAL_U			;Het 24 bits PIC getal
		P_VAL_H
		P_VAL_L
		
		SUM			;Berekende checksum
		CS_H			;Tussenwaardes checksum berekening
		CS_L

                OFFSET_T

		TOT_HRS_H		;Aantal verstreken uren huidige jaar
                TOT_HRS_L

		LIMD_HRS_H		;Grens voor winter- naar zomertijd
		LIMD_HRS_L

		LIMU_HRS_H              ;Grens voor zomer- naar wintertijd
		LIMU_HRS_L

		ADJUST_CNT		;Dode band voor time adjust
		
		GPS_ERR_CNT		;Teller gemiste GPS strings
		
		SUB1_U
		SUB1_H
		SUB1_L
		
		SUB2_U
		SUB2_H
		SUB2_L
		
		SUBRES_U
		SUBRES_H
		SUBRES_L
		
		SEC_0_BUF:0X04
		SEC_1_BUF:0X04
		SEC_2_BUF:0X04
		SEC_3_BUF:0X04
		SEC_4_BUF:0X04
		SEC_5_BUF:0X04
		SEC_6_BUF:0X04
		SEC_7_BUF:0X04
		SEC_8_BUF:0X04
		SEC_9_BUF:0X04
		
		MIN_CNT_H		;Minutenteller 0 t/m 1439
		MIN_CNT_L               ;in hex 000H t/m 59FH
                
		OLD_MIN			;Vorige minuut
		
		STATUS_CNT		;Teller statusscreen aan in sec.

		EE_IN			;Gelezen byte uit EEPROM
		
		DIV_H                   ;Gebruikt in deel door 60 routine
		DIV_L

		DRES_H
		DRES_L
		
		A_TIME			;Tijd in tijdvelden

		ERROR_CODE              ;Foutcode

		S_POS			;Schermlokatie in 20/21 stappen
		
		SW_STAT			;Status van de draaischakelaar
		TRANS			;Transitiebyte van DS
		SW_CNT_U		;Vertragingstellers DS
		SW_CNT_D
		SW_DELAY		;De vertraging
		
		LOWER_LIM		;Onderwaarde in INPUT_SCREEN
		
		GEN_MODE		;Hulpvariabele MODE bytes
		
		HP_STORE		;Tussenopslag lokatie
		VP_STORE

		MAX_VAL                 ;Maximum in ASK_TIME

		STEP                    ;Stapgrootte in ASK_TIME
		
		START_H			;Rekenvariabelen voor window_RAM
		START_L
		
		STOP_H
		STOP_L

		T_POINT_H
		T_POINT_L

		DURATION_H
		DURATION_L
		
		REL_BYTE                ;De uitgangsbits
		
		MODE_BU			;Hulpopslag MODE in STATUS_SCREEN
				
		;OLED variabelen

                CHAR_WIDTH              ;Breedte van karakter
		CHAR_HEIGHT             ;Hoogte van karakter
		CHAR_WIDTH2		;Kopie CHAR_WIDTH

                PAGE_NR			;Pagina, nodig in directe mode
		COL_NR
		COL_H
		COL_L

		H_POS 			;Startlokatie karakter(s) horizontaal,zero based
		V_POS                   ;Startlokatie karakter(s) verticaal, zero based

		ADR_LFH			;LFSR0 adres voor berekening
		ADR_LFL

                COMM_BYTE		;Gebruikt in manipulatieroutine
		COMM_BYTE1		;De 4 gelezen opdrachtbytes
                COMM_BYTE2
                COMM_BYTE3
                COMM_BYTE4
                COMM_BYTE5		;Extra voor shiftregister

                OFFSET			;0 .. 7 voor startlocatie in V richting

		TYPE_BYTE		;Gebruikt in manipulatieroutine
		TYPE_BYTE1 		;De 5 bytes met TYPE voor de
		TYPE_BYTE2              ;bitmanipulatie
		TYPE_BYTE3
		TYPE_BYTE4
		TYPE_BYTE5
		SEQ_BYTE 		;Volgorde; alleen voor testdoeleinden

		CONT_BYTE		;Contentbyte (inhoud shadowRAM)
		
		TPH_BU			;Push lokatie TBLPTRx
		TPL_BU

		TXT_AREA:0X20		;Tekstveld

		ANA_OLD                 ;Vergeljking in analoge meting
		ANA_NEW

		REL_STATUS		;Hulpbyte met REL_BYTE data
		
		FSR0H_BU
		FSR0L_BU
		FSR1H_BU
		FSR1L_BU
		FSR2H_BU
		FSR2L_BU
		
		AV_TOT			;Berekening gemiddelde lichtsterkte

		BLOCK_CNT_H		;Teller voor A/D blokkade
		BLOCK_CNT_L

		S_U_TIMER		;30 sec. vetraging A/D meting
		
		L3_CNT			;Teller voor knipper in blokkadestand

		BEAM_LENGTH		;Lengte balk bij init

		FORCE_TMRL

		GMT			;0,1 of 2 voor GMT zone 0,+1 of +2

		BRIGHT			;Helderheid OLED

		dd_nr			;aantal te zenden bytes (max. 16) tevens
					;indicatie bit 7 voor nieuw(0) of doortellen (1)
			
		dump_push0
		dump_push1
		dump_push2
		dump_push3

		dd_0 			;de dumpdata
		dd_1
		dd_2
		dd_3
                dd_4
		dd_5
		dd_6
		dd_7
		dd_8
		dd_9
		dd_10
		dd_11
		dd_12
		dd_13
		dd_14
		dd_15

	ENDC
	
	CBLOCK 0X100
	
		B0_0:0X3FF 		;ShadowRAM

	ENDC

        CBLOCK 0X500			;Serial data send buffer

		SEND_SER_BUF:0X40

		REC_SER_BUF:0X80

	ENDC

	CBLOCK	0X600			;Timervariabelen. LET OP: absoluut geprogrammeerd
					;			  dus niet verplaatsen
		T1A_H 			;T zijn de timervelden in minutenteller
		T1A_L                   ;dus van 0 t/m 1439
					;TxA staat voor aan
                T1U_H                   ;TxU staat voor uit
                T1U_L

                T2A_H
		T2A_L

                T2U_H
                T2U_L 			;608

                T3A_H
		T3A_L

                T3U_H
                T3U_L

                T4A_H
		T4A_L

                T4U_H
                T4U_L

                T5A_H 			;610
		T5A_L

                T5U_H
                T5U_L

                T6A_H
		T6A_L

                T6U_H
                T6U_L

                T7A_H			;618
		T7A_L

                T7U_H
                T7U_L

                T8A_H
		T8A_L

                T8U_H
                T8U_L
                
                MODE_T1 		;Modeveld per timer   ;620
                MODE_T2                 ;00 = Timer staat uit
                MODE_T3                 ;10 = Timer aan, 7 dagen
                MODE_T4                 ;20 = Timer aan, doordeweeks, geen weekend/feestdagen
                MODE_T5                 ;30 = Timer aan, alleen weekend/feestdagen
                MODE_T6                 ;40 = Timer aan, mode 10 met lichtdetectie
                MODE_T7                 ;50 = Timer aan, mode 20 met lichtdetectie
                MODE_T8                 ;60 = Timer aan, mode 30 met lichtdetectie
                                        ;70 = Timer uit, alleen lichtdetectie
                T1A_HD			;628
		T1A_LD

                T1U_HD
                T1U_LD

                T2A_HD
		T2A_LD

                T2U_HD
                T2U_LD

                T3A_HD			;630
		T3A_LD

                T3U_HD
                T3U_LD

                T4A_HD
		T4A_LD

                T4U_HD
                T4U_LD

                T5A_HD			;640
		T5A_LD

                T5U_HD
                T5U_LD

                T6A_HD
		T6A_LD

                T6U_HD
                T6U_LD

                T7A_HD			;648
		T7A_LD

                T7U_HD
                T7U_LD

                T8A_HD
		T8A_LD

                T8U_HD
                T8U_LD
                
                MODE_T1B 		;Bitwaarde mode
                MODE_T2B                ;Bit 0 = Timer aan/uit
                MODE_T3B                ;Bit 1 = 7 dagen mode
                MODE_T4B                ;Bit 2 = 5 dagen mode
                MODE_T5B                ;Bit 3 = 2 dagen mode
                MODE_T6B                ;Bit 4 = lichtcel aan/uit
                MODE_T7B
                MODE_T8B

                H_POS1
                H_POS2
                H_POS3
                H_POS4
                H_POS5
                H_POS6
                H_POS7
                H_POS8
                H_POS9
                H_POSD
                
                RAM_TEXT:0X20		;Tekstruimte vanuit RAM
                
                AV_0			;Gemm. lichtwaarden
		AV_1
		AV_2
		AV_3
		AV_4
		AV_5
		AV_6
		AV_7

        ENDC

	CBLOCK	0X700                   ;Adres wordt absoluut gebruikt: niet verplaatsen!
	                                ;Gecheckt wordt op eindadres CA0H (1440 vanaf 700H)
		W_0:0X5FF		;Schakelwindow RAM

	ENDC

;Start program area

	ORG 0X00

		GOTO	MAIN		;Reset Vector

	ORG 0X08

		GOTO	INT_TIME	;Interrupt vector timer

;Lookup table area start op 1000

	ORG 0X100
	
EE1_TXT:	DATA	"EEPROM is leeg."
		;DATA	"EEPROM empty.  "
		DB	0XFF
EE2_TXT:	DATA	"Defaults worden"
		;DATA	"Defaults are   "
		DB	0XFF
EE3_TXT:	DATA	"gebruikt."
		;DATA	"used.    "
		DB	0XFF

INTER_TXT:	DATA	": "
		DB	0XFF

DATE_ERR_TXT:	DATA	"Fout in GPS data"
		;DATA	"Error GPS data  "
		DB	0XFF
		
DATE_RES_TXT:	DB	0X24,0X24,0X24,0X24
		DB	0XFF

UNIT_TXT:	DATA	"       Timer 8"   
		DB	0XFF

START_TXT:	DATA	"GPS data lezen"
		;DATA	"Read GPS data "
		DB	0XFF

ASK_CL1_TXT:	DATA	"Druk toets als de"
		;DATA	"Press key when   "
                DB	0XFF
ASK_CL2_TXT:	DATA	"tijd correct is."
		;DATA	"time is correct."
                DB	0XFF

STORE_TXT1:	DATA	"   Data opslaan."
		;DATA	"    Data write. "
		DB	0XFF

STORE_TXT2:	DATA	" Reset nu de unit."
		;DATA	" Press reset key. "
		DB	0XFF

RUNNING_TXT:	DATA	"Unit start nu op"
		;DATA 	"Unit is starting"
		DB	0XFF

REL_TXT:	DATA	"Laat drukknop los"
		;DATA	"Release key      "
		DB	0XFF

BRIGHT_TXT1:	DATA	"Stel helderheid in. "
		;DATA	"Adjust brightness.  "
		DB	0XFF

BRIGHT_TXT2:	DATA	"Terug met reset."
		;DATA 	"Back with reset."
                DB	0XFF
                
BRIGHT_TXT3:	DATA	"Helderheid :  "
		;DATA	"Press key.  "
		DB	0XFF

TIMEZ_TXT:	DATA	"Instellen tijdzone"
		;DATA	"Adjust timezone.  "
		DB	0XFF
		
GMT_TXT:	DATA	"Zone : GMT +"
		DB	0XFF
		
SAVE_TXT0:	DATA	"Momentje... "
		;DATA	"Please wait .."
		DB	0XFF
		
SAVE_TXT1:	DATA	"Data opslaan en "
		;DATA	"Store data and  "
		DB	0XFF
		
SAVE_TXT2:	DATA	"unit resetten."
		;DATA	"reset unit.   "
		DB	0XFF

OK_TXT:		DB	"ok"
		DB	0XFF
                
T1_TXT:		DB	"t1"
		DB	0XFF

T2_TXT:		DB	"t2"
		DB	0XFF

T3_TXT:		DB	"t3"
		DB	0XFF

T4_TXT:		DB	"t4"
		DB	0XFF

T5_TXT:		DB	"t5"
		DB	0XFF

T6_TXT:		DB	"t6"
		DB	0XFF

T7_TXT:		DB	"t7"
		DB	0XFF

T8_TXT:		DB	"t8"
		DB	0XFF

MODE_TXT:	DB	"m="
		db	0XFF

MONTH_TABLE:	DB	0X01,0X00
		DATA	"jan."
		DB	0XFF

                DB	0X02,0X00
		DATA	"feb."
		DB	0XFF

		DB	0X03,0X00
		DATA	"mrt."
		;DATA	"mar"
		DB	0XFF

		DB	0X04,0X00
		DATA	"apr."
		DB	0XFF

		DB	0X05,0X80
		DATA	"mei "
		;DATA	"may"
		DB	0XFF

		DB	0X06,0X00
		DATA	"juni"
		;DATA	"june"
		DB	0XFF

		DB	0X07,0X00
		DATA	"juli"
		;DATA	"july"
		DB	0XFF

		DB	0X08,0X00
		DATA	"aug."
		DB	0XFF

		DB	0X09,0X00
		DATA	"sep."
		DB	0XFF

		DB	0X10,0X00
		DATA	"okt."
		;DATA	"oct."
		DB	0XFF

		DB	0X11,0X00
		DATA	"nov."
		DB	0XFF

		DB	0X12,0X00
		DATA	"dec."
		DB	0XFF
		
YEAR_TABLE:	DB	0X18,0X00
		DATA	"2018"
		DB	0XFF

		DB	0X19,0X00
		DATA	"2019"
		DB	0XFF

		DB	0X20,0X00
		DATA	"2020"
		DB	0XFF

		DB	0X21,0X00
		DATA	"2021"
		DB	0XFF

		DB	0X22,0X00
		DATA	"2022"
		DB	0XFF
		
		DB	0X23,0X00
		DATA	"2023"
		DB	0XFF

		DB	0X24,0X00
		DATA	"2024"
		DB	0XFF

		DB	0X25,0X00
		DATA	"2025"
		DB	0XFF

		DB	0X26,0X00
		DATA	"2026"
		DB	0XFF

		DB	0X27,0X00
		DATA	"2027"
		DB	0XFF
		
                DB	0X28,0X00
		DATA	"2028"
		DB	0XFF

		DB	0X29,0X00
		DATA	"2029"
		DB	0XFF

		DB	0X30,0X00
		DATA	"2030"
		DB	0XFF
		
T_LINE:		DB	"T 1 2 3 4 5 6 7 8 "
		DB	0XFF
		
DUMMY_TXT:	DB	"  "
		DB	0XFF

	ORG 0X1000
	
EE_INIT_TBL:	DB	0X03,0X84,0X03,0X85
		DB	0X03,0X85,0X03,0X86
                DB	0X03,0X86,0X03,0X87
		DB	0X03,0X87,0X03,0X88
		DB	0X03,0X88,0X03,0X89
		DB	0X03,0X89,0X03,0X8A
		DB	0X03,0X8A,0X03,0X8B
		DB	0X03,0X8B,0X03,0X8C

		DB	0X0A,0X0A,0X0A,0X0A
		DB	0X0A,0X0A,0X0A,0X0A
		
		DB	0X01,0X08

DISA_GGA:	DB	0X24,0X50,0X53,0X52,0X46,0X31,0X30,0X33
		DB	0X2C,0X30,0X30,0X2C,0X30,0X30
		DB	0X2C,0X30,0X30,0X2C,0X30,0X31
		DB	0X2A,0X32,0X34,0X0D,0X0A,0XFF

DISA_GLL:	DB	0X24,0X50,0X53,0X52,0X46,0X31,0X30,0X33
		DB	0X2C,0X30,0X31,0X2C,0X30,0X30
		DB	0X2C,0X30,0X30,0X2C,0X30,0X31
		DB	0X2A,0X32,0X35,0X0D,0X0A,0XFF
		
DISA_GSA:	DB	0X24,0X50,0X53,0X52,0X46,0X31,0X30,0X33
		DB	0X2C,0X30,0X32,0X2C,0X30,0X30
		DB	0X2C,0X30,0X30,0X2C,0X30,0X31
		DB	0X2A,0X32,0X36,0X0D,0X0A,0XFF
		
DISA_GSV:	DB	0X24,0X50,0X53,0X52,0X46,0X31,0X30,0X33
		DB	0X2C,0X30,0X33,0X2C,0X30,0X30
		DB	0X2C,0X30,0X30,0X2C,0X30,0X31
		DB	0X2A,0X32,0X37,0X0D,0X0A,0XFF
		
ENA_RMC:	DB	0X24,0X50,0X53,0X52,0X46,0X31,0X30,0X33
		DB	0X2C,0X30,0X34,0X2C,0X30,0X30
		DB	0X2C,0X30,0X31,0X2C,0X30,0X31
		DB	0X2A,0X32,0X31,0X0D,0X0A,0XFF
		
DISA_VTG:	DB	0X24,0X50,0X53,0X52,0X46,0X31,0X30,0X33
		DB	0X2C,0X30,0X35,0X2C,0X30,0X30
		DB	0X2C,0X30,0X30,0X2C,0X30,0X31
		DB	0X2A,0X32,0X31,0X0D,0X0A,0XFF

GPRMC_HEADER:	DB	0X47,0X50,0X52,0X4D,0X43,0X2C

H_POS_TBL:	DB	0X00,0X18,0X28,0X2B,0X40,0X50,0X53,0X68,0X7A,0XFF

MAIN_TABLE1:	DB	0X06,0X28,0X18,0X0D,0X17,0X01	;Regel 1, pagina 1
		DB	0X06,0X29,0X2B,0X0D,0X3B,0X01
		DB	0X06,0X2A,0X40,0X0D,0X17,0X01
		DB	0X06,0X2B,0X53,0X0D,0X3B,0X01
		DB	0X06,0X20,0X7A,0X0D,0X46,0X0A

		DB	0X06,0X2C,0X18,0X1A,0X17,0X01	;Regel 2, pagina 1
		DB	0X06,0X2D,0X2B,0X1A,0X3B,0X01
		DB	0X06,0X2E,0X40,0X1A,0X17,0X01
		DB	0X06,0X2F,0X53,0X1A,0X3B,0X01
		DB	0X06,0X21,0X7A,0X1A,0X46,0X0A

                DB	0X06,0X30,0X18,0X27,0X17,0X01	;Regel 3, pagina 1
		DB	0X06,0X31,0X2B,0X27,0X3B,0X01
		DB	0X06,0X32,0X40,0X27,0X17,0X01
		DB	0X06,0X33,0X53,0X27,0X3B,0X01
		DB	0X06,0X22,0X7A,0X27,0X46,0X0A

		DB	0X06,0X34,0X18,0X34,0X17,0X01	;Regel 4, pagina 1
		DB	0X06,0X35,0X2B,0X34,0X3B,0X01
		DB	0X06,0X36,0X40,0X34,0X17,0X01
		DB	0X06,0X37,0X53,0X34,0X3B,0X01
		DB	0X06,0X23,0X7A,0X34,0X46,0X0A

MAIN_TABLE2:	DB	0X06,0X38,0X18,0X0D,0X17,0X01	;Regel 1, pagina 2
		DB	0X06,0X39,0X2B,0X0D,0X3B,0X01
		DB	0X06,0X3A,0X40,0X0D,0X17,0X01
		DB	0X06,0X3B,0X53,0X0D,0X3B,0X01
		DB	0X06,0X24,0X7A,0X0D,0X46,0X0A

		DB	0X06,0X3C,0X18,0X1A,0X17,0X01	;Regel 2, pagina 2
		DB	0X06,0X3D,0X2B,0X1A,0X3B,0X01
		DB	0X06,0X3E,0X40,0X1A,0X17,0X01
		DB	0X06,0X3F,0X53,0X1A,0X3B,0X01
		DB	0X06,0X25,0X7A,0X1A,0X46,0X0A

                DB	0X06,0X40,0X18,0X27,0X17,0X01	;Regel 3, pagina 2
		DB	0X06,0X41,0X2B,0X27,0X3B,0X01
		DB	0X06,0X42,0X40,0X27,0X17,0X01
		DB	0X06,0X43,0X53,0X27,0X3B,0X01
		DB	0X06,0X26,0X7A,0X27,0X46,0X0A

		DB	0X06,0X44,0X18,0X34,0X17,0X01	;Regel 4, pagina 2
		DB	0X06,0X45,0X2B,0X34,0X3B,0X01
		DB	0X06,0X46,0X40,0X34,0X17,0X01
		DB	0X06,0X47,0X53,0X34,0X3B,0X01
		DB	0X06,0X27,0X7A,0X34,0X46,0X0A

	ORG 0X1200

CONV_TABLE:	DB	0X00,0X01,0X02,0X03,0X04,0X05,0X06,0X07;00H
		DB	0X08,0X09,0X00,0X00,0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00;10H
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00;20H
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00;30H
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00
		DB	0X00,0X0A,0X0B,0X0C,0X0D,0X0E,0X0F,0X00;40H

YEARS_TBL:	DB	0X18,0X00,0X19,0X01,0X20,0X02,0X21,0X03
		DB	0X22,0X04,0X23,0X05,0X24,0X06,0X25,0X07
		DB	0X26,0X08,0X27,0X09,0X27,0X0A,0X29,0X0B
		DB	0X30,0X0C,0XFF,0XFF

MONTH_TBL:	DB	0X01,0X00,0X02,0X01,0X03,0X02,0X04,0X03
		DB	0X05,0X04,0X06,0X05,0X07,0X06,0X08,0X07
		DB	0X09,0X08,0X10,0X09,0X11,0X0A,0X12,0X0B
		DB	0XFF,0XFF

	ORG 0X1400

TIME_HEX_TABLE:	DB	0X00,0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08,0X09,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
		DB      0X0A,0X0B,0X0C,0X0D,0X0E,0X0F,0X10,0X11,0X12,0X13,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
		DB	0X14,0X15,0X16,0X17,0X18,0X19,0X1A,0X1B,0X1C,0X1D,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
		DB	0X1E,0X1F,0X20,0X21,0X22,0X23,0X24,0X25,0X26,0X27,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
		DB	0X28,0X29,0X2A,0X2B,0X2C,0X2D,0X2E,0X2F,0X30,0X31,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
		DB	0X32,0X33,0X34,0X35,0X36,0X37,0X38,0X39,0X3A,0X3B,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
		DB	0X3C,0X3D,0X3E,0X3F,0X40,0X41,0X42,0X43,0X44,0X45,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF
                DB	0X46,0X47,0X48,0X49,0X4A,0X4B,0X4C,0X4D,0X4E,0X4F,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF

	ORG 0X1500

BCD_TABLE_DD:	DB	0X31,0X1E,0X30,0X1D,0X29,0X1C,0X28,0X1B,0X27,0X1A
		DB	0X26,0X19,0X25,0X18,0X24,0X17,0X23,0X16,0X22,0X15
		DB	0X21,0X14,0X20,0X13,0X19,0X12,0X18,0X11,0X17,0X10
		DB	0X16,0X0F,0X0E,0X0F,0X0D,0X0E,0X0C,0X0D,0X0B,0X0C
		DB	0X11,0X0A,0X10,0X09,0X09,0X08,0X08,0X07,0X07,0X06
		DB	0X06,0X05,0X05,0X04,0X04,0X03,0X03,0X02,0X02,0X01
		DB	0X01,0X00

BCD_TABLE_HR:	DB	0X00,0X00,0X01,0X01,0X02,0X02,0X03,0X03,0X04,0X04,0X05,0X05
		DB	0X06,0X06,0X07,0X07,0X08,0X08,0X09,0X09,0X10,0X0A,0X11,0X0B
		DB	0X12,0X0C,0X13,0X0D,0X14,0X0E,0X15,0X0F,0X16,0X10,0X17,0X11
		DB	0X18,0X12,0X19,0X13,0X20,0X14,0X21,0X15,0X22,0X16,0X23,0X17
		
NORM_YEAR:	DB	0X01,0X00,0X00,0X02,0X02,0XE8,0X03,0X05,0X88,0X00,0X00,0X00
		DB	0X04,0X08,0X70,0X05,0X0B,0X40,0X06,0X0E,0X28,0X00,0X00,0X00
		DB	0X07,0X10,0XF8,0X08,0X13,0XE0,0X09,0X16,0XC8,0X00,0X00,0X00
		DB	0X10,0X19,0X98,0X11,0X1C,0X80,0X12,0X1F,0X50,0X00,0X00,0X00

LEAP_YEAR:      DB	0X01,0X00,0X00,0X02,0X02,0XE8,0X03,0X05,0XA0,0X00,0X00,0X00
		DB	0X04,0X08,0X88,0X05,0X0B,0X58,0X06,0X0E,0X40,0X00,0X00,0X00
		DB	0X07,0X11,0X10,0X08,0X13,0XF8,0X09,0X16,0XE0,0X00,0X00,0X00
		DB	0X10,0X19,0XB0,0X11,0X1C,0X98,0X12,0X1F,0X68,0X00,0X00,0X00
		
	ORG 0X1600

NON_LEAP_CORR:	DB	0X01,0X32,0X02,0X29,0X03,0X32,0X04,0X31,0X05,0X32,0X06,0X31
                DB	0X07,0X32,0X08,0X32,0X09,0X31,0X10,0X32,0X11,0X31,0X12,0X32
                DB	0XFF,0XFF

LEAP_CORR:	DB	0X01,0X32,0X02,0X30,0X03,0X32,0X04,0X31,0X05,0X32,0X06,0X31
                DB	0X07,0X32,0X08,0X32,0X09,0X31,0X10,0X32,0X11,0X31,0X12,0X32
		DB	0XFF,0XFF

HEX_BCD_TABLE:	DB	0X00,0X01,0X02,0X03,0X04,0X05,0X06,0X07
		DB	0X08,0X09,0X10,0X11,0X12,0X13,0X14,0X15
		DB	0X16,0X17,0X18,0X19,0X20,0X21,0X22,0X23
		DB	0X24,0X25,0X26,0X27,0X28,0X29,0X30,0X31
		DB	0X32,0X33,0X34,0X35,0X36,0X37,0X38,0X39
		DB	0X40,0X41,0X42,0X43,0X44,0X45,0X46,0X47
		DB	0X48,0X49,0X50,0X51,0X52,0X53,0X54,0X55
		DB	0X56,0X57,0X58,0X59,0XFF,0XFF

	ORG 0X1700

LIMITS_TABLE:	DB	0X18,0X07,0XC9,0X1C,0X21,0X00
		DB	0X19,0X08,0X59,0X1C,0X09,0X00
		DB	0X20,0X08,0X41,0X1B,0XF1,0X00
		DB	0X21,0X08,0X11,0X1C,0X69,0X00
		DB	0X22,0X07,0XF9,0X1C,0X51,0X00
		DB	0X23,0X07,0XE1,0X1C,0X39,0X00
		DB	0X24,0X08,0X71,0X1C,0X21,0X00
		DB	0X25,0X08,0X41,0X1B,0XF1,0X00
		DB	0X26,0X08,0X29,0X1B,0XD9,0X00
		DB	0X27,0X08,0X11,0X1C,0X69,0X00
		DB	0X28,0X07,0XE1,0X1C,0X39,0X00
		DB	0X29,0X07,0XC9,0X1C,0X21,0X00
		DB	0X30,0X08,0X59,0X1C,0X09,0X00

SEC_TABLE:	DB	0X00,0X00,0X00,0X00
		DB	0X00,0X0E,0X10,0X00
		DB	0X00,0X1C,0X20,0X00
		DB	0X00,0X2A,0X30,0X00
		
		DB	0X00,0X38,0X40,0X00
		DB	0X00,0X46,0X50,0X00
		DB	0X00,0X54,0X60,0X00
		DB	0X00,0X62,0X70,0X00
		
		DB	0X00,0X70,0X80,0X00
		DB	0X00,0X7E,0X90,0X00
		DB	0X00,0X8C,0XA0,0X00
		DB	0X00,0X9A,0XB0,0X00
		
		DB	0X00,0XA8,0XC0,0X00
		DB	0X00,0XB6,0XD0,0X00
		DB	0X00,0XC4,0XE0,0X00
		DB	0X00,0XD2,0XF0,0X00

		DB	0X00,0XE1,0X00,0X00
		DB	0X00,0XEF,0X10,0X00
		DB	0X00,0XFD,0X20,0X00
		DB	0X01,0X0B,0X30,0X00

		DB	0X01,0X19,0X40,0X00
		DB	0X01,0X27,0X50,0X00
		DB	0X01,0X35,0X60,0X00
		DB	0X01,0X43,0X70,0X00

	ORG 0X1800
	
TYPE_TABLE:	DB	0X00,0X01,0X08,0X80,0X80,0X80,0X80,0X00 ;1     ;OFFSET, HEIGHT, 5 x type, sequence
                DB	0X00,0X02,0X09,0X80,0X80,0X80,0X80,0X01 ;2
                DB	0X00,0X03,0X0A,0X80,0X80,0X80,0X80,0X02 ;3
                DB	0X00,0X04,0X0B,0X80,0X80,0X80,0X80,0X03 ;4
                DB	0X00,0X05,0X0C,0X80,0X80,0X80,0X80,0X04 ;5
                DB	0X00,0X06,0X0D,0X80,0X80,0X80,0X80,0X05 ;6
                DB	0X00,0X07,0X0E,0X80,0X80,0X80,0X80,0X06 ;7
                DB	0X00,0X08,0X00,0X80,0X80,0X80,0X80,0X07 ;8

                DB	0X00,0X09,0X00,0X08,0X80,0X80,0X80,0X08 ;8+1
                DB	0X00,0X0A,0X00,0X09,0X80,0X80,0X80,0X09 ;8+2
                DB	0X00,0X0B,0X00,0X0A,0X80,0X80,0X80,0X0A ;8+3
                DB	0X00,0X0C,0X00,0X0B,0X80,0X80,0X80,0X0B ;8+4
                DB	0X00,0X0D,0X00,0X0C,0X80,0X80,0X80,0X0C ;8+5
                DB	0X00,0X0E,0X00,0X0D,0X80,0X80,0X80,0X0D ;8+6
                DB	0X00,0X0F,0X00,0X0E,0X80,0X80,0X80,0X0E ;8+7
                DB	0X00,0X10,0X00,0X00,0X80,0X80,0X80,0X0F ;8+8

                DB	0X00,0X11,0X00,0X00,0X08,0X80,0X80,0X10 ;8+8+1
                DB	0X00,0X12,0X00,0X00,0X09,0X80,0X80,0X11 ;8+8+2
                DB	0X00,0X13,0X00,0X00,0X0A,0X80,0X80,0X12 ;8+8+3
                DB	0X00,0X14,0X00,0X00,0X0B,0X80,0X80,0X13 ;8+8+4
                DB	0X00,0X15,0X00,0X00,0X0C,0X80,0X80,0X14 ;8+8+5
                DB	0X00,0X16,0X00,0X00,0X0D,0X80,0X80,0X15 ;8+8+6
                DB	0X00,0X17,0X00,0X00,0X0E,0X80,0X80,0X16 ;8+8+7
                DB	0X00,0X18,0X00,0X00,0X00,0X80,0X80,0X17 ;8+8+8

                DB	0X00,0X19,0X00,0X00,0X00,0X08,0X80,0X18 ;8+8+8+1
                DB	0X00,0X1A,0X00,0X00,0X00,0X09,0X80,0X19 ;8+8+8+2
                DB	0X00,0X1B,0X00,0X00,0X00,0X0A,0X80,0X1A ;8+8+8+3
                DB	0X00,0X1C,0X00,0X00,0X00,0X0B,0X80,0X1B ;8+8+8+4
                DB	0X00,0X1D,0X00,0X00,0X00,0X0C,0X80,0X1C ;8+8+8+5
                DB	0X00,0X1E,0X00,0X00,0X00,0X0D,0X80,0X1D ;8+8+8+6
                DB	0X00,0X1F,0X00,0X00,0X00,0X0E,0X80,0X1E ;8+8+8+7
                DB	0X00,0X20,0X00,0X00,0X00,0X00,0X80,0X1F ;8+8+8+8

                DB	0X01,0X01,0X10,0X80,0X80,0X80,0X80,0X20 ;(L+1)
                DB	0X01,0X02,0X20,0X80,0X80,0X80,0X80,0X21 ;(L+2)
                DB	0X01,0X03,0X30,0X80,0X80,0X80,0X80,0X22 ;(L+3)
                DB	0X01,0X04,0X40,0X80,0X80,0X80,0X80,0X23 ;(L+4)
                DB	0X01,0X05,0X50,0X80,0X80,0X80,0X80,0X24 ;(L+5)
                DB	0X01,0X06,0X60,0X80,0X80,0X80,0X80,0X25 ;(L+6)
                DB	0X01,0X07,0X01,0X80,0X80,0X80,0X80,0X26 ;(L+7)
                DB	0X01,0X08,0X01,0X08,0X80,0X80,0X80,0X27 ;(L+7)+1

                DB	0X01,0X09,0X01,0X09,0X80,0X80,0X80,0X28 ;(L+7)+2
                DB	0X01,0X0A,0X01,0X0A,0X80,0X80,0X80,0X29 ;(L+7)+3
                DB	0X01,0X0B,0X01,0X0B,0X80,0X80,0X80,0X2A ;(L+7)+4
                DB	0X01,0X0C,0X01,0X0C,0X80,0X80,0X80,0X2B ;(L+7)+5
                DB	0X01,0X0D,0X01,0X0D,0X80,0X80,0X80,0X2C ;(L+7)+6
                DB	0X01,0X0E,0X01,0X0E,0X80,0X80,0X80,0X2D ;(L+7)+7
                DB	0X01,0X0F,0X01,0X00,0X80,0X80,0X80,0X2E ;(L+7)+8
                DB	0X01,0X10,0X01,0X00,0X08,0X80,0X80,0X2F ;(L+7)+8+1

                DB	0X01,0X11,0X01,0X00,0X09,0X80,0X80,0X30 ;(L+7)+8+2
                DB	0X01,0X12,0X01,0X00,0X0A,0X80,0X80,0X31 ;(L+7)+8+3
                DB	0X01,0X13,0X01,0X00,0X0B,0X80,0X80,0X32 ;(L+7)+8+4
                DB	0X01,0X14,0X01,0X00,0X0C,0X80,0X80,0X33 ;(L+7)+8+5
                DB	0X01,0X15,0X01,0X00,0X0D,0X80,0X80,0X34 ;(L+7)+8+6
                DB	0X01,0X16,0X01,0X00,0X0E,0X80,0X80,0X35 ;(L+7)+8+7
                DB	0X01,0X17,0X01,0X00,0X00,0X80,0X80,0X36 ;(L+7)+8+8
                DB	0X01,0X18,0X01,0X00,0X00,0X08,0X80,0X37 ;(L+7)+8+8+1

                DB	0X01,0X19,0X01,0X00,0X00,0X09,0X80,0X38 ;(L+7)+8+8+2
                DB	0X01,0X1A,0X01,0X00,0X00,0X0A,0X80,0X39 ;(L+7)+8+8+3
                DB	0X01,0X1B,0X01,0X00,0X00,0X0B,0X80,0X3A ;(L+7)+8+8+4
                DB	0X01,0X1C,0X01,0X00,0X00,0X0C,0X80,0X3B ;(L+7)+8+8+5
                DB	0X01,0X1D,0X01,0X00,0X00,0X0D,0X80,0X3C ;(L+7)+8+8+6
                DB	0X01,0X1E,0X01,0X00,0X00,0X0E,0X80,0X3D ;(L+7)+8+8+7
                DB	0X01,0X1F,0X01,0X00,0X00,0X00,0X80,0X3E ;(L+7)+8+8+8
                DB	0X01,0X20,0X01,0X00,0X00,0X00,0X08,0X3F ;(L+7)+8+8+8+1

                DB	0X02,0X01,0X11,0X80,0X80,0X80,0X80,0X40 ;(2L+1)
                DB	0X02,0X02,0X21,0X80,0X80,0X80,0X80,0X41 ;(2L+2)
                DB	0X02,0X03,0X31,0X80,0X80,0X80,0X80,0X42 ;(2L+3)
                DB	0X02,0X04,0X41,0X80,0X80,0X80,0X80,0X43 ;(2L+4)
                DB	0X02,0X05,0X51,0X80,0X80,0X80,0X80,0X44 ;(2L+5)
                DB	0X02,0X06,0X02,0X80,0X80,0X80,0X80,0X45 ;(2L+6)
                DB	0X02,0X07,0X02,0X08,0X80,0X80,0X80,0X46 ;(2L+6)+1
                DB	0X02,0X08,0X02,0X09,0X80,0X80,0X80,0X47 ;(2L+6)+2

                DB	0X02,0X09,0X02,0X0A,0X80,0X80,0X80,0X48 ;(2L+6)+3
                DB	0X02,0X0A,0X02,0X0B,0X80,0X80,0X80,0X49 ;(2L+6)+4
                DB	0X02,0X0B,0X02,0X0C,0X80,0X80,0X80,0X4A ;(2L+6)+5
                DB	0X02,0X0C,0X02,0X0D,0X80,0X80,0X80,0X4B ;(2L+6)+6
                DB	0X02,0X0D,0X02,0X0E,0X80,0X80,0X80,0X4C ;(2L+6)+7
                DB	0X02,0X0E,0X02,0X00,0X80,0X80,0X80,0X4D ;(2L+6)+8
                DB	0X02,0X0F,0X02,0X00,0X08,0X80,0X80,0X4E ;(2L+6)+8+1
                DB	0X02,0X10,0X02,0X00,0X09,0X80,0X80,0X4F ;(2L+6)+8+2

                DB	0X02,0X11,0X02,0X00,0X0A,0X80,0X80,0X50 ;(2L+6)+8+3
                DB	0X02,0X12,0X02,0X00,0X0B,0X80,0X80,0X51 ;(2L+6)+8+4
                DB	0X02,0X13,0X02,0X00,0X0C,0X80,0X80,0X52 ;(2L+6)+8+5
                DB	0X02,0X14,0X02,0X00,0X0D,0X80,0X80,0X53 ;(2L+6)+8+6
                DB	0X02,0X15,0X02,0X00,0X0E,0X80,0X80,0X54 ;(2L+6)+8+7
                DB	0X02,0X16,0X02,0X00,0X00,0X80,0X80,0X55 ;(2L+6)+8+8
                DB	0X02,0X17,0X02,0X00,0X00,0X08,0X80,0X56 ;(2L+6)+8+8+1
                DB	0X02,0X18,0X02,0X00,0X00,0X09,0X80,0X57 ;(2L+6)+8+8+2

                DB	0X02,0X19,0X02,0X00,0X00,0X0A,0X80,0X58 ;(2L+6)+8+8+3
                DB	0X02,0X1A,0X02,0X00,0X00,0X0B,0X80,0X59 ;(2L+6)+8+8+4
                DB	0X02,0X1B,0X02,0X00,0X00,0X0C,0X80,0X5A ;(2L+6)+8+8+5
                DB	0X02,0X1C,0X02,0X00,0X00,0X0D,0X80,0X5B ;(2L+6)+8+8+6
                DB	0X02,0X1D,0X02,0X00,0X00,0X0E,0X80,0X5C ;(2L+6)+8+8+7
                DB	0X02,0X1E,0X02,0X00,0X00,0X00,0X80,0X5D ;(2L+6)+8+8+8
                DB	0X02,0X1F,0X02,0X00,0X00,0X00,0X08,0X5E ;(2L+6)+8+8+8+1
                DB	0X02,0X20,0X02,0X00,0X00,0X00,0X09,0X5F ;(2L+6)+8+8+8+2

                DB	0X03,0X01,0X12,0X80,0X80,0X80,0X80,0X60 ;(3L+1)
                DB	0X03,0X02,0X22,0X80,0X80,0X80,0X80,0X61 ;(3L+2)
                DB	0X03,0X03,0X32,0X80,0X80,0X80,0X80,0X62 ;(3L+3)
                DB	0X03,0X04,0X42,0X80,0X80,0X80,0X80,0X63 ;(3L+4)
                DB	0X03,0X05,0X03,0X80,0X80,0X80,0X80,0X64 ;(3L+5)
                DB	0X03,0X06,0X03,0X08,0X80,0X80,0X80,0X65 ;(3L+5)+1
                DB	0X03,0X07,0X03,0X09,0X80,0X80,0X80,0X66 ;(3L+5)+2
		DB	0X03,0X08,0X03,0X0A,0X80,0X80,0X80,0X67 ;(3L+5)+3

                DB	0X03,0X09,0X03,0X0B,0X80,0X80,0X80,0X68 ;(3L+5)+4
                DB	0X03,0X0A,0X03,0X0C,0X80,0X80,0X80,0X69 ;(3L+5)+5
		DB	0X03,0X0B,0X03,0X0D,0X80,0X80,0X80,0X6A ;(3L+5)+6
                DB	0X03,0X0C,0X03,0X0E,0X80,0X80,0X80,0X6B ;(3L+5)+7
                DB	0X03,0X0D,0X03,0X00,0X80,0X80,0X80,0X6C ;(3L+5)+8
                DB	0X03,0X0E,0X03,0X00,0X08,0X80,0X80,0X6D ;(3L+5)+8+1
                DB	0X03,0X0F,0X03,0X00,0X09,0X80,0X80,0X6E ;(3L+5)+8+2
                DB	0X03,0X10,0X03,0X00,0X0A,0X80,0X80,0X6F ;(3L+5)+8+3

                DB	0X03,0X11,0X03,0X00,0X0B,0X80,0X80,0X70 ;(3L+5)+8+4
                DB	0X03,0X12,0X03,0X00,0X0C,0X80,0X80,0X71 ;(3L+5)+8+5
                DB	0X03,0X13,0X03,0X00,0X0D,0X80,0X80,0X72 ;(3L+5)+8+6
                DB	0X03,0X14,0X03,0X00,0X0E,0X80,0X80,0X73 ;(3L+5)+8+7
                DB	0X03,0X15,0X03,0X00,0X00,0X80,0X80,0X74 ;(3L+5)+8+8
                DB	0X03,0X16,0X03,0X00,0X00,0X08,0X80,0X75 ;(3L+5)+8+8+1
                DB	0X03,0X17,0X03,0X00,0X00,0X09,0X80,0X76 ;(3L+5)+8+8+2
                DB	0X03,0X18,0X03,0X00,0X00,0X0A,0X80,0X77 ;(3L+5)+8+8+3

                DB	0X03,0X19,0X03,0X00,0X00,0X0B,0X80,0X78 ;(3L+5)+8+8+4
                DB	0X03,0X1A,0X03,0X00,0X00,0X0C,0X80,0X79 ;(3L+5)+8+8+5
                DB	0X03,0X1B,0X03,0X00,0X00,0X0D,0X80,0X7A ;(3L+5)+8+8+6
                DB	0X03,0X1C,0X03,0X00,0X00,0X0E,0X80,0X7B ;(3L+5)+8+8+7
                DB	0X03,0X1D,0X03,0X00,0X00,0X00,0X80,0X7C ;(3L+5)+8+8+8
                DB	0X03,0X1E,0X03,0X00,0X00,0X00,0X08,0X7D ;(3L+5)+8+8+8+1
                DB	0X03,0X1F,0X03,0X00,0X00,0X00,0X09,0X7E ;(3L+5)+8+8+8+2
                DB	0X03,0X20,0X03,0X00,0X00,0X00,0X0A,0X7F ;(3L+5)+8+8+8+3

                DB	0X04,0X01,0X13,0X80,0X80,0X80,0X80,0X80 ;(4L+1)
                DB	0X04,0X02,0X23,0X80,0X80,0X80,0X80,0X81 ;(4L+2)
                DB	0X04,0X03,0X33,0X80,0X80,0X80,0X80,0X82 ;(4L+3)
                DB	0X04,0X04,0X04,0X80,0X80,0X80,0X80,0X83 ;(4L+4)
                DB	0X04,0X05,0X04,0X08,0X80,0X80,0X80,0X84 ;(4L+4)+1
                DB	0X04,0X06,0X04,0X09,0X80,0X80,0X80,0X85 ;(4L+4)+2
		DB	0X04,0X07,0X04,0X0A,0X80,0X80,0X80,0X86 ;(4L+4)+3
		DB	0X04,0X08,0X04,0X0B,0X80,0X80,0X80,0X87 ;(4L+4)+4

                DB	0X04,0X09,0X04,0X0C,0X80,0X80,0X80,0X88 ;(4L+4)+5
                DB	0X04,0X0A,0X04,0X0D,0X80,0X80,0X80,0X89 ;(4L+4)+6
		DB	0X04,0X0B,0X04,0X0E,0X80,0X80,0X80,0X8A ;(4L+4)+7
                DB	0X04,0X0C,0X04,0X00,0X80,0X80,0X80,0X8B ;(4L+4)+8
                DB	0X04,0X0D,0X04,0X00,0X08,0X80,0X80,0X8C ;(4L+4)+8+1
                DB	0X04,0X0E,0X04,0X00,0X09,0X80,0X80,0X8D ;(4L+4)+8+2
                DB	0X04,0X0F,0X04,0X00,0X0A,0X80,0X80,0X8E ;(4L+4)+8+3
                DB	0X04,0X10,0X04,0X00,0X0B,0X80,0X80,0X8F ;(4L+4)+8+4

                DB	0X04,0X11,0X04,0X00,0X0C,0X80,0X80,0X90 ;(4L+4)+8+5
                DB	0X04,0X12,0X04,0X00,0X0D,0X80,0X80,0X91 ;(4L+4)+8+6
                DB	0X04,0X13,0X04,0X00,0X0E,0X80,0X80,0X92 ;(4L+4)+8+7
                DB	0X04,0X14,0X04,0X00,0X00,0X80,0X80,0X93 ;(4L+4)+8+8
                DB	0X04,0X15,0X04,0X00,0X00,0X08,0X80,0X94 ;(4L+4)+8+8+1
                DB	0X04,0X16,0X04,0X00,0X00,0X09,0X80,0X95 ;(4L+4)+8+8+2
                DB	0X04,0X17,0X04,0X00,0X00,0X0A,0X80,0X96 ;(4L+4)+8+8+3
                DB	0X04,0X18,0X04,0X00,0X00,0X0B,0X80,0X97 ;(4L+4)+8+8+4

                DB	0X04,0X19,0X04,0X00,0X00,0X0C,0X80,0X98 ;(4L+4)+8+8+5
                DB	0X04,0X1A,0X04,0X00,0X00,0X0D,0X80,0X99 ;(4L+4)+8+8+6
                DB	0X04,0X1B,0X04,0X00,0X00,0X0E,0X80,0X9A ;(4L+4)+8+8+7
                DB	0X04,0X1C,0X04,0X00,0X00,0X00,0X80,0X9B ;(4L+4)+8+8+8
                DB	0X04,0X1D,0X04,0X00,0X00,0X00,0X08,0X9C ;(4L+4)+8+8+8+1
                DB	0X04,0X1E,0X04,0X00,0X00,0X00,0X09,0X9D ;(4L+4)+8+8+8+2
                DB	0X04,0X1F,0X04,0X00,0X00,0X00,0X0A,0X9E ;(4L+4)+8+8+8+3
                DB	0X04,0X20,0X04,0X00,0X00,0X00,0X0B,0X9F ;(4L+4)+8+8+8+4

                DB	0X05,0X01,0X14,0X80,0X80,0X80,0X80,0XA0 ;(5L+1)
                DB	0X05,0X02,0X24,0X80,0X80,0X80,0X80,0XA1 ;(5L+2)
                DB	0X05,0X03,0X05,0X80,0X80,0X80,0X80,0XA2 ;(5L+3)
                DB	0X05,0X04,0X05,0X08,0X80,0X80,0X80,0XA3 ;(5L+3)+1
                DB	0X05,0X05,0X05,0X09,0X80,0X80,0X80,0XA4 ;(5L+3)+2
                DB	0X05,0X06,0X05,0X0A,0X80,0X80,0X80,0XA5 ;(5L+3)+3
		DB	0X05,0X07,0X05,0X0B,0X80,0X80,0X80,0XA6 ;(5L+3)+4
		DB	0X05,0X08,0X05,0X0C,0X80,0X80,0X80,0XA7 ;(5L+3)+5

                DB	0X05,0X09,0X05,0X0D,0X80,0X80,0X80,0XA8 ;(5L+3)+6
                DB	0X05,0X0A,0X05,0X0E,0X80,0X80,0X80,0XA9 ;(5L+3)+7
		DB	0X05,0X0B,0X05,0X00,0X80,0X80,0X80,0XAA ;(5L+3)+8
                DB	0X05,0X0C,0X05,0X00,0X08,0X80,0X80,0XAB ;(5L+3)+8+1
                DB	0X05,0X0D,0X05,0X00,0X09,0X80,0X80,0XAC ;(5L+3)+8+2
                DB	0X05,0X0E,0X05,0X00,0X0A,0X80,0X80,0XAD ;(5L+3)+8+3
                DB	0X05,0X0F,0X05,0X00,0X0B,0X80,0X80,0XAE ;(5L+3)+8+4
                DB	0X05,0X10,0X05,0X00,0X0C,0X80,0X80,0XAF ;(5L+3)+8+5

                DB	0X05,0X11,0X05,0X00,0X0D,0X80,0X80,0XB0 ;(5L+3)+8+6
                DB	0X05,0X12,0X05,0X00,0X0E,0X80,0X80,0XB1 ;(5L+3)+8+7
                DB	0X05,0X13,0X05,0X00,0X00,0X80,0X80,0XB2 ;(5L+3)+8+8
                DB	0X05,0X14,0X05,0X00,0X00,0X08,0X80,0XB3 ;(5L+3)+8+8+1
                DB	0X05,0X15,0X05,0X00,0X00,0X09,0X80,0XB4 ;(5L+3)+8+8+2
                DB	0X05,0X16,0X05,0X00,0X00,0X0A,0X80,0XB5 ;(5L+3)+8+8+3
                DB	0X05,0X17,0X05,0X00,0X00,0X0B,0X80,0XB6 ;(5L+3)+8+8+4
                DB	0X05,0X18,0X05,0X00,0X00,0X0C,0X80,0XB7 ;(5L+3)+8+8+5

                DB	0X05,0X19,0X05,0X00,0X00,0X0D,0X80,0XB8 ;(5L+3)+8+8+6
                DB	0X05,0X1A,0X05,0X00,0X00,0X0E,0X80,0XB9 ;(5L+3)+8+8+7
                DB	0X05,0X1B,0X05,0X00,0X00,0X00,0X80,0XBA ;(5L+3)+8+8+8
                DB	0X05,0X1C,0X05,0X00,0X00,0X00,0X08,0XBB ;(5L+3)+8+8+8+1
                DB	0X05,0X1D,0X05,0X00,0X00,0X00,0X09,0XBC ;(5L+3)+8+8+8+2
                DB	0X05,0X1E,0X05,0X00,0X00,0X00,0X0A,0XBD ;(5L+3)+8+8+8+3
                DB	0X05,0X1F,0X05,0X00,0X00,0X00,0X0B,0XBE ;(5L+3)+8+8+8+4
                DB	0X05,0X20,0X05,0X00,0X00,0X00,0X0C,0XBF ;(5L+3)+8+8+8+5

                DB	0X06,0X01,0X15,0X80,0X80,0X80,0X80,0XC0 ;(6L+1)
                DB	0X06,0X02,0X06,0X80,0X80,0X80,0X80,0XC1 ;(6L+2)
                DB	0X06,0X03,0X06,0X08,0X80,0X80,0X80,0XC2 ;(6L+2)+1
                DB	0X06,0X04,0X06,0X09,0X80,0X80,0X80,0XC3 ;(6L+2)+2
                DB	0X06,0X05,0X06,0X0A,0X80,0X80,0X80,0XC4 ;(6L+2)+3
                DB	0X06,0X06,0X06,0X0B,0X80,0X80,0X80,0XC5 ;(6L+2)+4
		DB	0X06,0X07,0X06,0X0C,0X80,0X80,0X80,0XC6 ;(6L+2)+5
		DB	0X06,0X08,0X06,0X0D,0X80,0X80,0X80,0XC7 ;(6L+2)+6

                DB	0X06,0X09,0X06,0X0E,0X80,0X80,0X80,0XC8 ;(6L+2)+7
                DB	0X06,0X0A,0X06,0X00,0X80,0X80,0X80,0XC9 ;(6L+2)+8
		DB	0X06,0X0B,0X06,0X00,0X08,0X80,0X80,0XCA ;(6L+2)+8+1
                DB	0X06,0X0C,0X06,0X00,0X09,0X80,0X80,0XCB ;(6L+2)+8+2
                DB	0X06,0X0D,0X06,0X00,0X0A,0X80,0X80,0XCC ;(6L+2)+8+3
                DB	0X06,0X0E,0X06,0X00,0X0B,0X80,0X80,0XCD ;(6L+2)+8+4
                DB	0X06,0X0F,0X06,0X00,0X0C,0X80,0X80,0XCE ;(6L+2)+8+5
                DB	0X06,0X10,0X06,0X00,0X0D,0X80,0X80,0XCF ;(6L+2)+8+6

                DB	0X06,0X11,0X06,0X00,0X0E,0X80,0X80,0XD0 ;(6L+2)+8+7
                DB	0X06,0X12,0X06,0X00,0X00,0X80,0X80,0XD1 ;(6L+2)+8+8
                DB	0X06,0X13,0X06,0X00,0X00,0X08,0X80,0XD2 ;(6L+2)+8+8+1
                DB	0X06,0X14,0X06,0X00,0X00,0X09,0X80,0XD3 ;(6L+2)+8+8+2
                DB	0X06,0X15,0X06,0X00,0X00,0X0A,0X80,0XD4 ;(6L+2)+8+8+3
                DB	0X06,0X16,0X06,0X00,0X00,0X0B,0X80,0XD5 ;(6L+2)+8+8+4
                DB	0X06,0X17,0X06,0X00,0X00,0X0C,0X80,0XD6 ;(6L+2)+8+8+5
                DB	0X06,0X18,0X06,0X00,0X00,0X0D,0X80,0XD7 ;(6L+2)+8+8+6

                DB	0X06,0X19,0X06,0X00,0X00,0X0E,0X80,0XD8 ;(6L+2)+8+8+7
                DB	0X06,0X1A,0X06,0X00,0X00,0X00,0X80,0XD9 ;(6L+2)+8+8+8
                DB	0X06,0X1B,0X06,0X00,0X00,0X00,0X08,0XDA ;(6L+2)+8+8+8+1
                DB	0X06,0X1C,0X06,0X00,0X00,0X00,0X09,0XDB ;(6L+2)+8+8+8+2
                DB	0X06,0X1D,0X06,0X00,0X00,0X00,0X0A,0XDC ;(6L+2)+8+8+8+3
                DB	0X06,0X1E,0X06,0X00,0X00,0X00,0X0B,0XDD ;(6L+2)+8+8+8+4
                DB	0X06,0X1F,0X06,0X00,0X00,0X00,0X0C,0XDE ;(6L+2)+8+8+8+5
                DB	0X06,0X20,0X06,0X00,0X00,0X00,0X0D,0XDF ;(6L+2)+8+8+8+6

                DB	0X07,0X01,0X07,0X80,0X80,0X80,0X80,0XE0 ;(7L+1)
                DB	0X07,0X02,0X07,0X08,0X80,0X80,0X80,0XE1 ;(7L+1)+1
                DB	0X07,0X03,0X07,0X09,0X80,0X80,0X80,0XE2 ;(7L+1)+2
                DB	0X07,0X04,0X07,0X0A,0X80,0X80,0X80,0XE3 ;(7L+1)+3
                DB	0X07,0X05,0X07,0X0B,0X80,0X80,0X80,0XE4 ;(7L+1)+4
                DB	0X07,0X06,0X07,0X0C,0X80,0X80,0X80,0XE5 ;(7L+1)+5
		DB	0X07,0X07,0X07,0X0D,0X80,0X80,0X80,0XE6 ;(7L+1)+6
		DB	0X07,0X08,0X07,0X0E,0X80,0X80,0X80,0XE7 ;(7L+1)+7

                DB	0X07,0X09,0X07,0X00,0X80,0X80,0X80,0XE8 ;(7L+1)+8
                DB	0X07,0X0A,0X07,0X00,0X08,0X80,0X80,0XE9 ;(7L+1)+8+1
		DB	0X07,0X0B,0X07,0X00,0X09,0X80,0X80,0XEA ;(7L+1)+8+2
                DB	0X07,0X0C,0X07,0X00,0X0A,0X80,0X80,0XEB ;(7L+1)+8+3
                DB	0X07,0X0D,0X07,0X00,0X0B,0X80,0X80,0XEC ;(7L+1)+8+4
                DB	0X07,0X0E,0X07,0X00,0X0C,0X80,0X80,0XED ;(7L+1)+8+5
                DB	0X07,0X0F,0X07,0X00,0X0D,0X80,0X80,0XEE ;(7L+1)+8+6
                DB	0X07,0X10,0X07,0X00,0X0E,0X80,0X80,0XEF ;(7L+1)+8+7

                DB	0X07,0X11,0X07,0X00,0X00,0X80,0X80,0XF0 ;(7L+1)+8+8
                DB	0X07,0X12,0X07,0X00,0X00,0X08,0X80,0XF1 ;(7L+1)+8+8+1
                DB	0X07,0X13,0X07,0X00,0X00,0X09,0X80,0XF2 ;(7L+1)+8+8+2
                DB	0X07,0X14,0X07,0X00,0X00,0X0A,0X80,0XF3 ;(7L+1)+8+8+3
                DB	0X07,0X15,0X07,0X00,0X00,0X0B,0X80,0XF4 ;(7L+1)+8+8+4
                DB	0X07,0X16,0X07,0X00,0X00,0X0C,0X80,0XF5 ;(7L+1)+8+8+5
                DB	0X07,0X17,0X07,0X00,0X00,0X0D,0X80,0XF6 ;(7L+1)+8+8+6
                DB	0X07,0X18,0X07,0X00,0X00,0X0E,0X80,0XF7 ;(7L+1)+8+8+7

                DB	0X07,0X19,0X07,0X00,0X00,0X00,0X80,0XF8 ;(7L+1)+8+8+8
                DB	0X07,0X1A,0X07,0X00,0X00,0X00,0X08,0XF9 ;(7L+1)+8+8+8+1
                DB	0X07,0X1B,0X07,0X00,0X00,0X00,0X09,0XFA ;(7L+1)+8+8+8+2
                DB	0X07,0X1C,0X07,0X00,0X00,0X00,0X0A,0XFB ;(7L+1)+8+8+8+3
                DB	0X07,0X1D,0X07,0X00,0X00,0X00,0X0B,0XFC ;(7L+1)+8+8+8+4
                DB	0X07,0X1E,0X07,0X00,0X00,0X00,0X0C,0XFD ;(7L+1)+8+8+8+5
                DB	0X07,0X1F,0X07,0X00,0X00,0X00,0X0D,0XFE ;(7L+1)+8+8+8+6
                DB	0X07,0X20,0X07,0X00,0X00,0X00,0X0E,0XFF ;(7L+1)+8+8+8+7
                
        ORG 0X2100

COL_TABLE:	DB	0X10,0X00
		DB	0X10,0X01
                DB	0X10,0X02
		DB	0X10,0X03
		DB	0X10,0X04
		DB	0X10,0X05
		DB	0X10,0X06
		DB	0X10,0X07

		DB	0X10,0X08
		DB	0X10,0X09
                DB	0X10,0X0A
		DB	0X10,0X0B
		DB	0X10,0X0C
		DB	0X10,0X0D
		DB	0X10,0X0E
		DB	0X10,0X0F

		DB	0X11,0X00
		DB	0X11,0X01
                DB	0X11,0X02
		DB	0X11,0X03
		DB	0X11,0X04
		DB	0X11,0X05
		DB	0X11,0X06
		DB	0X11,0X07

		DB	0X11,0X08
		DB	0X11,0X09
                DB	0X11,0X0A
		DB	0X11,0X0B
		DB	0X11,0X0C
		DB	0X11,0X0D
		DB	0X11,0X0E
		DB	0X11,0X0F

		DB	0X12,0X00
		DB	0X12,0X01
                DB	0X12,0X02
		DB	0X12,0X03
		DB	0X12,0X04
		DB	0X12,0X05
		DB	0X12,0X06
		DB	0X12,0X07

		DB	0X12,0X08
		DB	0X12,0X09
                DB	0X12,0X0A
		DB	0X12,0X0B
		DB	0X12,0X0C
		DB	0X12,0X0D
		DB	0X12,0X0E
		DB	0X12,0X0F

		DB	0X13,0X00
		DB	0X13,0X01
                DB	0X13,0X02
		DB	0X13,0X03
		DB	0X13,0X04
		DB	0X13,0X05
		DB	0X13,0X06
		DB	0X13,0X07

		DB	0X13,0X08
		DB	0X13,0X09
                DB	0X13,0X0A
		DB	0X13,0X0B
		DB	0X13,0X0C
		DB	0X13,0X0D
		DB	0X13,0X0E
		DB	0X13,0X0F

		DB	0X14,0X00
		DB	0X14,0X01
                DB	0X14,0X02
		DB	0X14,0X03
		DB	0X14,0X04
		DB	0X14,0X05
		DB	0X14,0X06
		DB	0X14,0X07

		DB	0X14,0X08
		DB	0X14,0X09
                DB	0X14,0X0A
		DB	0X14,0X0B
		DB	0X14,0X0C
		DB	0X14,0X0D
		DB	0X14,0X0E
		DB	0X14,0X0F

		DB	0X15,0X00
		DB	0X15,0X01
                DB	0X15,0X02
		DB	0X15,0X03
		DB	0X15,0X04
		DB	0X15,0X05
		DB	0X15,0X06
		DB	0X15,0X07

		DB	0X15,0X08
		DB	0X15,0X09
                DB	0X15,0X0A
		DB	0X15,0X0B
		DB	0X15,0X0C
		DB	0X15,0X0D
		DB	0X15,0X0E
		DB	0X15,0X0F

		DB	0X16,0X00
		DB	0X16,0X01
                DB	0X16,0X02
		DB	0X16,0X03
		DB	0X16,0X04
		DB	0X16,0X05
		DB	0X16,0X06
		DB	0X16,0X07

		DB	0X16,0X08
		DB	0X16,0X09
                DB	0X16,0X0A
		DB	0X16,0X0B
		DB	0X16,0X0C
		DB	0X16,0X0D
		DB	0X16,0X0E
		DB	0X16,0X0F
		
		DB	0X17,0X00
		DB	0X17,0X01
                DB	0X17,0X02
		DB	0X17,0X03
		DB	0X17,0X04
		DB	0X17,0X05
		DB	0X17,0X06
		DB	0X17,0X07

		DB	0X17,0X08
		DB	0X17,0X09
                DB	0X17,0X0A
		DB	0X17,0X0B
		DB	0X17,0X0C
		DB	0X17,0X0D
		DB	0X17,0X0E
		DB	0X17,0X0F
		
	ORG 0X2300
	
ASK_T_TABLE:	DB	0X30,0X30,0XFF,0X00
                DB	0X30,0X31,0XFF,0X00
		DB	0X30,0X32,0XFF,0X00
                DB	0X30,0X33,0XFF,0X00
                DB	0X30,0X34,0XFF,0X00
                DB	0X30,0X35,0XFF,0X00
                DB	0X30,0X36,0XFF,0X00
                DB	0X30,0X37,0XFF,0X00
                DB	0X30,0X38,0XFF,0X00
                DB	0X30,0X39,0XFF,0X00

                DB	0X31,0X30,0XFF,0X00
                DB	0X31,0X31,0XFF,0X00
                DB	0X31,0X32,0XFF,0X00
                DB	0X31,0X33,0XFF,0X00
                DB	0X31,0X34,0XFF,0X00
                DB	0X31,0X35,0XFF,0X00
                DB	0X31,0X36,0XFF,0X00
                DB	0X31,0X37,0XFF,0X00
                DB	0X31,0X38,0XFF,0X00
                DB	0X31,0X39,0XFF,0X00
                
                DB	0X32,0X30,0XFF,0X00
                DB	0X32,0X31,0XFF,0X00
                DB	0X32,0X32,0XFF,0X00
                DB	0X32,0X33,0XFF,0X00
                DB	0X32,0X34,0XFF,0X00
                DB	0X32,0X35,0XFF,0X00
                DB	0X32,0X36,0XFF,0X00
                DB	0X32,0X37,0XFF,0X00
                DB	0X32,0X38,0XFF,0X00
                DB	0X32,0X39,0XFF,0X00

                DB	0X33,0X30,0XFF,0X00
                DB	0X33,0X31,0XFF,0X00
                DB	0X33,0X32,0XFF,0X00
                DB	0X33,0X33,0XFF,0X00
                DB	0X33,0X34,0XFF,0X00
                DB	0X33,0X35,0XFF,0X00
                DB	0X33,0X36,0XFF,0X00
                DB	0X33,0X37,0XFF,0X00
                DB	0X33,0X38,0XFF,0X00
                DB	0X33,0X39,0XFF,0X00

                DB	0X34,0X30,0XFF,0X00
                DB	0X34,0X31,0XFF,0X00
                DB	0X34,0X32,0XFF,0X00
                DB	0X34,0X33,0XFF,0X00
                DB	0X34,0X34,0XFF,0X00
                DB	0X34,0X35,0XFF,0X00
                DB	0X34,0X36,0XFF,0X00
                DB	0X34,0X37,0XFF,0X00
                DB	0X34,0X38,0XFF,0X00
                DB	0X34,0X39,0XFF,0X00

                DB	0X35,0X30,0XFF,0X00
                DB	0X35,0X31,0XFF,0X00
                DB	0X35,0X32,0XFF,0X00
                DB	0X35,0X33,0XFF,0X00
                DB	0X35,0X34,0XFF,0X00
                DB	0X35,0X35,0XFF,0X00
                DB	0X35,0X36,0XFF,0X00
                DB	0X35,0X37,0XFF,0X00
                DB	0X35,0X38,0XFF,0X00
                DB	0X35,0X39,0XFF,0X00
                
                DB	0X36,0X30,0XFF,0X00
                DB	0X36,0X31,0XFF,0X00
                DB	0X36,0X32,0XFF,0X00
                DB	0X36,0X33,0XFF,0X00
                DB	0X36,0X34,0XFF,0X00
                DB	0X36,0X35,0XFF,0X00
                DB	0X36,0X36,0XFF,0X00
                DB	0X36,0X37,0XFF,0X00
                DB	0X36,0X38,0XFF,0X00
                DB	0X36,0X39,0XFF,0X00
                
                DB	0X37,0X30,0XFF,0X00
                DB	0X37,0X31,0XFF,0X00
                DB	0X37,0X32,0XFF,0X00
                DB	0X37,0X33,0XFF,0X00
                DB	0X37,0X34,0XFF,0X00
                DB	0X37,0X35,0XFF,0X00
                DB	0X37,0X36,0XFF,0X00
                DB	0X37,0X37,0XFF,0X00
                DB	0X37,0X38,0XFF,0X00
                DB	0X37,0X39,0XFF,0X00

                DB	0X38,0X30,0XFF,0X00
                
	ORG 0X2500

GOOD_VAL_TBL:	DB	0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08
		DB	0X09,0X10,0X11,0X12
GOOD_Y_TBL:	DB	0X13,0X14,0X15,0X16,0X17,0X18,0X19,0X20
		DB	0X21,0X22,0X23,0X24,0X25,0X26,0X27,0X28
		DB	0X29,0X30,0X31,0XFF
		
	ORG 0X2600
		
BCD_TABLE:	DB	0X20,0X30,0XFF,0XFF
		DB	0X20,0X31,0XFF,0XFF
		DB	0X20,0X32,0XFF,0XFF
		DB	0X20,0X33,0XFF,0XFF
		DB	0X20,0X34,0XFF,0XFF
		DB	0X20,0X35,0XFF,0XFF
		DB	0X20,0X36,0XFF,0XFF
		DB	0X20,0X37,0XFF,0XFF
		DB	0X20,0X38,0XFF,0XFF
		DB	0X20,0X39,0XFF,0XFF
		
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		
		DB	0X31,0X30,0XFF,0XFF
		DB	0X31,0X31,0XFF,0XFF
		DB	0X31,0X32,0XFF,0XFF
		DB	0X31,0X33,0XFF,0XFF
		DB	0X31,0X34,0XFF,0XFF
		DB	0X31,0X35,0XFF,0XFF
		DB	0X31,0X36,0XFF,0XFF
		DB	0X31,0X37,0XFF,0XFF
		DB	0X31,0X38,0XFF,0XFF
		DB	0X31,0X39,0XFF,0XFF

		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		
		DB	0X32,0X30,0XFF,0XFF
		DB	0X32,0X31,0XFF,0XFF
		DB	0X32,0X32,0XFF,0XFF
		DB	0X32,0X33,0XFF,0XFF
		DB	0X32,0X34,0XFF,0XFF
		DB	0X32,0X35,0XFF,0XFF
		DB	0X32,0X36,0XFF,0XFF
		DB	0X32,0X37,0XFF,0XFF
		DB	0X32,0X38,0XFF,0XFF
		DB	0X32,0X39,0XFF,0XFF

		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		DB	0X00,0X00,0XFF,0XFF
		
		DB	0X33,0X30,0XFF,0XFF
		DB	0X33,0X31,0XFF,0XFF
		DB	0X33,0X32,0XFF,0XFF
		DB	0X33,0X33,0XFF,0XFF
		DB	0X33,0X34,0XFF,0XFF
		DB	0X33,0X35,0XFF,0XFF
		DB	0X33,0X36,0XFF,0XFF
		DB	0X33,0X37,0XFF,0XFF
		DB	0X33,0X38,0XFF,0XFF
		DB	0X33,0X39,0XFF,0XFF
		
	ORG 0X2700
		
TABLE_28DAYS:	DB	0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08
		DB	0X09,0X10,0X11,0X12,0X13,0X14,0X15,0X16
		DB	0X17,0X18,0X19,0X20,0X21,0X22,0X23,0X24
		DB	0X25,0X26,0X27,0X28,0X01,0X00

TABLE_29DAYS:	DB	0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08
		DB	0X09,0X10,0X11,0X12,0X13,0X14,0X15,0X16
		DB	0X17,0X18,0X19,0X20,0X21,0X22,0X23,0X24
		DB	0X25,0X26,0X27,0X28,0X29,0X01

TABLE_30DAYS:	DB	0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08
		DB	0X09,0X10,0X11,0X12,0X13,0X14,0X15,0X16
		DB	0X17,0X18,0X19,0X20,0X21,0X22,0X23,0X24
		DB	0X25,0X26,0X27,0X28,0X29,0X30,0X01,0X00

TABLE_31DAYS:   DB	0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08
		DB	0X09,0X10,0X11,0X12,0X13,0X14,0X15,0X16
		DB	0X17,0X18,0X19,0X20,0X21,0X22,0X23,0X24
		DB	0X25,0X26,0X27,0X28,0X29,0X30,0X31,0X01

TABLE_MONTHS	DB	0X01,0X02,0X03,0X04,0X05,0X06,0X07,0X08
		DB	0X09,0X10,0X11,0X12,0X01,0X00
		
TABLE_YEARS:	DB	0X18,0X19,0X20,0X21,0X22,0X23,0X24,0X25
		DB	0X26,0X27,0X28,0X29,0X30,0X00

        ORG 0X3000

;Digits/symbolen

LETTER_TABLE:	DB	0X0C,0X03

		DB      0X00,0X00,0X00,0X00 ;Spatie
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC21
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 22
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 23
                DB      0X00,0X00,0X00,0X00 ;Hier gebruikt als
                DB      0X00,0X00,0X00,0X00 ;brede spatie van 6
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X04

		DB      0X00,0X00,0X00,0X00 ;ASC 24
                DB      0X00,0X00,0X00,0X00 ;Hier gebruikt als spatie
                DB      0X00,0X00,0X00,0X00 ;van 4
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

                DB      0X70,0X00,0X00,0X00 ;%
                DB      0X50,0X02,0X00,0X00
                DB      0X70,0X01,0X00,0X00
                DB      0X80,0X00,0X00,0X00
                DB      0X40,0X00,0X00,0X00
                DB      0XA0,0X03,0X00,0X00
                DB      0X90,0X02,0X00,0X00
                DB      0X80,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 26
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

                DB      0X00,0X00,0X00,0X00 ;ASC 27
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X02

                DB	0XE0,0X03,0X00,0X00 ;(
                DB	0X10,0X04,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X02

		DB	0X10,0X04,0X00,0X00 ;)
                DB	0XE0,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X02,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 2A
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X05

		DB      0X00,0X01,0X00,0X00 ;ASC 2B
                DB      0X00,0X01,0X00,0X00
                DB      0XC0,0X07,0X00,0X00
                DB      0X00,0X01,0X00,0X00
                DB      0X00,0X01,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 2C
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X04  	   ;-

		DB      0X80,0X00,0X00,0X00
                DB      0X80,0X00,0X00,0X00
                DB      0X80,0X00,0X00,0X00
                DB      0X80,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X01
                
                DB      0X00,0X04,0X00,0X00 ;ASC 2E
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X02,0X00,0X00 ;/
		DB      0X00,0X01,0X00,0X00
		DB      0X80,0X00,0X00,0X00
		DB      0X40,0X00,0X00,0X00
		DB      0X20,0X00,0X00,0X00
		DB      0X10,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

		DB	0X0C,0X06

		DB      0XE0,0X03,0X00,0X00 ;0
                DB      0X10,0X04,0X00,0X00
                DB      0X10,0X04,0X00,0X00
                DB      0X10,0X04,0X00,0X00
                DB      0X10,0X04,0X00,0X00
                DB      0XE0,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

                DB      0X00,0X00,0X00,0X00 ;1
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
		DB      0XF0,0X07,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
		DB      0X00,0X00,0X00,0X00
		DB      0X00,0X00,0X00,0X00

		DB	0X0C,0X06

		DB      0X20,0X04,0X00,0X00 ;2
                DB      0X10,0X06,0X00,0X00
                DB      0X10,0X05,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X50,0X04,0X00,0X00
                DB      0X20,0X04,0X00,0X00
		DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X20,0X02,0X00,0X00 ;3
                DB      0X10,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X60,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0XF0,0X01,0X00,0X00 ;4
                DB      0X00,0X01,0X00,0X00
                DB      0X00,0X01,0X00,0X00
                DB      0X00,0X01,0X00,0X00
                DB      0XC0,0X07,0X00,0X00
                DB      0X00,0X01,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

		DB	0X0C,0X06

		DB      0XF0,0X02,0X00,0X00 ;5
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X00,0X03,0X00,0X00
		DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB      0XE0,0X03,0X00,0X00 ;6
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X20,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB      0X10,0X00,0X00,0X00 ;7
                DB      0X10,0X06,0X00,0X00
                DB      0X10,0X01,0X00,0X00
                DB      0X90,0X00,0X00,0X00
                DB      0X50,0X00,0X00,0X00
                DB      0X30,0X00,0X00,0X00
		DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X60,0X03,0X00,0X00 ;8
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X60,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06
                
		DB      0X60,0X02,0X00,0X00 ;9
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0X90,0X04,0X00,0X00
                DB      0XE0,0X03,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X02

		DB      0X40,0X04,0X00,0X00 ; :
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 3B
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 3C
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;=
                DB      0X80,0X02,0X00,0X00
                DB      0X80,0X02,0X00,0X00
                DB      0X80,0X02,0X00,0X00
                DB      0X80,0X02,0X00,0X00
                DB      0X80,0X02,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 3E
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X05

		DB      0X20,0X00,0X00,0X00 ;?
		DB      0X10,0X05,0X00,0X00
		DB      0X90,0X00,0X00,0X00
		DB      0X90,0X00,0X00,0X00
		DB      0X60,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 40
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

        	DB	0X0C,0X08

		DB	0XFC,0X07,0X00,0X00 ;A
                DB	0X42,0X00,0X00,0X00
                DB	0X41,0X00,0X00,0X00
                DB	0X41,0X00,0X00,0X00
                DB	0X41,0X00,0X00,0X00
                DB	0X41,0X00,0X00,0X00
                DB	0X42,0X00,0X00,0X00
                DB	0XFC,0X07,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

                DB	0XFF,0X07,0X00,0X00 ;B
                DB	0X22,0X02,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X52,0X02,0X00,0X00
                DB	0X8C,0X01,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFC,0X01,0X00,0X00 ;C
                DB	0X02,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X02,0X02,0X00,0X00
                DB	0X04,0X01,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFF,0X07,0X00,0X00 ;D
                DB	0X02,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X02,0X02,0X00,0X00
                DB	0XFC,0X01,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
		DB	0X0C,0X08

		DB	0XFF,0X07,0X00,0X00 ;E
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFF,0X07,0X00,0X00 ;F
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X01,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFC,0X01,0X00,0X00 ;G
                DB	0X02,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X22,0X02,0X00,0X00
                DB	0XE4,0X01,0X00,0X00
                DB      0X00,0X00,0X00,0X00

		DB	0X0C,0X08

		DB	0XFF,0X07,0X00,0X00 ;H
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0XFF,0X07,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X05

		DB	0X01,0X04,0X00,0X00 ;I
                DB	0X01,0X04,0X00,0X00
                DB	0XFF,0X07,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
		DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
                DB	0X0C,0X07

                DB	0X01,0X01,0X00,0X00 ;J
                DB	0X01,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X02,0X00,0X00
                DB	0XFF,0X01,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFF,0X07,0X00,0X00 ;K
                DB	0X40,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X50,0X00,0X00,0X00
                DB	0X88,0X00,0X00,0X00
                DB	0X04,0X01,0X00,0X00
                DB	0X02,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFF,0X07,0X00,0X00 ;L
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X09

		DB	0XFF,0X07,0X00,0X00 ;M
                DB	0X02,0X00,0X00,0X00
                DB	0X04,0X00,0X00,0X00
                DB	0X08,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X08,0X00,0X00,0X00
                DB	0X04,0X00,0X00,0X00
                DB	0X02,0X00,0X00,0X00
                DB	0XFF,0X07,0X00,0X00

		DB	0X0C,0X08

                DB	0XFF,0X07,0X00,0X00 ;N
                DB	0X04,0X00,0X00,0X00
                DB	0X08,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X40,0X00,0X00,0X00
                DB	0X80,0X00,0X00,0X00
                DB	0XFF,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X08
                
		DB	0XFC,0X01,0X00,0X00 ;O
                DB	0X02,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X02,0X02,0X00,0X00
                DB	0XFC,0X01,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X08
                
		DB	0XFF,0X07,0X00,0X00 ;P
                DB	0X12,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X12,0X00,0X00,0X00
                DB	0X0C,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X08
                
		DB	0XFC,0X01,0X00,0X00 ;Q
                DB	0X02,0X02,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X01,0X04,0X00,0X00
                DB	0X81,0X04,0X00,0X00
                DB	0X01,0X05,0X00,0X00
                DB	0X02,0X02,0X00,0X00
                DB	0XFC,0X05,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X08
                
		DB	0XFF,0X07,0X00,0X00 ;R
                DB	0X12,0X00,0X00,0X00
                DB	0X21,0X00,0X00,0X00
                DB	0X61,0X00,0X00,0X00
                DB	0XA1,0X00,0X00,0X00
                DB	0X21,0X01,0X00,0X00
                DB	0X12,0X02,0X00,0X00
                DB	0X0C,0X04,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X08

		DB	0X0C,0X02,0X00,0X00 ;S
                DB	0X12,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X41,0X02,0X00,0X00
                DB	0X82,0X01,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X09
                
		DB	0X01,0X00,0X00,0X00 ;T
                DB	0X01,0X00,0X00,0X00
                DB	0X01,0X00,0X00,0X00
                DB	0X01,0X00,0X00,0X00
                DB	0XFF,0X07,0X00,0X00
                DB	0X01,0X00,0X00,0X00
                DB	0X01,0X00,0X00,0X00
                DB	0X01,0X00,0X00,0X00
                DB	0X01,0X00,0X00,0X00

                DB	0X0C,0X08

		DB	0XFF,0X01,0X00,0X00 ;U
                DB	0X00,0X02,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X02,0X00,0X00
                DB	0XFF,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X09
                
		DB	0X0F,0X00,0X00,0X00 ;V
                DB	0X30,0X00,0X00,0X00
                DB	0XC0,0X00,0X00,0X00
                DB	0X00,0X03,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X03,0X00,0X00
                DB	0XC0,0X00,0X00,0X00
                DB	0X30,0X00,0X00,0X00
                DB	0X0F,0X00,0X00,0X00

                DB	0X0C,0X09

		DB	0XFF,0X01,0X00,0X00 ;W
                DB	0X00,0X02,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X02,0X00,0X00
                DB	0XC0,0X01,0X00,0X00
                DB	0X00,0X02,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X02,0X00,0X00
                DB	0XFF,0X01,0X00,0X00
                
		DB	0X0C,0X09

		DB	0X03,0X06,0X00,0X00 ;X
                DB	0X04,0X01,0X00,0X00
                DB	0X88,0X00,0X00,0X00
                DB	0X50,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X50,0X00,0X00,0X00
                DB	0X88,0X00,0X00,0X00
                DB	0X04,0X01,0X00,0X00
                DB	0X03,0X06,0X00,0X00

                DB	0X0C,0X09

		DB	0X07,0X00,0X00,0X00 ;Y
                DB	0X08,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0XC0,0X07,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X08,0X00,0X00,0X00
                DB	0X07,0X00,0X00,0X00
                
                DB	0X0C,0X08

                DB	0X01,0X07,0X00,0X00 ;Z
                DB	0X81,0X04,0X00,0X00
                DB	0X41,0X04,0X00,0X00
                DB	0X21,0X04,0X00,0X00
                DB	0X11,0X04,0X00,0X00
                DB	0X09,0X04,0X00,0X00
                DB	0X05,0X04,0X00,0X00
                DB	0X03,0X04,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 5B
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 5C
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 5D
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 5E
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB	0X00,0X04,0X00,0X00 ;_
		DB	0X00,0X04,0X00,0X00
		DB	0X00,0X04,0X00,0X00
		DB	0X00,0X04,0X00,0X00
		DB	0X00,0X04,0X00,0X00
		DB	0X00,0X04,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 60
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

		DB	0X0C,0X06

		DB	0XE0,0X03,0X00,0X00 ;a
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X20,0X02,0X00,0X00
                DB	0XF0,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

                DB	0XFE,0X07,0X00,0X00 ;b
                DB	0X20,0X02,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X02,0X00,0X00
                DB	0XE0,0X03,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06
                
		DB	0XC0,0X01,0X00,0X00 ;c
                DB	0X20,0X02,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X20,0X02,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

                DB	0XE0,0X03,0X00,0X00 ;d
		DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X20,0X02,0X00,0X00
                DB	0XFE,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06
                
		DB	0XE0,0X03,0X00,0X00 ;e
                DB	0X90,0X04,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X60,0X02,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X05
                
		DB	0XF0,0X0F,0X00,0X00 ;f
                DB	0X88,0X00,0X00,0X00
                DB	0X88,0X00,0X00,0X00
                DB	0X88,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB	0X60,0X04,0X00,0X00 ;g
                DB	0X90,0X08,0X00,0X00
                DB	0X90,0X08,0X00,0X00
                DB	0X90,0X08,0X00,0X00
                DB	0X90,0X08,0X00,0X00
                DB	0XF0,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB	0XFE,0X07,0X00,0X00 ;h
                DB	0X40,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0XC0,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X01
                
		DB	0XE8,0X07,0X00,0X00 ;i
		DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X05

		DB	0X00,0X04,0X00,0X00 ;j
                DB	0X00,0X08,0X00,0X00
                DB	0X00,0X08,0X00,0X00
                DB	0X00,0X08,0X00,0X00
                DB	0XE8,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB	0XFC,0X07,0X00,0X00 ;k
                DB	0X00,0X01,0X00,0X00
                DB	0X80,0X00,0X00,0X00
                DB	0X40,0X01,0X00,0X00
                DB	0X20,0X02,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

		DB	0X0C,0X05

		DB	0XFC,0X03,0X00,0X00 ;l
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X02,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X07

		DB	0XE0,0X07,0X00,0X00 ;m
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0XE0,0X07,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0XE0,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06
                
		DB	0XF0,0X07,0X00,0X00 ;n
                DB	0X20,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0XE0,0X07,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB	0XE0,0X03,0X00,0X00 ;o
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0XE0,0X03,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06
                
		DB	0XF0,0X0F,0X00,0X00 ;p
                DB	0XA0,0X00,0X00,0X00
                DB	0X10,0X01,0X00,0X00
                DB	0X10,0X01,0X00,0X00
                DB	0X10,0X01,0X00,0X00
                DB	0XE0,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

		DB	0X0C,0X06

		DB	0XE0,0X00,0X00,0X00 ;q
                DB	0X10,0X01,0X00,0X00
                DB	0X10,0X01,0X00,0X00
                DB	0X10,0X01,0X00,0X00
                DB	0XA0,0X00,0X00,0X00
                DB	0XF0,0X0F,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06

		DB	0XE0,0X07,0X00,0X00 ;r
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X10,0X00,0X00,0X00
                DB	0X20,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X06
                
		DB	0XE0,0X02,0X00,0X00 ;s
                DB	0X90,0X04,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X20,0X03,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X05

		DB	0XFE,0X03,0X00,0X00 ;t
                DB	0X20,0X04,0X00,0X00
                DB	0X20,0X04,0X00,0X00
                DB	0X20,0X04,0X00,0X00
                DB	0X00,0X02,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

		DB	0X0C,0X06

		DB	0XF0,0X03,0X00,0X00 ;u
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0XF0,0X03,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X07

		DB	0X30,0X00,0X00,0X00 ;v
                DB	0XC0,0X00,0X00,0X00
                DB	0X00,0X03,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X03,0X00,0X00
                DB	0XC0,0X00,0X00,0X00
                DB	0X30,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X07
                
	        DB	0XF0,0X00,0X00,0X00 ;w
                DB	0X00,0X03,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X80,0X03,0X00,0X00
                DB	0X00,0X04,0X00,0X00
                DB	0X00,0X03,0X00,0X00
                DB	0XF0,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

		DB	0X0C,0X07

		DB	0X10,0X04,0X00,0X00 ;x
                DB	0X20,0X02,0X00,0X00
                DB	0X40,0X01,0X00,0X00
                DB	0X80,0X00,0X00,0X00
                DB	0X40,0X01,0X00,0X00
                DB	0X20,0X02,0X00,0X00
                DB	0X10,0X04,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                
                DB	0X0C,0X07
                
		DB	0X30,0X00,0X00,0X00 ;y
                DB	0X40,0X00,0X00,0X00
                DB	0X80,0X00,0X00,0X00
                DB	0X00,0X0F,0X00,0X00
                DB	0X80,0X00,0X00,0X00
                DB	0X40,0X00,0X00,0X00
                DB	0X30,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB	0X10,0X04,0X00,0X00 ;z
                DB	0X10,0X06,0X00,0X00
                DB	0X10,0X05,0X00,0X00
                DB	0X90,0X04,0X00,0X00
                DB	0X50,0X04,0X00,0X00
                DB	0X30,0X04,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 7B
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 7C
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 7D
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 7E
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

                DB	0X0C,0X06

		DB      0X00,0X00,0X00,0X00 ;ASC 7F
                DB      0X00,0X00,0X00,0X00 ;Speciale brede spatie
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
D_ERROR:	DB	0X11,0X3E

		DB	0XFF,0XFF,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0X80,0XFF,0X01,0X00
		DB	0XC0,0X03,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X00,0X03,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0X80,0XFF,0X01,0X00
		DB	0XC0,0X03,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X00,0X03,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X80,0XFF,0X00,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0XC0,0XC1,0X01,0X00
		DB	0XC0,0XC1,0X01,0X00
		DB	0XC0,0XC1,0X01,0X00
		DB	0XC0,0XC1,0X01,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0X80,0XFF,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0XC0,0XFF,0X01,0X00
		DB	0X80,0XFF,0X01,0X00
		DB	0XC0,0X03,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X01,0X00,0X00
		DB	0XC0,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X00,0X03,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00

D0:		DB	0X11,0X0D

		DB	0XFC,0X7F,0X00,0X00
		DB	0XFE,0XFF,0X00,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0X0F,0XE0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X0F,0XE0,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFE,0XFF,0X00,0X00
		DB	0XFC,0X7F,0X00,0X00

D1:		DB	0X11,0X0D

		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X04,0X00,0X00,0X00
		DB	0X06,0X80,0X01,0X00
		DB	0X07,0X80,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0X00,0X80,0X01,0X00
		DB	0X00,0X80,0X01,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00

D2:		DB	0X11,0X0D

		DB	0X0C,0XE0,0X01,0X00
		DB	0X0E,0XF0,0X01,0X00
		DB	0X0F,0XF8,0X01,0X00
		DB	0X0F,0XFC,0X01,0X00
		DB	0X07,0XFE,0X01,0X00
		DB	0X07,0XDF,0X01,0X00
		DB	0X87,0XCF,0X01,0X00
		DB	0XC7,0XC7,0X01,0X00
		DB	0XE7,0XC3,0X01,0X00
		DB	0XFF,0XC1,0X01,0X00
		DB	0XFF,0XC0,0X01,0X00
		DB	0X7E,0XC0,0X01,0X00
		DB	0X3C,0XC0,0X01,0X00

D3:		DB	0X11,0X0D

		DB	0X0C,0X60,0X00,0X00
		DB	0X0E,0XE0,0X00,0X00
		DB	0X0F,0XE0,0X01,0X00
		DB	0X0F,0XE0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0XCF,0XE7,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFE,0XFE,0X00,0X00
		DB	0X7C,0X7C,0X00,0X00

D4:		DB	0X11,0X0D

		DB	0XFF,0X0F,0X00,0X00
		DB	0XFF,0X0F,0X00,0X00
		DB	0XFF,0X0F,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0X00,0X0E,0X00,0X00
		DB	0XF0,0XFF,0X01,0X00
		DB	0XF0,0XFF,0X01,0X00
		DB	0XF0,0XFF,0X01,0X00

D5:		DB	0X11,0X0D

		DB	0XFF,0X63,0X00,0X00
		DB	0XFF,0XE3,0X00,0X00
		DB	0XFF,0XE3,0X01,0X00
		DB	0X87,0XE3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XE7,0X01,0X00
		DB	0X87,0XFF,0X01,0X00
		DB	0X07,0XFF,0X00,0X00
		DB	0X07,0X7E,0X00,0X00

D6:		DB	0X11,0X0D

		DB	0XFC,0X7F,0X00,0X00
		DB	0XFE,0XFF,0X00,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0X8F,0XE7,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X8F,0XE7,0X01,0X00
		DB	0X8F,0XFF,0X01,0X00
		DB	0X0E,0XFF,0X00,0X00
		DB	0X0C,0X7E,0X00,0X00

D7:		DB	0X11,0X0D

		DB	0X00,0X00,0X00,0X00
		DB	0X07,0X00,0X00,0X00
		DB	0X07,0X00,0X00,0X00
		DB	0X07,0XC0,0X01,0X00
		DB	0X07,0XF0,0X01,0X00
		DB	0X07,0XFC,0X01,0X00
		DB	0X07,0XFF,0X01,0X00
		DB	0XC7,0X3F,0X00,0X00
		DB	0XF7,0X0F,0X00,0X00
		DB	0XFF,0X03,0X00,0X00
		DB	0XFF,0X00,0X00,0X00
		DB	0X3F,0X00,0X00,0X00
		DB	0X0F,0X00,0X00,0X00

D8:		DB	0X11,0X0D

		DB	0X7C,0X7C,0X00,0X00
		DB	0XFE,0XFE,0X00,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XCF,0XE7,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0XCF,0XE7,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFE,0XFE,0X00,0X00
		DB	0X7C,0X7C,0X00,0X00

D9:		DB	0X11,0X0D

		DB	0X7C,0X60,0X00,0X00
		DB	0XFE,0XE1,0X00,0X00
		DB	0XFF,0XE3,0X01,0X00
		DB	0XCF,0XE3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0X87,0XC3,0X01,0X00
		DB	0XCF,0XE7,0X01,0X00
		DB	0XFF,0XFF,0X01,0X00
		DB	0XFE,0XFF,0X00,0X00
		DB	0XFC,0X7F,0X00,0X00

D_SPACE:	DB	0X11,0X0D

		DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

D_MINUS:	DB	0X11,0X0D

		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X80,0X03,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		
D_CROSS:	DB	0X11,0X0D

		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		DB	0X80,0X80,0X00,0X00
		DB	0X00,0X41,0X00,0X00
		DB	0X00,0X22,0X00,0X00
		DB	0X00,0X14,0X00,0X00
		DB	0X00,0X08,0X00,0X00
		DB	0X00,0X14,0X00,0X00
		DB	0X00,0X22,0X00,0X00
		DB	0X00,0X41,0X00,0X00
		DB	0X80,0X80,0X00,0X00
		DB	0X00,0X00,0X00,0X00
		
ARROW_LR:	DB	0X0A,0X0A

                DB      0X08,0X00,0X00,0X00
                DB      0X1C,0X00,0X00,0X00
                DB      0X2A,0X00,0X00,0X00
                DB      0X08,0X00,0X00,0X00
                DB      0X08,0X00,0X00,0X00
                DB      0X08,0X00,0X00,0X00
                DB      0X08,0X00,0X00,0X00
                DB      0X2A,0X00,0X00,0X00
                DB      0X1C,0X00,0X00,0X00
                DB      0X08,0X00,0X00,0X00

ARROW_UD:	DB	0X0A,0X0A

		DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X84,0X00,0X00,0X00
                DB      0X02,0X01,0X00,0X00
                DB      0XFF,0X03,0X00,0X00
                DB      0X02,0X01,0X00,0X00
                DB      0X84,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                
ARROW_E:	DB	0X0A,0X0A

		DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

CHOICE_N:       DB	0X0C,0X0B

		DB      0XF0,0X0F,0X00,0X00
                DB      0X10,0X08,0X00,0X00
                DB      0X10,0X08,0X00,0X00
                DB      0X10,0X08,0X00,0X00
                DB      0X10,0X08,0X00,0X00
                DB      0X10,0X08,0X00,0X00
                DB      0X10,0X08,0X00,0X00
                DB      0XF0,0X0F,0X00,0X00
		DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00
                DB	0X00,0X00,0X00,0X00

CHOICE_Y:	DB	0X0C,0X0B

                DB      0XF0,0X0F,0X00,0X00
                DB      0XD0,0X08,0X00,0X00
                DB      0X90,0X09,0X00,0X00
                DB      0X10,0X0B,0X00,0X00
                DB      0X10,0X0E,0X00,0X00
                DB      0X10,0X0B,0X00,0X00
                DB      0X90,0X09,0X00,0X00
                DB      0XF0,0X0F,0X00,0X00
                DB      0XC0,0X00,0X00,0X00
                DB      0X60,0X00,0X00,0X00
                DB      0X30,0X00,0X00,0X00
                
USCORE:		DB	0X01,0X0E

                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                DB      0X01,0X00,0X00,0X00
                
USCORE_E:	DB	0X01,0X0E

                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

WT:		DB	0X18,0X30

                DB      0XFF,0XFF,0XFF,0XFF
                DB      0XFF,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0X87,0XFF,0XFF,0XFF
                DB      0X1F,0XFE,0XFF,0XFF
                DB      0X7F,0XF8,0XFF,0XFF
                DB      0XFF,0XE1,0XFF,0XFF
                DB      0XFF,0X87,0XFF,0XFF
                DB      0XFF,0X1F,0XFE,0XFF
                DB      0XFF,0X7F,0XF8,0XFF
                DB      0XFF,0XFF,0XF1,0XFF
                DB      0XFF,0X7F,0XF0,0XFF
                DB      0XFF,0X1F,0XFE,0XFF
                DB      0XFF,0X87,0XFF,0XFF
                DB      0XFF,0XE3,0XFF,0XFF
                DB      0XFF,0X87,0XFF,0XFF
                DB      0XFF,0X1F,0XFE,0XFF
                DB      0XFF,0X7F,0XF8,0XFF
                DB      0XFF,0XFF,0XF1,0XFF
                DB      0XFF,0X7F,0XF8,0XFF
                DB      0XFF,0X1F,0XFE,0XFF
                DB      0XFF,0X87,0XFF,0XFF
                DB      0XFF,0XE1,0XFF,0XFF
                DB      0X7F,0XF8,0XFF,0XFF
                DB      0X1F,0XFE,0XFF,0XFF
                DB      0X87,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XFF,0XFF,0XFF,0XFF
                DB      0XFF,0XFF,0XFF,0XFF
                DB      0XFF,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0X07,0X00,0XF8,0XFF
                DB      0X07,0X00,0XF8,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XE7,0XFF,0XFF,0XFF
                DB      0XFF,0XFF,0XFF,0XFF
                DB      0XFF,0XFF,0XFF,0XFF

BLOCK:		DB	0X08,0X08

		DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                DB      0XFF,0X00,0X00,0X00
                
NO_BLOCK:	DB	0X08,0X08

		DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00
                DB      0X00,0X00,0X00,0X00

;Kalender gegevens

	ORG 0X5000

CAL_TBL:	DB	0X01,0X06,0X07,0X13,0X14,0X20,0X21,0X27;jan.2018
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;feb.2018
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;mrt.2018
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X07,0X08,0X14,0X15,0X21,0X22;apr.2018
		DB	0X27,0X28,0X29,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X10,0X12,0X13,0X19,0X20,0X21;mei 2018
		DB	0X26,0X27,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;juni 2018
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;juli 2018
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;aug.2018
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;sep.2018
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;okt.2018
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;nov.2018
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;dec.2018
		DB	0X25,0X26,0X29,0X30,0X00,0X00,0X00,0X00
		
		
		DB	0X01,0X05,0X06,0X12,0X13,0X19,0X20,0X26;jan.2019
		DB	0X27,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;feb.2019
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;mrt.2019
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X22,0X27;apr.2019
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;mei 2019
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X10,0X15,0X16,0X22;juni 2019
		DB	0X23,0X29,0X30,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;juli 2019
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;aug.2019
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;sep.2019
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;okt.2019
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;nov.2019
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X25;dec.2019
		DB	0X26,0X28,0X29,0X00,0X00,0X00,0X00,0X00
		
		
		DB	0X01,0X04,0X05,0X11,0X12,0X18,0X19,0X25;jan.2020
		DB	0X26,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;feb.2020
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;mrt.2020
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X13,0X18,0X19,0X25;apr.2020
		DB	0X26,0X27,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X05,0X09,0X10,0X16,0X17,0X21;mei 2020
		DB	0X23,0X24,0X30,0X31,0X00,0X00,0X00,0X00

		DB	0X01,0X06,0X07,0X13,0X14,0X20,0X21,0X27;juni 2020
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;juli 2020
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;aug.2020
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;sep.2020
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;okt.2020
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;nov.2020
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X12,0X13,0X19,0X20,0X25,0X26;dec.2020
		DB	0X27,0X00,0X00,0X00,0X00,0X00,0X00,0X00
		
		
		DB	0X01,0X02,0X03,0X09,0X10,0X16,0X17,0X23;jan.2021
		DB	0X24,0X30,0X31,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;feb.2021
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;mrt.2021
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X05,0X10,0X11,0X17,0X18,0X24;apr.2021
		DB	0X25,0X27,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X13,0X15,0X16,0X22;mei 2021
		DB	0X23,0X24,0X29,0X30,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;juni 2021
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;juli 2021
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;aug.2021
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;sep.2021
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;okt.2021
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;nov.2021
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;dec.2021
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00
		
		
		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;jan.2022
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;feb.2022
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;mrt.2022
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X16,0X17,0X18,0X23;apr.2022
		DB	0X24,0X27,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X26;mei 2022
		DB	0X28,0X29,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X06,0X11,0X12,0X18,0X19,0X25;juni 2022
		DB	0X26,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X23;juli 2022
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;aug.2022
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;sep.2022
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;okt.2022
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;nov.2022
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;dec.2022
		DB	0X26,0X00,0X00,0X00,0X00,0X00,0X00,0X00
		
		
		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;jan.2023
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;feb.2023
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;mrt.2023
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X10,0X15,0X16,0X22;apr.2023
		DB	0X23,0X27,0X29,0X30,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X18,0X20,0X21,0X27;mei 2023
		DB	0X28,0X29,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;juni 2023
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;juli 2023
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;aug.2023
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;sep.2023
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;okt.2023
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;nov.2023
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;dec.2023
		DB	0X25,0X26,0X00,0X00,0X00,0X00,0X00,0X00
		
		
		DB	0X01,0X06,0X07,0X13,0X14,0X20,0X21,0X27;jan.2024
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;feb.2024
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;mrt.2024
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X06,0X07,0X13,0X14,0X20,0X21,0X27;apr.2024
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X09,0X11,0X12,0X18,0X19,0X20;mei 2024
		DB	0X25,0X26,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;juni 2024
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;juli 2024
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;aug.2024
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;sep.2024
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;okt.2024
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;nov.2024
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X25;dec.2024
		DB	0X26,0X28,0X29,0X00,0X00,0X00,0X00,0X00


		DB	0X01,0X04,0X05,0X11,0X12,0X18,0X19,0X25;jan.2025
		DB	0X26,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;feb.2025
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;mrt.2025
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X12,0X13,0X19,0X20,0X21,0X26;apr.2025
		DB	0X27,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X05,0X10,0X11,0X17,0X18,0X24;mei 2025
		DB	0X25,0X29,0X31,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X09,0X14,0X15,0X21,0X22;juni 2025
		DB	0X28,0X29,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;juli 2025
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;aug.2025
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;sep.2025
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;okt.2025
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;nov.2025
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X25,0X26;dec.2025
		DB	0X27,0X28,0X00,0X00,0X00,0X00,0X00,0X00


		DB	0X01,0X03,0X04,0X10,0X11,0X17,0X18,0X24;jan.2026
		DB	0X25,0X31,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;feb.2026
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;mrt.2026
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X06,0X11,0X12,0X18,0X19,0X25;apr.2026
		DB	0X26,0X27,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X14,0X16,0X17,0X23;mei 2026
		DB	0X24,0X25,0X30,0X31,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;juni 2026
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;juli 2026
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;aug.2026
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;sep.2026
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;okt.2026
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;nov.2026
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X12,0X13,0X19,0X20,0X25,0X26;dec.2026
		DB	0X27,0X00,0X00,0X00,0X00,0X00,0X00,0X00


		DB	0X01,0X02,0X03,0X09,0X10,0X16,0X17,0X23;jan.2027
		DB	0X24,0X30,0X31,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;feb.2027
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;mrt.2027
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;apr.2027
		DB	0X27,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X06,0X08,0X09,0X15,0X16,0X17;mei 2027
		DB	0X22,0X23,0X29,0X30,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;juni 2027
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;juli 2027
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;aug.2027
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;sep.2027
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;okt.2027
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;nov.2027
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;dec.2027
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00


		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;jan.2028
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;feb.2028
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;mrt.2028
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X17,0X22;apr.2028
		DB	0X23,0X27,0X29,0X30,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X25,0X27;mei 2028
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X05,0X10,0X11,0X17,0X18,0X24;juni 2028
		DB	0X25,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;juli 2028
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;aug.2028
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;sep.2028
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;okt.2028
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;nov.2028
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;dec.2028
		DB	0X25,0X26,0X30,0X31,0X00,0X00,0X00,0X00

		
		DB	0X01,0X06,0X07,0X13,0X14,0X20,0X21,0X27;jan.2029
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;feb.2029
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;mrt.2029
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X07,0X08,0X14,0X15,0X21,0X22;apr.2029
		DB	0X27,0X28,0X29,0X00,0X00,0X00,0X00,0X00

                DB	0X05,0X06,0X10,0X12,0X13,0X19,0X20,0X21;mei 2029
		DB	0X26,0X27,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;juni 2029
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;juli 2029
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;aug.2029
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;sep.2029
		DB	0X29,0X30,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;okt.2029
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;nov.2029
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X02,0X08,0X09,0X15,0X16,0X22,0X23;dec.2029
		DB	0X25,0X26,0X29,0X30,0X00,0X00,0X00,0X00

		
		DB	0X01,0X05,0X06,0X12,0X13,0X19,0X20,0X26;jan.2030
		DB	0X27,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;feb.2030
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;mrt.2030
		DB	0X30,0X31,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X22,0X27;apr.2030
		DB	0X28,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X04,0X05,0X11,0X12,0X18,0X19,0X25,0X26;mei 2030
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X02,0X08,0X09,0X10,0X15,0X16,0X22;juni 2030
		DB	0X23,0X29,0X30,0X00,0X00,0X00,0X00,0X00

                DB	0X06,0X07,0X13,0X14,0X20,0X21,0X27,0X28;juli 2030
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X03,0X04,0X10,0X11,0X17,0X18,0X24,0X25;aug.2030
		DB	0X31,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X28;sep.2030
		DB	0X29,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X05,0X06,0X12,0X13,0X19,0X20,0X26,0X27;okt.2030
		DB	0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00

		DB	0X02,0X03,0X09,0X10,0X16,0X17,0X23,0X24;nov.2030
		DB	0X30,0X00,0X00,0X00,0X00,0X00,0X00,0X00

                DB	0X01,0X07,0X08,0X14,0X15,0X21,0X22,0X25;dec.2030
		DB	0X26,0X28,0X29,0X00,0X00,0X00,0X00,0X00

;-----------
;Subroutines
;-----------


;---------------------------------------------------------------------------
;
;Interrupt routine
;
;---------------------------------------------------------------------------

INT_TIME:	MOVFF	WREG,WREG_LOC 	;Push W
		MOVFF	TEMP,TEMP_LOC	;Push TEMP

		BCF	T0CON,TMR0ON    ;Stop de timer
		MOVLW	INT_VAL_H       ;Preset de waarde voor de
		MOVWF	TMR0H           ;tijd tussen 2 interrupts
		MOVLW	INT_VAL_L
		MOVWF	TMR0L
		BSF	T0CON,TMR0ON    ;Geef timer weer vrij

		BSF	INT		;Hijs de interruptflag

		BTG	T50		;Toon de 50 msec

		BTFSC	DISA_ROT_SW	;Mogen we de drukschakelaar bekijken
		GOTO	C50             ;Nee

		BTFSC	ROT_SW          ;Ja, is er gedrukt?
		GOTO	C50             ;Nee
		BSF	PRESS_IN_INT    ;Ja zet flag en
		BSF	DISA_ROT_SW     ;blokkeer meteen

C50:		INCF	INT_CNT		;Tel de 50 msec.
		MOVLW	0X14
		CPFSEQ	INT_CNT
		GOTO	END_I_T
		CLRF	INT_CNT

		INCF	PIC_SEC_HEX	;Maak de volgende seconde
		MOVLW	0X3C
		CPFSEQ	PIC_SEC_HEX
		GOTO	END_S_S

		CLRF	PIC_SEC_HEX
		INCF	PIC_MIN_HEX
		MOVLW	0X3C
		CPFSEQ	PIC_MIN_HEX
		GOTO	END_S_S

		CLRF	PIC_MIN_HEX
		INCF	PIC_HR_HEX
		MOVLW	0X18
		CPFSEQ	PIC_HR_HEX
		GOTO	END_S_S
		CLRF	PIC_HR_HEX

END_S_S:        MOVFF	OLD_MIN,WREG 	;Kijk of er een nieuwe minuut is
		CPFSEQ	PIC_MIN_HEX
		GOTO	MAKE_EQ         ;Ja hoor
		GOTO	RPH             ;Nee

MAKE_EQ:	MOVFF	PIC_MIN_HEX,OLD_MIN

RPH:            MOVFF	PIC_HR_HEX,TEMP_H;Ga de minutenteller vullen
		MOVFF	PIC_MIN_HEX,TEMP_L

		CALL	MAKE_MIN_CNT

                MOVFF	MIN_CNT_H,T_POINT_H;Actuele tijd
		MOVFF	MIN_CNT_L,T_POINT_L
		CALL	TIME_TO_RAM     ;Bereken windowRAM lokatie
		MOVFF	INDF2,REL_BYTE	;Haal actuele opdracht uit windowRAM
                
		CALL	CALC_PIC_VAL	;Bepaal aantal secondes PIC tijd

DO_RPH:		BTFSC	JUMPER		;Vertragingsjumper?
		GOTO	DO_ANA          ;Nee
		
		BTFSS	BLOCKED		;Is A/D toegestaan
		GOTO	DO_ANA		;Ja hoor
		
		MOVLW	0X00   		;Lage counter op 0?
		CPFSEQ	BLOCK_CNT_L
		GOTO    ADJ_BLOCK       ;Nee
		MOVLW	0X00            ;Ja, ook hoge op 0?
		CPFSEQ	BLOCK_CNT_H
		GOTO	ADJ_BLOCK       ;Nee

		BCF	BLOCKED         ;Ja, meten maar
		GOTO	DO_ANA
		
ADJ_BLOCK:	CALL	FL_PH_LED	;Zet de fotoLED

		DECF	BLOCK_CNT_L	;Lage teller omlaag
		MOVLW	0XFF            ;Door de nul?
		CPFSEQ	BLOCK_CNT_L
		GOTO	DO_D_S          ;Nee
		DECF	BLOCK_CNT_H     ;Ja, dus hoge omlaag
		GOTO    DO_D_S

DO_ANA:		CALL	READ_PHOTO	;Fotocel lezen

DO_D_S:	 	CALL	DAY_SELECT	;Bepaal of dag speciaal is

		CALL	WRITE_IO        ;Set de uitgangen

CH_NM:		BTFSC	CHECKIT		;Is de checkbit al actief?
		GOTO	TEST_CS         ;Ja

		MOVLW	0X00   		;Is de minuut 0?
		CPFSEQ	PIC_MIN_HEX
		GOTO	TEST_CS         ;Nee hoor

		MOVLW	0X1E   		;Is het xx.00.30?
		CPFSEQ	PIC_SEC_HEX     ;Weer niet
		GOTO	TEST_CS

		BSF	CHECKIT         ;Ja, dus bij gelegenheid checken

TEST_CS:	BTFSS	PRESS_IN_INT	;Gedrukt?
		GOTO	INT_CONT_S      ;Nee hoor

		BTFSS	STATUS_ACT      ;Is status al actief
		GOTO	FIRST_STATUS    ;Nee
		INCF	STATUS_CNT	;Teller omhoog
		MOVLW	0X04    	;Na 4 sec. mag inputscreen
		CPFSLT	STATUS_CNT
		BSF	ENA_INPS        ;Geef dat aan
		MOVLW	STATUS_TIME
		CPFSEQ	STATUS_CNT
		GOTO	INT_CONT_S      ;Geen nul

		BCF	STATUS_ACT 	;Sessie afgelopen
		BCF	ENA_INPS
		CLRF	STATUS_CNT
		BSF	ERA_STAT	;Dadelijk wissen
		BCF	DISA_ROT_SW     ;Er mag weer gedrukt worden
		BCF	PRESS_IN_INT
		GOTO	INT_CONT_S

FIRST_STATUS:	BSF	STATUS_ACT      ;Status wordt actief
		CLRF	STATUS_CNT

INT_CONT_S:    	;call	disp_dump	;Schrijf op hulpdisplay

END_I_T:	BCF	INTCON,TMR0IF 	;Clear de interrupt

		MOVFF	TEMP_LOC,TEMP	;Pop TEMP
		MOVFF	WREG_LOC,WREG   ;Pop W

		RETFIE
		
;FL_PH_LED = Hulproutine interruptroutine
;----------------------------------------

FL_PH_LED:	INCF	L3_CNT		;Counter omhoog
		MOVLW	0X03            ;Wordt'ie 3?
		CPFSEQ	L3_CNT          ;Ja
		GOTO	SET_FL          ;Nee
		
		CLRF	L3_CNT		;Terug op 0
		
SET_FL:		MOVLW	0X00		;Is'ie 0?
		CPFSEQ	L3_CNT
		GOTO	CH_SF1          ;Nee, kijk naar 1
		GOTO	SF0             ;Situatie 0

CH_SF1:		MOVLW	0X01            ;Is'ie 1?
		CPFSEQ	L3_CNT
		GOTO	CH_SF2          ;Nee, kijk naar 2
		GOTO	SF1             ;Situatie 1

CH_SF2:		MOVLW	0X02            ;Is'ie 2?
		CPFSEQ	L3_CNT
		GOTO    SF_ERROR	;Nee, maar wat dan....
		GOTO	SF2
		
SF0:		BSF	PHOTO_LED	;Situatie 0 is altijd aan
		GOTO	END_FLP
		
SF1:            BSF	PHOTO_LED	;Zet aan
		BTFSS	PHOTOCELL       ;Is het licht?
		BCF	PHOTO_LED       ;Oh nee, dus uit
		GOTO	END_FLP
		
SF2:		BCF	PHOTO_LED	;Situatie 2 is altijd uit
		GOTO	END_FLP

SF_ERROR:	MOVLW	0X01
		MOVFF	WREG,ERROR_CODE
		CALL	ERROR_DISP

END_FLP:	RETURN
		
;-------------------------------------------------------------------------
;
;Timer routines
;
;-------------------------------------------------------------------------

;--------------------------------------------------------------------------
;DELAY = Algemene vertragingsroutine
;		 Entry : DELAY_TYPE is 1 of 2 voor resp. 0.5 of 5 msec
;			 DELAY_LOC is aantal malen de gewenste tijd
;--------------------------------------------------------------------------

DELAY:	 	MOVFF	DELAY_LOC,MAIN_DELAY

		MOVLW	0X01 		;Is DELAY_TYPE soms 1?
		CPFSEQ	DELAY_TYPE
		GOTO	CH_F_2
		GOTO	DELAY_1         ;Ja,dus op naar DELAY_1

CH_F_2:         MOVLW	0X02            ;Geen 1,dan misschien 2?
		CPFSEQ	DELAY_TYPE
		GOTO    END_DELAY	;Weet ik veel
		GOTO	DELAY_10        ;Ja het is 2

DELAY_1:        MOVLW	DELAY_H_1     	;Presets vanuit de EQU waardes
		MOVFF	WREG,DELAY_H    ;voor delay 0.5 msec
		MOVLW	DELAY_L_1
		MOVFF	WREG,DELAY_L
		GOTO	DO_DELAY

DELAY_10:       MOVLW	DELAY_H_10     	;Presets vanuit de EQU waardes
		MOVFF	WREG,DELAY_H    ;voor delay 5 msec
		MOVLW	DELAY_L_10
		MOVFF	WREG,DELAY_L

DO_DELAY:       MOVFF	DELAY_H,CNT_DELAY_H ;Tussenteller vullen omdat de
		MOVFF	DELAY_L,CNT_DELAY_L ;loop hersteld moet kunnen worden

SH_LOOP:	DECFSZ	CNT_DELAY_L,1   ;Binnenloop
		GOTO	SH_LOOP

		MOVFF   DELAY_L,CNT_DELAY_L
		DECFSZ	CNT_DELAY_H     ;Buitenloop
		GOTO	SH_LOOP

		MOVFF	DELAY_H,CNT_DELAY_H
		DECFSZ	MAIN_DELAY   	;Aantal malen
		GOTO	SH_LOOP

END_DELAY:	RETURN

;-------------------
;DDT1 = 1 sec. delay
;-------------------

DDT1:		MOVLW	0X02
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0XC4
		MOVFF	WREG,DELAY_LOC
		
		CALL	DELAY
		
		RETURN

;-----------------------------------
;DTS = Timer voor aantal maal 5 usec
;      Entry: W heeft aantal
;-----------------------------------

DTS:    	MOVFF	WREG,DTS_TIMER

DTS_LOOP:       NOP
		NOP
		NOP
		NOP
		NOP
		NOP

		DECFSZ	DTS_TIMER
		GOTO	DTS_LOOP

		RETURN
		
;-------------------------------
;L_TIMER = x sec. timer
;          Entry: W heeft aantal
;-------------------------------

L_TIMER:	MOVFF	WREG,L_CNT

		MOVLW	0X02
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0XC8
		MOVFF	WREG,DELAY_LOC
		
L_LOOP:		CALL	DELAY

		DECFSZ	L_CNT
		GOTO	L_LOOP

		RETURN
		
;--------------------------------------------
;BOUNCE_DELAY = Bounce tijd schakelaar rotary
;--------------------------------------------

BOUNCE_DELAY:	MOVLW	0X02
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0X20
		MOVFF	WREG,DELAY_LOC
		
		CALL	DELAY
		
		RETURN

;--------------------------------------------------------------------
;
;Klok routines
;
;--------------------------------------------------------------------


;----------------------------------------------------------------
;SEND_SERIAL = Schrijf RS232
;              Entry : bytes in SEND_SER_BUF in ASCII, einde = FF
;----------------------------------------------------------------

SEND_SERIAL:	LFSR	0,SEND_SER_BUF	;De source

SEND_LOOP:	MOVFF	POSTINC0,TEMP	;Testen op einde string
		MOVLW	0XFF
		CPFSEQ	TEMP            ;Ja
		GOTO	DO_CHAR         ;Nee, dus zenden
		GOTO	END_SEND_SERIAL

DO_CHAR:	MOVFF	TEMP,TXREG      ;In het zendregister

WAIT_SENDED:	BTFSS	TXSTA,TRMT      ;Send flag pollen
		GOTO	WAIT_SENDED

		GOTO	SEND_LOOP

END_SEND_SERIAL:RETURN

;-------------------------------------------------------------
;REC_SERIAL = Ontvang een string beginnend met $, eindig na CR
;             Exit: string in REC_SER_BUF
;-------------------------------------------------------------

REC_SERIAL:	BTFSS	RCSTA,OERR	;Overrun?
		GOTO	NON_OE
		BCF	RCSTA,CREN 	;Reset evt. overrun serial
		BSF	RCSTA,CREN

NON_OE:		LFSR	0,REC_SER_BUF	;Hier gaat de data heen

ST_REC_LOOP1:	CLRF	WD_TIMER_RS232H
                CLRF    WD_TIMER_RS232L

REC_LOOP1:	CALL	WD_RS232
		BTFSC	RS232_ERR
		GOTO	END_REC_S
		BTFSS	PIR1,RCIF 	;Wacht op karakter
		GOTO	REC_LOOP1

		MOVFF	RCREG,TEMP      ;Vergelijk met een $

		MOVLW	0X24
		CPFSEQ	TEMP		;Indien gevonden
		GOTO	ST_REC_LOOP1

                BCF	RCSTA,CREN 	;Reset evt. overrun
		BSF	RCSTA,CREN

		CLRF	CHAR_CNT	;Reset teller incoming characters

ST_REC_LOOP2:   CLRF	WD_TIMER_RS232H
                CLRF    WD_TIMER_RS232L
                BCF	RS232_ERR

REC_LOOP2:      CALL	WD_RS232
		BTFSC	RS232_ERR
		GOTO	END_REC_S
		BTFSS	PIR1,RCIF 	;Wacht op karakter
		GOTO	REC_LOOP2

		INCF	CHAR_CNT        ;Tel aantal karakters

		MOVFF	RCREG,TEMP      ;Bewaar even
		MOVFF	TEMP,POSTINC0   ;Sla op in tabel
		MOVLW	0X0D            ;Vergelijk met CR
		CPFSEQ	TEMP		;Indien gevonden
		GOTO    ST_REC_LOOP2

END_REC_S:	RETURN

;-------------------------------------------------
;WD_RS232 = Watchdog om 'hangs' RS232 te voorkomen
;           Exit : RS232_ERR geset bij fout
;-------------------------------------------------

WD_RS232:	MOVLW	0X09		;Maak wachttijd ca. 1.3 sec.
		CALL	DTS

		INCF	WD_TIMER_RS232L	;Wachttijd teller omhoog
		MOVLW	0X00
		CPFSEQ	WD_TIMER_RS232L
		GOTO	N_RS232_ERR
		INCF	WD_TIMER_RS232H
		MOVLW	0XFF	
		CPFSEQ	WD_TIMER_RS232H
		GOTO	N_RS232_ERR

RS_WD_ERROR:	BSF	RS232_ERR	;Fout, error flag hijsen
		GOTO	END_WD_RS

N_RS232_ERR:	BCF	RS232_ERR 	;Geen fout, reset error bit

END_WD_RS:	RETURN

;------------------------------------------------------------------
;GPRMC = Analyseer de GPRMC string en lees/converteer tijd en datum
;        Exit : Indien correct DATA_VALID =1
;------------------------------------------------------------------

GPRMC:		BCF	DATA_VALID	;Begin met data ongeldig

		MOVLW	HIGH GPRMC_HEADER;Wijs naar de header gegevens
		MOVWF	TBLPTRH
                MOVLW	LOW GPRMC_HEADER
		MOVWF	TBLPTRL

		LFSR	0,REC_SER_BUF	;De ontvangen data

		MOVLW	0X06            ;6 bytes vergelijken
		MOVFF	WREG,LOOP_CNT

CONT_COMP1:	TBLRD*+                 ;Lees header gegevens
		MOVFF	TABLAT,TEMP
		MOVFF	POSTINC0,WREG   ;Lees binnengekomen data
		CPFSEQ	TEMP            ;Vergelijk
		GOTO	END_GPRMC       ;Helaas

		DECFSZ	LOOP_CNT        ;Byte was goed, volgende
		GOTO	CONT_COMP1

		CALL	CHECKSUM	;Test de checksum

		BTFSS	CH_SUM_OK
		GOTO	END_GPRMC

START_VAL_CH:	LFSR	0,REC_SER_BUF	;Wijs weer naar begin buffer
		MOVLW	0X01            ;We gaan 1 komma zoeken
		MOVFF	WREG,LOOP_CNT

FIND_2C1:	MOVFF	POSTINC0,TEMP   ;Lees uit de buffer
		MOVLW	0X2C            ;Is dit de komma
		CPFSEQ	TEMP            ;Ja
		GOTO	FIND_2C1

		DECFSZ	LOOP_CNT
		GOTO	FIND_2C1        ;Volgende komma zoeken

		MOVFF	POSTINC0,HR_10_ASC;Tijd wegschrijven
                MOVFF	POSTINC0,HR_1_ASC
                MOVFF	POSTINC0,MIN_10_ASC
                MOVFF	POSTINC0,MIN_1_ASC
                MOVFF	POSTINC0,SEC_10_ASC
                MOVFF	POSTINC0,SEC_1_ASC

		LFSR	0,REC_SER_BUF	;Wijs weer naar begin buffer
		MOVLW	0X09            ;We gaan 9 komma's zoeken
		MOVFF	WREG,LOOP_CNT
                
FIND_2C2:	MOVFF	POSTINC0,TEMP   ;Lees uit de buffer
		MOVLW	0X2C            ;Is dit de komma
		CPFSEQ	TEMP            ;Ja
		GOTO	FIND_2C2

		DECFSZ	LOOP_CNT
		GOTO	FIND_2C2        ;Volgende komma zoeken

		MOVFF	POSTINC0,DAY_10_ASC;Datum wegschrijven
                MOVFF	POSTINC0,DAY_1_ASC
                MOVFF	POSTINC0,MONTH_10_ASC
                MOVFF	POSTINC0,MONTH_1_ASC
                MOVFF	POSTINC0,YEAR_10_ASC
                MOVFF	POSTINC0,YEAR_1_ASC

		MOVFF	HR_10_ASC,TEMP	;ASCII uren tientallen halen
		RLNCF	TEMP            ;Is altijd numeriek dus kan
		RLNCF	TEMP            ;simpel geconverteerd worden
		RLNCF	TEMP
		RLNCF	TEMP
		MOVLW	0XF0
		ANDWF	TEMP            ;TEMP heeft hoge waarde

		MOVFF	HR_1_ASC,WREG	;Lage byte
		BCF	WREG,7          ;Gewoon de MSB's strepen
		BCF	WREG,6
		BCF	WREG,5
		BCF	WREG,4

		IORWF	TEMP            ;ORren met de hoge
		MOVFF	TEMP,HR_BCD     ;en opslaan

		MOVFF	MIN_10_ASC,TEMP	;ASCII minuten tientallen halen
		RLNCF	TEMP            ;Is altijd numeriek dus kan
		RLNCF	TEMP            ;simpel geconverteerd worden
		RLNCF	TEMP
		RLNCF	TEMP
		MOVLW	0XF0
		ANDWF	TEMP            ;TEMP heeft hoge waarde

		MOVFF	MIN_1_ASC,WREG	;Lage byte
		BCF	WREG,7          ;Gewoon de MSB's strepen
		BCF	WREG,6
		BCF	WREG,5
		BCF	WREG,4

		IORWF	TEMP            ;ORren met de hoge
		MOVFF	TEMP,MIN_BCD    ;en opslaan

		MOVFF	SEC_10_ASC,TEMP	;ASCII seconden tientallen halen
		RLNCF	TEMP            ;Is altijd numeriek dus kan
		RLNCF	TEMP            ;simpel geconverteerd worden
		RLNCF	TEMP
		RLNCF	TEMP
		MOVLW	0XF0
		ANDWF	TEMP            ;TEMP heeft hoge waarde

		MOVFF	SEC_1_ASC,WREG	;Lage byte
		BCF	WREG,7          ;Gewoon de MSB's strepen
		BCF	WREG,6
		BCF	WREG,5
		BCF	WREG,4

		IORWF	TEMP            ;ORren met de hoge
		MOVFF	TEMP,SEC_BCD    ;en opslaan

		MOVFF	DAY_10_ASC,TEMP	;ASCII dagen tientallen halen
		RLNCF	TEMP            ;Is altijd numeriek dus kan
		RLNCF	TEMP            ;simpel geconverteerd worden
		RLNCF	TEMP
		RLNCF	TEMP
		MOVLW	0XF0
		ANDWF	TEMP            ;TEMP heeft hoge waarde

		MOVFF	DAY_1_ASC,WREG	;Lage byte
		BCF	WREG,7          ;Gewoon de MSB's strepen
		BCF	WREG,6
		BCF	WREG,5
		BCF	WREG,4

		IORWF	TEMP            ;ORren met de hoge
		MOVFF	TEMP,DAY_BCD    ;en opslaan

		MOVFF	MONTH_10_ASC,TEMP;ASCII maanden tientallen halen
		RLNCF	TEMP            ;Is altijd numeriek dus kan
		RLNCF	TEMP            ;simpel geconverteerd worden
		RLNCF	TEMP
		RLNCF	TEMP
		MOVLW	0XF0
		ANDWF	TEMP            ;TEMP heeft hoge waarde

		MOVFF	MONTH_1_ASC,WREG;Lage byte
		BCF	WREG,7          ;Gewoon de MSB's strepen
		BCF	WREG,6
		BCF	WREG,5
		BCF	WREG,4

		IORWF	TEMP            ;ORren met de hoge
		MOVFF	TEMP,MONTH_BCD  ;en opslaan

		MOVFF	YEAR_10_ASC,TEMP;ASCII jaren tientallen halen
		RLNCF	TEMP            ;Is altijd numeriek dus kan
		RLNCF	TEMP            ;simpel geconverteerd worden
		RLNCF	TEMP
		RLNCF	TEMP
		MOVLW	0XF0
		ANDWF	TEMP            ;TEMP heeft hoge waarde

		MOVFF	YEAR_1_ASC,WREG	;Lage byte
		BCF	WREG,7          ;Gewoon de MSB's strepen
		BCF	WREG,6
		BCF	WREG,5
		BCF	WREG,4

		IORWF	TEMP            ;ORren met de hoge
		MOVFF	TEMP,YEAR_BCD   ;en opslaan

		BSF	DATA_VALID

		CALL	CALC_OFFSET 	;Roep de zomer/wintertijd aanpassing
		
		CALL	TIME_TO_HEX

		BCF	DATE_COR	;Reset flag datum correctie

		MOVLW	0X00		;Is de GMT 0?
		CPFSEQ	GMT
		GOTO	NON_GMT0        ;Nee
		GOTO	SW_COR          ;Ja, dus geen datumaanpassing vanwege GMT zone

NON_GMT0:  	MOVFF	GMT,WREG  	;Haal GMT (1 of 2)
		ADDWF   GPS_HR_HEX      ;Tel op bij het GPS uur
		MOVLW	0X18            ;Wordt dat nu 24?
		CPFSEQ	GPS_HR_HEX
		GOTO	CH_19		;Nee, geen 24 dus check 25
		GOTO	HR_18           ;Ja, uur is 24 geworden
CH_19:		MOVLW	0X19            ;Is uur nu 25?
		CPFSEQ	GPS_HR_HEX
		GOTO	SW_COR          ;Nee, dus niets aan de hand
		GOTO	HR_19 		;Ja, uur is 25 geworden

HR_18:		MOVLW	0X00		;Uur 24 wordt 00
		MOVFF	WREG,GPS_HR_HEX
		BSF	DATE_COR        ;Datum aanpassing vereist
		GOTO	SW_COR

HR_19:		MOVLW	0X01            ;Uur 25 wordt 01
		MOVFF	WREG,GPS_HR_HEX
		BSF	DATE_COR        ;Datum aanpassing vereist

SW_COR:         MOVLW	0X01    	;Is het wintertijd?
		CPFSEQ	OFFSET_T	;Dan is OFFSET_T 1 en geen correctie vereist
		GOTO	SW_CALC         ;Helaas, correctie zomertijd toepassen
		GOTO	CALC_MIN        ;Gewoon doorgaan

SW_CALC:	INCF	GPS_HR_HEX	;Zomertijd dus uurtje erbij
		MOVLW	0X18            ;Wordt 'ie 24?
		CPFSEQ	GPS_HR_HEX
		GOTO	CALC_MIN        ;Nee
		CLRF	GPS_HR_HEX      ;Ja dus 0
		BSF	DATE_COR	;en datumcorrectie vereist

		;maak minutenteller

CALC_MIN:	CALL	TIME_TO_BCD

		MOVFF	PIC_HR_HEX,TEMP_H;Ga de minutenteller vullen
		MOVFF	PIC_MIN_HEX,TEMP_L

		CALL	MAKE_MIN_CNT

		MOVFF	TEMP_H,MIN_CNT_H;Minutenteller gevuld
		MOVFF	TEMP_L,MIN_CNT_L

		;maak echte datum
		
		CALL	CORRECT_DATE

END_GPRMC: 	RETURN

;---------------------------------------------------------
;CHECKSUM = Test de checksum van een string in REC_SER_BUF
;           Exit: CH_SUM_OK 0 bij fout, 1 bij goed
;---------------------------------------------------------

CHECKSUM:	BCF	CH_SUM_OK	;Ga uit van verkeerde checksum

		LFSR	0,REC_SER_BUF	;Wijs naar begin buffer

		CLRF	SUM             ;Hier komt de checksum

FIND_ASTERIX:	MOVFF	POSTINC0,TEMP   ;Lees een byte

		MOVLW	0X2A		;Test op een *
		CPFSEQ	TEMP
		GOTO	NEXT_F_SUM      ;Geen *, doorgaan
		GOTO	END_STRING      ;gevonden dus testen

NEXT_F_SUM:     MOVFF	TEMP,WREG	;Gelezen byte gaan Xorren
		XORWF	SUM             ;met de huidige checksum

		GOTO	FIND_ASTERIX

END_STRING:     MOVFF	POSTINC0,TEMP	;Lees de hoge byte van checksum uit tabel

		CALL	CONV_ASC	;Zet ASCII om in hex getal
		MOVFF	TEMP,CS_H       ;Omgezet, nu naar hoge nibble
		RLNCF	CS_H
                RLNCF	CS_H
                RLNCF	CS_H
                RLNCF	CS_H

                MOVFF	POSTINC0,TEMP	;Lees de lage byte van checksum uit tabel

		CALL	CONV_ASC
		MOVFF	TEMP,CS_L

		MOVFF	CS_L,WREG
		ADDWF	CS_H

                MOVFF	SUM,WREG	;De berekende checksum in WREG
                CPFSEQ	CS_H            ;Vergelijk met de gelezen checksum
                GOTO	END_CHECKSUM    ;Ongelijk

                BSF	CH_SUM_OK       ;Gelijk, dus bingo

END_CHECKSUM:	RETURN

;CONV_ASC = Hulproutine checksum
;-------------------------------

CONV_ASC:	MOVLW	HIGH CONV_TABLE	;Vreemde omzettingstabel
		MOVWF	TBLPTRH
		MOVLW	LOW CONV_TABLE
		MOVWF	TBLPTRL

		MOVLW	0X4F		;Van 39H spring je naar 41H (9 naar A)
		ANDWF	TEMP

		BCF	STATUS,C       ;Reken tabelpositie uit
		MOVFF	TEMP,WREG
		ADDWF	TBLPTRL
		BTFSC	STATUS,C
		INCF	TBLPTRH

		TBLRD*      		;Lees
		MOVFF	TABLAT,TEMP
		
		RETURN

;------------------------------------------------------------------
;CALC_OFFSET = Bereken de offset van de UCT t.b.v. zomer/wintertijd
;              Exit : OFFSET, 1 bij wintertijd en 2 bij zomertijd
;              Let op: data is geprogrammeerd t/m 2030
;------------------------------------------------------------------

CALC_OFFSET:	MOVLW	0X01  		;Zet vast op 1
		MOVFF	WREG,OFFSET_T

TEST_LEAPS:	MOVLW	0X20            ;2020
		CPFSEQ	YEAR_BCD
		GOTO	TEST_24
		GOTO	ITS_LEAP

TEST_24:	MOVLW	0X24    	;2024
		CPFSEQ	YEAR_BCD
		GOTO	TEST_28
		GOTO	ITS_LEAP
		
TEST_28:	MOVLW	0X28    	;2028
		CPFSEQ	YEAR_BCD
		GOTO	NON_LEAP

ITS_LEAP:       MOVLW	HIGH LEAP_YEAR	;Tabel met schrikkeljaar aanduiden
		MOVWF	TBLPTRH
		MOVLW	LOW LEAP_YEAR
		MOVWF	TBLPTRL
		GOTO	MONTH_LOOP
		
NON_LEAP:       MOVLW	HIGH NORM_YEAR	;Tabel met niet-schrikkeljaar aanduiden
		MOVWF	TBLPTRH
		MOVLW	LOW NORM_YEAR
		MOVWF	TBLPTRL

MONTH_LOOP:	TBLRD*			;Lees een maandnummer
		MOVFF	TABLAT,WREG
		CPFSEQ	MONTH_BCD       ;Vergelijk met huidige
		GOTO	NEXT_MONTH      ;Niet gevonden
		GOTO	MONTH_FOUND

NEXT_MONTH:	INCF	TBLPTRL		;Skip over aantal uren
		INCF	TBLPTRL
		INCF	TBLPTRL
		GOTO	MONTH_LOOP	;Volgende maand

MONTH_FOUND:	INCF	TBLPTRL
		TBLRD*+
		MOVFF	TABLAT,TOT_HRS_H
		TBLRD*
		MOVFF	TABLAT,TOT_HRS_L
		
                MOVLW	HIGH BCD_TABLE_DD;Tabel de dagen terugtellend
		MOVWF	TBLPTRH
		MOVLW	LOW BCD_TABLE_DD
		MOVWF	TBLPTRL

DAY_LOOP:	TBLRD*			;Lees een dag
		MOVFF	TABLAT,WREG
		CPFSEQ	DAY_BCD         ;Vergelijk met huidige
		GOTO	NEXT_DAY        ;Niet gevonden
		GOTO	DAY_FOUND
		
NEXT_DAY:	INCF	TBLPTRL
		INCF	TBLPTRL
		GOTO	DAY_LOOP

DAY_FOUND:	INCF	TBLPTRL
		TBLRD*                  ;Gecorrigeerde dag in TEMP
		MOVFF	TABLAT,TEMP

		MOVLW	0X18 		;Vermenigvuldig met 24
		MULWF   TEMP
		
		CLRF	STATUS		;Wis de carry

                MOVFF	PRODL,WREG	;Tel het resultaat op
		ADDWF	TOT_HRS_L       ;bij het maandtotaal
		MOVFF	PRODH,WREG
		ADDWFC	TOT_HRS_H

		CLRF	STATUS   	;Carry weer weg
		
                MOVLW	HIGH BCD_TABLE_HR
		MOVWF	TBLPTRH
		MOVLW	LOW BCD_TABLE_HR
		MOVWF	TBLPTRL
		
HR_LOOP:	TBLRD*			;Lees een uur
		MOVFF	TABLAT,WREG
		CPFSEQ	HR_BCD          ;Vergelijk met huidige
		GOTO	NEXT_HRG        ;Niet gevonden
		GOTO	HR_FOUND

NEXT_HRG:	INCF	TBLPTRL
		INCF	TBLPTRL
		GOTO	HR_LOOP

HR_FOUND:	INCF	TBLPTRL
		TBLRD*                  ;Gecorrigeerde uur in TEMP
		MOVFF	TABLAT,TEMP

		MOVFF	TEMP,WREG    	;Nu de huidige uren van
		ADDWF   TOT_HRS_L       ;de dag nog
		BNC	HRS_KNOWN
		INCF	TOT_HRS_H

HRS_KNOWN:      MOVLW	HIGH LIMITS_TABLE;De omschakelpunten per jaar
		MOVWF	TBLPTRH
		MOVLW	LOW LIMITS_TABLE
		MOVWF	TBLPTRL
		
TEST_YEAR:	TBLRD*+ 		;Lees een jaarveld
		MOVFF	TABLAT,WREG
		CPFSEQ	YEAR_BCD        ;Gelijk aan huidige?
		GOTO	NEXT_YEAR       ;Nee

		TBLRD*+                 ;Ja, dus lees de grenswaarden
		MOVFF	TABLAT,LIMD_HRS_H
                TBLRD*+
		MOVFF	TABLAT,LIMD_HRS_L
		TBLRD*+
		MOVFF	TABLAT,LIMU_HRS_H
		TBLRD*+
		MOVFF	TABLAT,LIMU_HRS_L

		GOTO	DO_CALC 	;En aan het rekenen

NEXT_YEAR:	INCF	TBLPTR          ;Skip alle overbodige waarden
		INCF	TBLPTR
		INCF	TBLPTR
		INCF	TBLPTR
		INCF	TBLPTR
		GOTO	TEST_YEAR

DO_CALC:	MOVFF	LIMD_HRS_H,WREG	;Ondergrens uren hoge waarde
		CPFSLT	TOT_HRS_H       ;kleiner?
		GOTO	COMP_D_EQ
		GOTO	END_C_O

COMP_D_EQ:	CPFSEQ	TOT_HRS_H	;Ondergrens uren hoge waarde
		GOTO	TEST_UPPER      ;gelijk?

	  	MOVFF	LIMD_HRS_L,WREG ;Test nu de lage waardes van
		CPFSLT	TOT_HRS_L       ;de ondergrens
		GOTO	TEST_UPPER
		GOTO	END_C_O

TEST_UPPER:     MOVFF	LIMU_HRS_H,WREG	;Hoge waarde bovengrens
		CPFSGT	TOT_HRS_H
		GOTO	COMP_U_EQ
		GOTO	END_C_O

COMP_U_EQ:      CPFSEQ	TOT_HRS_H 	;Hoge waarde gelijk, test lage waarde
		GOTO	SET_OFFSET	;Moet kleiner zijn, dus zomertijd

		MOVFF	LIMU_HRS_L,WREG ;Vergelijking zou eigenlijk 'groter dan
		CPFSEQ	TOT_HRS_L       ;of gelijk aan' moeten zijn; wordt hier
		GOTO	NOT_L_EQ        ;gecorrigeerd voor 'gelijk aan'
		GOTO	END_C_O

NOT_L_EQ:	CPFSGT	TOT_HRS_L       ;Test nu op groter
		GOTO	SET_OFFSET
		GOTO	END_C_O

SET_OFFSET:	MOVLW	0X02   		;02 is zomertijd; 01 is wintertijd
		MOVFF	WREG,OFFSET_T

END_C_O:	RETURN

;--------------------------------------------------------------------
;CORRECT_DATE = Bepaal de huidige datum na correctie zomer/wintertijd
;--------------------------------------------------------------------

CORRECT_DATE:   BTFSS	DATE_COR	;Moet de datum omhoog?
		GOTO	END_C_D         ;Nee
		
DO_D_CORR:      CALL	INC_DAY		;Dag omhoog, BCD waarde +1 in tabel bekijken

		MOVLW	0X20            ;Test 2020
		CPFSEQ	YEAR_BCD
		GOTO    CH_24
		GOTO	LEAP_Y          ;Schrikkeljaar

CH_24:		MOVLW	0X24            ;Test 2024
		CPFSEQ	YEAR_BCD
		GOTO	CH_28
		GOTO	LEAP_Y          ;Schrikkeljaar

CH_28:		MOVLW	0X28            ;Test 2028
		CPFSEQ	YEAR_BCD
		GOTO	NON_LEAP_Y

LEAP_Y:		MOVLW	HIGH LEAP_CORR
		MOVWF	TBLPTRH
		MOVLW	LOW LEAP_CORR
		MOVWF	TBLPTRL
		GOTO	DO_D_CORR_LOOP

NON_LEAP_Y:	MOVLW	HIGH NON_LEAP_CORR;Normaal jaar
		MOVWF	TBLPTRH
		MOVLW	LOW NON_LEAP_CORR
		MOVWF	TBLPTRL

DO_D_CORR_LOOP:	TBLRD*+			;Lees het maandnummer
		MOVFF	TABLAT,TEMP	;Zet in TEMP

		MOVLW	0XFF		;Als dat FF is dan heb ik een probleem
		CPFSEQ	TEMP
		GOTO	NON_CM          ;Gelukkig niet
		MOVLW	0X02            ;Tsja, hang je maar op met error 2
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP

NON_CM:        	MOVFF	TEMP,WREG
		CPFSEQ	MONTH_BCD
		GOTO	NEXT_MONTHC 	;Ongelijk, volgende poging
		GOTO	MONTH_FOUNDC    ;Maand gevonden

NEXT_MONTHC:	TBLRD*+               	;Over getal heen naar volgende maand
		GOTO	DO_D_CORR_LOOP

MONTH_FOUNDC:   TBLRD*     		;Lees aantal dagen per maand +1
		MOVFF	TABLAT,WREG
		CPFSEQ	DAY_BCD         ;Vergelijk met aangepaste dag
                GOTO    END_C_D  	;Ongelijk, geen probleem

		MOVLW	0X01    	;We staan weer op de 1e van de maand
		MOVFF	WREG,DAY_BCD
		CALL	INC_MONTH	;Nieuwe maand

		MOVLW	0X13         	;Maar is de maand nu 13?
		CPFSEQ	MONTH_BCD       ;Ja
		GOTO	END_C_D         ;Nee, dus geen probleem

		MOVLW	0X01          	;Dan is de maand weer 1 (gelukkig nieuwjaar)
		MOVFF	WREG,MONTH_BCD
		CALL	INC_YEAR

END_C_D:	RETURN

;INC_DAY = Hulproutine CORRECT_DATE
;-----------------------------------

INC_DAY:	MOVLW	HIGH HEX_BCD_TABLE;Tabel met volgende BCD waarde
		MOVWF	TBLPTRH
		MOVLW	LOW HEX_BCD_TABLE
		MOVWF	TBLPTRL

INC_DAY_LOOP:	TBLRD*+			;Lees tabel
		MOVFF	TABLAT,TEMP     ;Zet in TEMP
		MOVLW	0XFF            ;FF is een probleem
		CPFSEQ	TEMP
		GOTO	CH_INC_DAY

INC_DAY_ERROR:	MOVLW	0X03     	;Hang je op met errorcode 3
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP

CH_INC_DAY:	MOVFF	DAY_BCD,WREG	;Zet huidige dag in WREG
		CPFSEQ	TEMP            ;Vergelijk met de gelezen dag
		GOTO	INC_DAY_LOOP	;Ongelijk, doorzoeken

		TBLRD*+			;Gelijk, haal de volgende uit de tabel
		MOVFF	TABLAT,DAY_BCD  ;Die is eentje hoger

		RETURN
		
;INC_MONTH = Hulproutine CORRECT_DATE
;------------------------------------

INC_MONTH:	MOVLW	HIGH HEX_BCD_TABLE;Tabel met volgende BCD waarde
		MOVWF	TBLPTRH
		MOVLW	LOW HEX_BCD_TABLE
		MOVWF	TBLPTRL

INC_MONTH_LOOP:	TBLRD*+			;Lees tabel
		MOVFF	TABLAT,TEMP     ;Zet in TEMP
		MOVLW	0XFF            ;FF is een probleem
		CPFSEQ	TEMP
		GOTO	CH_INC_MONTH

INC_MONTH_ERROR:MOVLW	0X04      	;Hang je op met errorcode 4
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP

CH_INC_MONTH:	MOVFF	MONTH_BCD,WREG	;Zet huidige maand in WREG
		CPFSEQ	TEMP            ;Vergelijk met de gelezen maand
		GOTO	INC_MONTH_LOOP	;Ongelijk, doorzoeken

		TBLRD*+			;Gelijk, haal de volgende uit de tabel
		MOVFF	TABLAT,MONTH_BCD;Die is eentje hoger

		RETURN

;INC_YEAR = Hulproutine CORRECT_DATE
;-----------------------------------

INC_YEAR:	MOVLW	HIGH HEX_BCD_TABLE;Tabel met volgende BCD waarde
		MOVWF	TBLPTRH
		MOVLW	LOW HEX_BCD_TABLE
		MOVWF	TBLPTRL

INC_YEAR_LOOP:	TBLRD*+			;Lees tabel
		MOVFF	TABLAT,TEMP     ;Zet in TEMP
		MOVLW	0XFF            ;FF is een probleem (dat is 2060....)
		CPFSEQ	TEMP
		GOTO	CH_INC_YEAR

INC_YEAR_ERROR:	MOVLW	0X05      	;Hang je op met errorcode 5
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP

CH_INC_YEAR:	MOVFF	YEAR_BCD,WREG	;Zet huidige jaard in WREG
		CPFSEQ	TEMP            ;Vergelijk met het gelezen jaar
		GOTO	INC_YEAR_LOOP	;Ongelijk, doorzoeken

		TBLRD*+			;Gelijk, haal de volgende uit de tabel
		MOVFF	TABLAT,YEAR_BCD;Die is eentje hoger

		RETURN

;-------------------------------------------
;TIME_TO_HEX = Maak de GPS tijd hexadecimaal
;-------------------------------------------

TIME_TO_HEX:	MOVLW	HIGH TIME_HEX_TABLE;Tabel aanwijzen
		MOVWF	TBLPTRH
		MOVLW	LOW TIME_HEX_TABLE
		MOVWF	TBLPTRL
		
		BCF	STATUS,C	;Carry weg
		MOVFF	HR_BCD,WREG     ;BCD uur optellen
		ADDWF	TBLPTRL         ;bij TBLPTR
		BTFSC	STATUS,C
		INCF	TBLPTRH

		TBLRD*
		MOVFF	TABLAT,GPS_HR_HEX

                MOVLW	HIGH TIME_HEX_TABLE;Tabel aanwijzen
		MOVWF	TBLPTRH
		MOVLW	LOW TIME_HEX_TABLE
		MOVWF	TBLPTRL

		BCF	STATUS,C	;Carry weg
		MOVFF	MIN_BCD,WREG    ;BCD minuut optellen
		ADDWF	TBLPTRL         ;bij TBLPTR
		BTFSC	STATUS,C
		INCF	TBLPTRH

		TBLRD*
		MOVFF	TABLAT,GPS_MIN_HEX

		MOVLW	HIGH TIME_HEX_TABLE;Tabel aanwijzen
		MOVWF	TBLPTRH
		MOVLW	LOW TIME_HEX_TABLE
		MOVWF	TBLPTRL

		BCF	STATUS,C	;Carry weg
		MOVFF	SEC_BCD,WREG    ;BCD seconde optellen
		ADDWF	TBLPTRL         ;bij TBLPTR
		BTFSC	STATUS,C
		INCF	TBLPTRH

		TBLRD*
		MOVFF	TABLAT,GPS_SEC_HEX
		
		RETURN
		
;--------------------------------------
;TIME_TO_BCD = Maak de hex PIC tijd BCD
;--------------------------------------

TIME_TO_BCD:	MOVLW	HIGH HEX_BCD_TABLE
		MOVWF	TBLPTRH
		MOVLW	LOW HEX_BCD_TABLE
		MOVWF	TBLPTRL
		
		BCF	STATUS,C
		MOVFF	PIC_HR_HEX,WREG
		ADDWF	TBLPTRL
		BTFSC	STATUS,C
		INCF	TBLPTRH
		
		TBLRD*
		MOVFF	TABLAT,PIC_HR_BCD
		
	        MOVLW	HIGH HEX_BCD_TABLE
		MOVWF	TBLPTRH
		MOVLW	LOW HEX_BCD_TABLE
		MOVWF	TBLPTRL

		BCF	STATUS,C
		MOVFF	PIC_MIN_HEX,WREG
		ADDWF	TBLPTRL
		BTFSC	STATUS,C
		INCF	TBLPTRH

		TBLRD*
		MOVFF	TABLAT,PIC_MIN_BCD
		
		MOVLW	HIGH HEX_BCD_TABLE
		MOVWF	TBLPTRH
		MOVLW	LOW HEX_BCD_TABLE
		MOVWF	TBLPTRL

		BCF	STATUS,C
		MOVFF	PIC_SEC_HEX,WREG
		ADDWF	TBLPTRL
		BTFSC	STATUS,C
		INCF	TBLPTRH

		TBLRD*
		MOVFF	TABLAT,PIC_SEC_BCD
		
		RETURN

;---------------------------------------------------
;CALC_PIC_VAL = Bereken PIC tijd in secondes (P_VAL)
;---------------------------------------------------

CALC_PIC_VAL:   MOVFF	PIC_HR_HEX,P_HR_HEX;GPS opslaan in rekenvelden
		MOVFF	PIC_MIN_HEX,P_MIN_HEX
		MOVFF	PIC_SEC_HEX,P_SEC_HEX

		MOVLW	0X3C		;Vermenigvuldigfactor 60
		MULWF	P_MIN_HEX	;Minuten x 60

		MOVFF	PRODH,P_VAL_H	;Aantal secondes van de minuten
		MOVFF	PRODL,P_VAL_L

		BCF	STATUS,C	;Carry weg
		MOVFF	P_SEC_HEX,WREG  ;Haal de seconden
		ADDWF	P_VAL_L         ;Tel op bij lage seconden totaal
		BTFSC	STATUS,C        ;Carry?
		INCF	P_VAL_H         ;Ja

		MOVLW	HIGH SEC_TABLE	;Tabel met verstreken secondes
		MOVWF	TBLPTRH
		MOVLW	LOW SEC_TABLE
		MOVWF	TBLPTRL

		MOVLW	0X04  		;Tabel heeft entries van 4
		MULWF	P_HR_HEX

		BCF	STATUS,C	;Zet de pointer goed
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		TBLRD*+              	;Verstreken secondes in de
		MOVFF	TABLAT,P_VAL_U  ;reeds verstreken uren inlezen
		TBLRD*+
		MOVFF	TABLAT,TEMP_H
		TBLRD*
		MOVFF	TABLAT,TEMP_L

		BCF	STATUS,C	;Lage byte optellen
		MOVFF	TEMP_L,WREG
		ADDWF   P_VAL_L
		BTFSC	STATUS,C
		INCF	P_VAL_H

		BCF	STATUS,C	;Hoge byte ook
		MOVFF	TEMP_H,WREG
		ADDWF	P_VAL_H
		BTFSC	STATUS,C
		INCF	P_VAL_U         ;Evt. upper byte aanpassen

		RETURN

;------------------------------------------------------------
;TEST_STABILITY = Zijn er 10 opeenvolgende goede GPS strings?
;                 Exit: GPS_STABLE flag
;------------------------------------------------------------

TEST_STABILITY: BCF	GPS_STABLE 	;Begin verkeerd en bereken eerst
		   			;het GPS getal in secondes (G_VAL)
		
		MOVFF	GPS_HR_HEX,G_HR_HEX;GPS opslaan in rekenvelden
		MOVFF	GPS_MIN_HEX,G_MIN_HEX
		MOVFF	GPS_SEC_HEX,G_SEC_HEX

		MOVLW	0X3C		;Vermenigvuldigfactor 60
		MULWF	G_MIN_HEX	;Minuten x 60

		MOVFF	PRODH,G_VAL_H	;Aantal secondes van de minuten
		MOVFF	PRODL,G_VAL_L

		BCF	STATUS,C	;Carry weg
		MOVFF	G_SEC_HEX,WREG  ;Haal de seconden
		ADDWF	G_VAL_L         ;Tel op bij lage seconden totaal
		BTFSC	STATUS,C        ;Carry?
		INCF	G_VAL_H         ;Ja

		MOVLW	HIGH SEC_TABLE	;Tabel met verstreken secondes
		MOVWF	TBLPTRH
		MOVLW	LOW SEC_TABLE
		MOVWF	TBLPTRL
		
		MOVLW	0X04  		;Tabel heeft entries van 4
		MULWF	GPS_HR_HEX

		BCF	STATUS,C	;Zet de pointer goed
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH
		
		TBLRD*+              	;Verstreken secondes in de
		MOVFF	TABLAT,G_VAL_U  ;reeds verstreken uren inlezen
		TBLRD*+
		MOVFF	TABLAT,TEMP_H
		TBLRD*
		MOVFF	TABLAT,TEMP_L
		
		BCF	STATUS,C	;Lage byte optellen
		MOVFF	TEMP_L,WREG
		ADDWF   G_VAL_L
		BTFSC	STATUS,C
		INCF	G_VAL_H
		
		BCF	STATUS,C	;Hoge byte ook
		MOVFF	TEMP_H,WREG
		ADDWF	G_VAL_H
		BTFSC	STATUS,C
		INCF	G_VAL_U         ;Evt. upper byte aanpassen

UPD_TABLE:      LFSR	0,SEC_1_BUF	;Update de tabel met de 10 laatste tijden
		LFSR	1,SEC_0_BUF
		CALL	MU_4

                LFSR	0,SEC_2_BUF
		LFSR	1,SEC_1_BUF
		CALL	MU_4

		LFSR	0,SEC_3_BUF
		LFSR	1,SEC_2_BUF
		CALL	MU_4

		LFSR	0,SEC_4_BUF
		LFSR	1,SEC_3_BUF
		CALL	MU_4

		LFSR	0,SEC_5_BUF
		LFSR	1,SEC_4_BUF
		CALL	MU_4

		LFSR	0,SEC_6_BUF
		LFSR	1,SEC_5_BUF
		CALL	MU_4

		LFSR	0,SEC_7_BUF
		LFSR	1,SEC_6_BUF
		CALL	MU_4

		LFSR	0,SEC_8_BUF
		LFSR	1,SEC_7_BUF
		CALL	MU_4

		LFSR	0,SEC_9_BUF
		LFSR	1,SEC_8_BUF
		CALL	MU_4

		LFSR	0,SEC_9_BUF 	;Schrijf de laatste tijd
		MOVFF	G_VAL_U,WREG
		MOVFF	WREG,POSTINC0
		MOVFF	G_VAL_H,WREG
		MOVFF	WREG,POSTINC0
		MOVFF	G_VAL_L,WREG
		MOVFF	WREG,POSTINC0
		CLRF	WREG
		MOVFF	WREG,POSTINC0

		LFSR	0,SEC_0_BUF 	;Haal de oudste tijd
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,SUB1_U
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,SUB1_H
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,SUB1_L

                LFSR	0,SEC_9_BUF 	;Haal de nieuwste tijd
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,SUB2_U
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,SUB2_H
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,SUB2_L

		CALL	SUB_24
		
		MOVLW	0X00		;Test resultaat op 000009
		CPFSEQ	SUBRES_U
		GOTO	END_T_S

		MOVLW	0X00
		CPFSEQ	SUBRES_H
		GOTO	END_T_S

		MOVLW	0X09
		CPFSEQ	SUBRES_L
		GOTO	END_T_S

		BSF	GPS_STABLE
				
END_T_S:	RETURN

;MU_4 = Hulproutine TEST_STABILITY
;---------------------------------

MU_4:		MOVFF	POSTINC0,WREG
		MOVFF	WREG,POSTINC1

                MOVFF	POSTINC0,WREG
		MOVFF	WREG,POSTINC1
		
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,POSTINC1
		
		MOVFF	POSTINC0,WREG
		MOVFF	WREG,POSTINC1

		RETURN

;------------------------------------------------
;SUB_24 = Trek 2 24 bits getallen van elkaar af
;         Entry: SUB1_x, SUB2_x
;         Exit : Resultaat in SUBRES_x (positief)
;------------------------------------------------

SUB_24:		MOVFF	SUB2_U,WREG	;Upper byte
		CPFSGT	SUB1_U          ;S1 > S2?
		GOTO    CH_U_2          ;Nee
		GOTO	S1_GR_S2        ;Ja

CH_U_2:         MOVFF	SUB2_U,WREG
		CPFSLT	SUB1_U          ;S1 < S2?
		GOTO	CH_U_3
		GOTO	S2_GR_S1        ;Ja

CH_U_3:         MOVFF	SUB2_H,WREG	;High byte
		CPFSGT	SUB1_H          ;S1 > S2?
		GOTO    CH_U_4          ;Nee
		GOTO	S1_GR_S2        ;Ja

CH_U_4:         MOVFF	SUB2_H,WREG
		CPFSLT	SUB1_H          ;S1 < S2?
		GOTO	CH_U_5          ;Nee
		GOTO	S2_GR_S1        ;Ja

CH_U_5:         MOVFF	SUB2_L,WREG	;Lage byte
		CPFSGT	SUB1_L          ;S1 > S2?
		GOTO    CH_U_6          ;Nee
		GOTO	S1_GR_S2        ;Ja

CH_U_6:         MOVFF	SUB2_L,WREG
		CPFSLT	SUB1_L          ;S1 < S2?
		GOTO	S1_EQ_S2        ;Nee, dus gelijk
		GOTO	S2_GR_S1        ;Ja

S1_GR_S2:       BSF	STATUS,C
		MOVFF	SUB2_L,WREG
		SUBWF	SUB1_L
		BTFSS	STATUS,C
		DECF	SUB1_H
		
                BSF	STATUS,C
		MOVFF	SUB2_H,WREG
		SUBWF	SUB1_H
		BTFSS	STATUS,C
		DECF	SUB1_U
		
		BSF	STATUS,C
		MOVFF	SUB2_U,WREG
		SUBWF	SUB1_U
		
		MOVFF	SUB1_U,SUBRES_U
		MOVFF	SUB1_H,SUBRES_H
		MOVFF	SUB1_L,SUBRES_L
		
		GOTO	END_SUB_24
		
S2_GR_S1:       BSF	STATUS,C
		MOVFF	SUB1_L,WREG
		SUBWF	SUB2_L
		BTFSS	STATUS,C
		DECF	SUB2_H

                BSF	STATUS,C
		MOVFF	SUB1_H,WREG
		SUBWF	SUB2_H
		BTFSS	STATUS,C
		DECF	SUB2_U

		BSF	STATUS,C
		MOVFF	SUB1_U,WREG
		SUBWF	SUB2_U

		MOVFF	SUB2_U,SUBRES_U
		MOVFF	SUB2_H,SUBRES_H
		MOVFF	SUB2_L,SUBRES_L

		GOTO	END_SUB_24

S1_EQ_S2:       CLRF	SUBRES_U
		CLRF	SUBRES_H
		CLRF	SUBRES_L
		
END_SUB_24:	RETURN
		
;-------------------------------------------------
;SYNC_TIME = Zet de GPS tijd in de controller tijd
;-------------------------------------------------

SYNC_TIME:	MOVFF	GPS_HR_HEX,PIC_HR_HEX
		MOVFF	GPS_MIN_HEX,PIC_MIN_HEX
		MOVFF	GPS_SEC_HEX,PIC_SEC_HEX
		
		MOVLW	0X0A
		MOVFF	WREG,INT_CNT
		
		RETURN
		
;----------------------------------------------------
;CORRECTION = Stem de GPS- en de PICtijd op elkaar af
;----------------------------------------------------

CORRECTION:	BTFSC	NO_GPS		;Is er eigenlijk GPS?
		GOTO	END_CORR        ;Jammer

		BTFSC	CHECKIT		;Moet de eens per uur correctie
		GOTO	ONE_HOUR	;Ja
		GOTO	CH_DEVIATION

ONE_HOUR:	CALL	SYNC_TIME	;Zet PIC tijd op GPS tijd

		BCF	CHECKIT         ;Vlaggetje weg
		
		GOTO	END_CORR

CH_DEVIATION:   MOVFF	G_VAL_U,SUB1_U
		MOVFF	G_VAL_H,SUB1_H
		MOVFF	G_VAL_L,SUB1_L

                MOVFF	P_VAL_U,SUB2_U
		MOVFF	P_VAL_H,SUB2_H
		MOVFF	P_VAL_L,SUB2_L
		
		CALL	SUB_24		;Bereken verschil GPS en PIC tijd

		MOVLW	0X00            ;Dat moet < 00.00.04 zijn
		CPFSEQ	SUBRES_U
		GOTO    CORR_NEEDED
		
		MOVLW	0X00
		CPFSEQ	SUBRES_H
		GOTO    CORR_NEEDED

		MOVLW	0X03
		CPFSGT	SUBRES_L
		GOTO    END_CORR

CORR_NEEDED:	CALL	SYNC_TIME

END_CORR:	RETURN 

;------------------------------------------------------------------------
;
;OLED besturing
;
;------------------------------------------------------------------------
		
		
;---------------------------------
;OLED_MIN_T = Vertraging voor OLED
;---------------------------------

OLED_MIN_T:	NOP

		RETURN

;----------
;RESET_OLED
;----------

RESET_OLED:	BSF	WRITE
		BCF	I_D             ;Zet maar op instructie

		NOP
		NOP

		BSF	RESET  		;Reset ook hoog

		MOVLW	0X10
		CALL	DTS

		BCF	RESET 		;Reset de chip

		MOVLW	0X10
		CALL	DTS

		BSF	RESET

		MOVLW	0X10
		CALL	DTS
		
		MOVLW	0X21   		;Zet eindadressen
		CALL	WR_COMMAND
		MOVLW	0X00
		CALL	WR_COMMAND
		MOVLW	0X7F
		CALL	WR_COMMAND

                MOVLW	0X22
		CALL	WR_COMMAND
		MOVLW	0X00
		CALL	WR_COMMAND
		MOVLW	0X07
		CALL	WR_COMMAND

                MOVLW	0X10
		CALL	DTS

                ;----------------------------
		;Noodzakelijk bij MCOB128064T
		;Weglaten bij oudere displays

		MOVLW	0XA1
		CALL	WR_COMMAND

		;----------------------------

                MOVLW	0X10
		CALL	DTS

		MOVLW	0XC8   		;Normal scan
		CALL	WR_COMMAND

                MOVLW	0X10
		CALL	DTS

		MOVLW	0X20      	;Selecteer verticale adressering
		CALL	WR_COMMAND
		MOVLW	0X01
		CALL	WR_COMMAND
		
		MOVLW	0X10
		CALL	DTS

		MOVFF	BRIGHT,TEMP
		SWAPF	TEMP
		MOVLW	0X81        	;Stel helderheid in
		CALL	WR_COMMAND
		MOVFF	TEMP,WREG
		CALL	WR_COMMAND

                MOVLW	0X10
		CALL	DTS

		MOVLW	0XAF  		;Zet de display aan
		CALL	WR_COMMAND

		MOVLW	0X80
		CALL	DTS

		RETURN
		
;---------------------------------------
;WR_COMMAND = Schrijf commando naar OLED
;             Entry: WREG heeft commando
;---------------------------------------

WR_COMMAND:	BSF	WRITE           ;Hou write laag
		BCF	I_D             ;Instructie

		MOVFF	WREG,LATD 	;Data op de uitgang

		BCF	WRITE 		;Write laag

		BSF	WRITE           ;Write puls

		RETURN

;-----------------------------------
;WR_DATA = Schrijf data naar OLED
;             Entry: WREG heeft data
;-----------------------------------

WR_DATA:	BSF	WRITE           ;Write laag
		BSF	I_D             ;Data

		MOVFF	WREG,LATD 	;Data op de uitgang

		BCF	WRITE 		;Write laag

		BSF	WRITE           ;Write puls

		RETURN

;---------------------------------------
;ERASE_BOTH = Wis OLED RAM en shadow RAM
;---------------------------------------

ERASE_BOTH:     CALL	CLEAR_SHADOW

		CALL	WRITE_SHADOW

		RETURN

;-------------------------------------------
;WR_CHAR = Zet karakters in shadowRAM
;          Entry: TBLPTR wijst naar karakter
;-------------------------------------------

WR_CHAR:	TBLRD*+               	;Lees karakter specifieke data
		MOVFF	TABLAT,CHAR_HEIGHT
		TBLRD*+
		MOVFF	TABLAT,CHAR_WIDTH
		MOVFF	CHAR_WIDTH,CHAR_WIDTH2

		MOVFF	V_POS,OFFSET	;OFFSET afleiden uit V_POS
		MOVLW	0X07
		ANDWF	OFFSET

		CALL	SELECT_LFSR	;Adresseer de shadowRAM

CHAR_LOOP:	TBLRD*+			;Lees de opdrachtbytes
		MOVFF	TABLAT,COMM_BYTE1
                TBLRD*+
		MOVFF	TABLAT,COMM_BYTE2
		TBLRD*+
		MOVFF	TABLAT,COMM_BYTE3
		TBLRD*+
		MOVFF	TABLAT,COMM_BYTE4
		
		CALL	CONV_COMM_BYTES

		CALL	FILL_SHADOW	;Manipuleer 1 kolom in de shadowRAM

		DECFSZ	CHAR_WIDTH	;Karakter klaar?
		GOTO    NEXT_CHAR       ;Nee
		GOTO	END_WR_CHAR     ;Ja

NEXT_CHAR:	BCF	STATUS,C        ;LFSR moet naar volgende kolom
		MOVLW	0X08
		ADDWF	ADR_LFL
		BTFSC	STATUS,C
		INCF	ADR_LFH
		GOTO	CHAR_LOOP

END_WR_CHAR:	RETURN

;------------------------------------------------
;FILL_SHADOW = Manipuleer 1 kolom in de shadowRAM
;------------------------------------------------

FILL_SHADOW:	MOVFF	TBLPTRH,TPH_BU 	;Red de pointer naar de displaydata
		MOVFF	TBLPTRL,TPL_BU

            	MOVLW	HIGH TYPE_TABLE	;TYPE_TABLE aanwijzen
		MOVWF	TBLPTRH
		MOVLW	LOW TYPE_TABLE
		MOVWF	TBLPTRL

	        MOVFF	OFFSET,WREG	;In stappen van 256
	        ADDWF	TBLPTRH

	        MOVFF	CHAR_HEIGHT,TEMP;Hoogte behoeft aanpassing
		DECF	TEMP		;Loopt nl. van 1 t/m 32

		MOVLW	0X08            ;Stappen van 8
	        MULWF	TEMP
	        MOVFF	PRODL,WREG
	        ADDWF   TBLPTRL         ;Pointer staat nu goed
	        
	        TBLRD*+			;Over OFFSET heen
	        TBLRD*+                 ;Over HEIGHT heen

	        TBLRD*+
	        MOVFF	TABLAT,TYPE_BYTE1
                TBLRD*+
	        MOVFF	TABLAT,TYPE_BYTE2
	        TBLRD*+
	        MOVFF	TABLAT,TYPE_BYTE3
	        TBLRD*+
	        MOVFF	TABLAT,TYPE_BYTE4
	        TBLRD*+
	        MOVFF	TABLAT,TYPE_BYTE5
	        TBLRD*+
	        MOVFF	TABLAT,SEQ_BYTE

		MOVFF	ADR_LFH,FSR0H	;Maak LFSR0 adres
		MOVFF	ADR_LFL,FSR0L

		;Stap1

		MOVFF	INDF0,CONT_BYTE	;Lees de huidige shadowRAM locatie
		MOVFF	COMM_BYTE1,COMM_BYTE;De geshifte commandobyte
		MOVFF	TYPE_BYTE1,TYPE_BYTE;Het type

		CALL	MAKE_SH_BYTE	;Maak 1 byte in shadowRAM

		MOVFF	CONT_BYTE,POSTINC0;Schrijf weg en zet adres op volgende

		;Stap 2

		BTFSC	TYPE_BYTE2,7	;Test of kolom klaar is
		GOTO	END_F_S

                MOVFF	INDF0,CONT_BYTE	;Lees de huidige shadowRAM locatie
		MOVFF	COMM_BYTE2,COMM_BYTE;De geshifte commandobyte
		MOVFF	TYPE_BYTE2,TYPE_BYTE;Het type

		CALL	MAKE_SH_BYTE	;Maak 1 byte in shadowRAM

		MOVFF	CONT_BYTE,POSTINC0;Schrijf weg en zet adres op volgende
		
		;Stap 3

		BTFSC	TYPE_BYTE3,7	;Test of kolom klaar is
		GOTO	END_F_S

                MOVFF	INDF0,CONT_BYTE	;Lees de huidige shadowRAM locatie
		MOVFF	COMM_BYTE3,COMM_BYTE;De geshifte commandobyte
		MOVFF	TYPE_BYTE3,TYPE_BYTE;Het type

		CALL	MAKE_SH_BYTE	;Maak 1 byte in shadowRAM

		MOVFF	CONT_BYTE,POSTINC0;Schrijf weg en zet adres op volgende
		
		;Stap 4

		BTFSC	TYPE_BYTE4,7	;Test of kolom klaar is
		GOTO	END_F_S
                
		MOVFF	INDF0,CONT_BYTE	;Lees de huidige shadowRAM locatie
		MOVFF	COMM_BYTE4,COMM_BYTE;De geshifte commandobyte
		MOVFF	TYPE_BYTE4,TYPE_BYTE;Het type

		CALL	MAKE_SH_BYTE	;Maak 1 byte in shadowRAM

		MOVFF	CONT_BYTE,POSTINC0;Schrijf weg en zet adres op volgende
		
		;Stap 5

		BTFSC	TYPE_BYTE5,7	;Test of kolom klaar is
		GOTO	END_F_S

		MOVFF	INDF0,CONT_BYTE	;Lees de huidige shadowRAM locatie
		MOVFF	COMM_BYTE5,COMM_BYTE;De geshifte commandobyte
		MOVFF	TYPE_BYTE5,TYPE_BYTE;Het type

		CALL	MAKE_SH_BYTE	;Maak 1 byte in shadowRAM

		MOVFF	CONT_BYTE,POSTINC0;Schrijf weg en zet adres op volgende

END_F_S:        MOVFF	TPH_BU,TBLPTRH 	;Pointer terug
		MOVFF	TPL_BU,TBLPTRL

		RETURN

;-----------------------------------------------------------
;SELECT_LFSR = Bereid LFSR0 voor op juiste byte in shadowRAM
;              Entry: H_POS en V_POS
;              Exit: ADR_LFx
;-----------------------------------------------------------

SELECT_LFSR:	CLRF	ADR_LFL

		MOVFF	H_POS,TEMP	;H_POS redden
		MOVLW	0X08            ;Horizontale stappen van 8
		MULWF	TEMP            ;Vermenigvuldig

		MOVFF	PRODH,ADR_LFH
		MOVFF	PRODL,ADR_LFL

		INCF	ADR_LFH		;Vaste offset van 100H
		
		MOVFF	V_POS,TEMP	;V_POS delen door 7
		RRNCF	TEMP
		RRNCF	TEMP
		RRNCF	TEMP
		MOVLW	0X07
		ANDWF	TEMP
		MOVFF	TEMP,WREG
		ADDWF	ADR_LFL 	;Correcte adres

LF_FOUND:       RETURN

;---------------------------------------------------
;WRITE_SHADOW = Schrijf de shadowRAM naar de display
;---------------------------------------------------

WRITE_SHADOW:	LFSR	0,B0_0		;Startpunt shadowRAM

                MOVLW	0X20      	;Selecteer verticale adressering
		CALL	WR_COMMAND
		MOVLW	0X01
		CALL	WR_COMMAND
		
		MOVLW	0XB0		;Page nr
		CALL	WR_COMMAND
		
		MOVLW	0X00      	;Kolomnr
		CALL	WR_COMMAND
		MOVLW	0X10
		CALL	WR_COMMAND

		CALL	WR_128
		CALL	WR_128
		CALL	WR_128
		CALL	WR_128
		
		CALL	WR_128
		CALL	WR_128
		CALL	WR_128
		CALL	WR_128

		RETURN

;WR_128 = Hulproutine WRITE_SHADOW
;---------------------------------

WR_128:		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA
		MOVFF	POSTINC0,WREG
		CALL	WR_DATA

		RETURN

;---------------------------------------------------
;MAKE_SH_BYTE = Manipuleer de bits van 1 shadowbyte
;               Entry: DB = displaybyte in shadowRAM
;                      TYPE = type manipulatie
;               Exit: DB is aangepast
;---------------------------------------------------

MAKE_SH_BYTE:	MOVLW	0X0F		;Speciale bytes?
		CPFSGT	TYPE_BYTE       ;Ja
		GOTO	B0_0E_BYTES     ;Nee

		MOVLW	0X10
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M11
		GOTO	HANDLE_M10

CH_M11:         MOVLW	0X11
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M12
		GOTO	HANDLE_M11

CH_M12:         MOVLW	0X12
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M13
		GOTO	HANDLE_M12

CH_M13:         MOVLW	0X13
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M14
		GOTO	HANDLE_M13

CH_M14:         MOVLW	0X14
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M15
		GOTO	HANDLE_M14

CH_M15:         MOVLW	0X15
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M20
		GOTO	HANDLE_M15

CH_M20:         MOVLW	0X20
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M21
		GOTO	HANDLE_M20

CH_M21:         MOVLW	0X21
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M22
		GOTO	HANDLE_M21

CH_M22:         MOVLW	0X22
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M23
		GOTO	HANDLE_M22

CH_M23:         MOVLW	0X23
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M24
		GOTO	HANDLE_M23

CH_M24:         MOVLW	0X24
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M30
		GOTO	HANDLE_M24

CH_M30:         MOVLW	0X30
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M31
		GOTO	HANDLE_M20

CH_M31:         MOVLW	0X31
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M32
		GOTO	HANDLE_M31

CH_M32:         MOVLW	0X32
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M33
		GOTO	HANDLE_M32

CH_M33:         MOVLW	0X33
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M40
		GOTO	HANDLE_M33

CH_M40:         MOVLW	0X40
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M41
		GOTO	HANDLE_M40

CH_M41:         MOVLW	0X41
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M42
		GOTO	HANDLE_M41
		
CH_M42:         MOVLW	0X42
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M50
		GOTO	HANDLE_M42
		
CH_M50:         MOVLW	0X50
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M51
		GOTO	HANDLE_M50
		
CH_M51:         MOVLW	0X51
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M60
		GOTO	HANDLE_M51
		
CH_M60:         MOVLW	0X60
		CPFSEQ	TYPE_BYTE
		GOTO	E10_60_ERR
		GOTO	HANDLE_M51
		
B0_0E_BYTES:	MOVLW	0X00
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M01
		GOTO	HANDLE_M00
		
CH_M01:         MOVLW	0X01
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M02
		GOTO	HANDLE_M01
		
CH_M02:         MOVLW	0X02
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M03
		GOTO	HANDLE_M02
		
CH_M03:         MOVLW	0X03
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M04
		GOTO	HANDLE_M03
		
CH_M04:         MOVLW	0X04
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M05
		GOTO	HANDLE_M04
		
CH_M05:         MOVLW	0X05
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M06
		GOTO	HANDLE_M05
		
CH_M06:         MOVLW	0X06
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M07
		GOTO	HANDLE_M06
		
CH_M07:         MOVLW	0X07
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M08
		GOTO	HANDLE_M07
		
CH_M08:         MOVLW	0X08
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M09
		GOTO	HANDLE_M08

CH_M09:         MOVLW	0X09
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M0A
		GOTO	HANDLE_M09

CH_M0A:         MOVLW	0X0A
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M0B
		GOTO	HANDLE_M0A
		
CH_M0B:         MOVLW	0X0B
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M0C
		GOTO	HANDLE_M0B
		
CH_M0C:         MOVLW	0X0C
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M0D
		GOTO	HANDLE_M0C

CH_M0D:         MOVLW	0X0D
		CPFSEQ	TYPE_BYTE
		GOTO	CH_M0E
		GOTO	HANDLE_M0D
		
CH_M0E:         MOVLW	0X0E
		CPFSEQ	TYPE_BYTE
		GOTO	E00_0E_ERR
		GOTO	HANDLE_M0E

E10_60_ERR:	MOVLW	0X06
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP
		
E00_0E_ERR:	MOVLW	0X07
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP
		
END_MSHB:	RETURN

;BIT_HANDLERS
;------------

HANDLE_M00:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0
		
		BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M01:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB
		
HANDLE_M02:     BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M03:     BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M04:     BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M05:     BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M06:     BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M07:     BCF	CONT_BYTE,7
		BTFSC	COMM_BYTE,7
		BSF	CONT_BYTE,7

		GOTO	END_MSHB

HANDLE_M08:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0

		GOTO	END_MSHB

HANDLE_M09:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0

                BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1

		GOTO	END_MSHB

HANDLE_M0A:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0
		
		BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		GOTO	END_MSHB

HANDLE_M0B:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0
		
		BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		GOTO	END_MSHB

HANDLE_M0C:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0
		
		BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3
		
		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		GOTO	END_MSHB

HANDLE_M0D:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0
		
		BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3
		
		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4
		
		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		GOTO	END_MSHB

HANDLE_M0E:     BCF	CONT_BYTE,0
		BTFSC	COMM_BYTE,0
		BSF	CONT_BYTE,0
		
		BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3
		
		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4
		
		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5
		
		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB
		
HANDLE_M10:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1

		GOTO	END_MSHB
		
HANDLE_M11:     BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		GOTO	END_MSHB

HANDLE_M12:     BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		GOTO	END_MSHB

HANDLE_M13:     BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		GOTO	END_MSHB

HANDLE_M14:     BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		GOTO	END_MSHB

HANDLE_M15:     BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB
		
HANDLE_M20:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		GOTO	END_MSHB

HANDLE_M21:     BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		GOTO	END_MSHB

HANDLE_M22:     BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		GOTO	END_MSHB

HANDLE_M23:     BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		GOTO	END_MSHB

HANDLE_M24:     BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB
		
HANDLE_M30:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
                BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		GOTO	END_MSHB

HANDLE_M31:     BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

                BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		GOTO	END_MSHB

HANDLE_M32:     BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

                BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		GOTO	END_MSHB

HANDLE_M33:     BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

                BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB
		
HANDLE_M40:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		GOTO	END_MSHB

HANDLE_M41:     BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		GOTO	END_MSHB

HANDLE_M42:     BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB
		
HANDLE_M50:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4
		
		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		GOTO	END_MSHB

HANDLE_M51:     BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2

		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB
		
HANDLE_M60:     BCF	CONT_BYTE,1
		BTFSC	COMM_BYTE,1
		BSF	CONT_BYTE,1
		
		BCF	CONT_BYTE,2
		BTFSC	COMM_BYTE,2
		BSF	CONT_BYTE,2
		
		BCF	CONT_BYTE,3
		BTFSC	COMM_BYTE,3
		BSF	CONT_BYTE,3

		BCF	CONT_BYTE,4
		BTFSC	COMM_BYTE,4
		BSF	CONT_BYTE,4

		BCF	CONT_BYTE,5
		BTFSC	COMM_BYTE,5
		BSF	CONT_BYTE,5

		BCF	CONT_BYTE,6
		BTFSC	COMM_BYTE,6
		BSF	CONT_BYTE,6

		GOTO	END_MSHB

;--------------------------------------------------------
;CONV_COMM_BYTES = 40 bit shiftregister met de COMM_BYTES
;--------------------------------------------------------

CONV_COMM_BYTES:CLRF	COMM_BYTE5

		MOVLW	0X00            ;OFFSET 0 behoeft geen shift
		CPFSEQ	OFFSET
		GOTO	DO_SHIFT
		GOTO	END_CCB

DO_SHIFT:	MOVFF	OFFSET,LOOP_CNT	;Aantal malen shiften
		
SH40_LOOP:	RLNCF	COMM_BYTE5
		
		BCF	COMM_BYTE5,0
		BTFSC	COMM_BYTE4,7
		BSF	COMM_BYTE5,0

                RLNCF	COMM_BYTE4

		BCF	COMM_BYTE4,0
		BTFSC	COMM_BYTE3,7
		BSF	COMM_BYTE4,0
		
		RLNCF	COMM_BYTE3

		BCF	COMM_BYTE3,0
		BTFSC	COMM_BYTE2,7
		BSF	COMM_BYTE3,0
		
		RLNCF	COMM_BYTE2

		BCF	COMM_BYTE2,0
		BTFSC	COMM_BYTE1,7
		BSF	COMM_BYTE2,0
		
		RLNCF	COMM_BYTE1
		BCF	COMM_BYTE1,0
		
		DECFSZ	LOOP_CNT
		GOTO	SH40_LOOP

END_CCB:	RETURN

;--------------------------------------------------------------------------
;DIRECT_WRITE = Schrijf karakters, directe mode
;               Entry: H_POS (0 .. 7F)
;		       V_POS (0 .. 3F, wordt teruggerekend naar PAGE begin)
;		       TBLPTR wijst naar tabel
;--------------------------------------------------------------------------

DIRECT_WRITE:   TBLRD*+                 ;Lees specifieke karakter data
		MOVFF	TABLAT,CHAR_HEIGHT
		TBLRD*+
		MOVFF	TABLAT,CHAR_WIDTH
		
CHAR_LOOPD:	CALL	MAKE_COL_PAGE	;Leidt colom en page af

		TBLRD*+                 ;Lees de 4 opdracht bytes
		MOVFF	TABLAT,COMM_BYTE1
                TBLRD*+
		MOVFF	TABLAT,COMM_BYTE2
		TBLRD*+
		MOVFF	TABLAT,COMM_BYTE3
		TBLRD*+
		MOVFF	TABLAT,COMM_BYTE4

                MOVFF	PAGE_NR,WREG	;Schrijf pagenr
		CALL	WR_COMMAND

	        MOVFF	COL_L,WREG	;Schrijf juiste kolom weg
		CALL	WR_COMMAND
		MOVFF	COL_H,WREG
		CALL	WR_COMMAND

		MOVFF	COMM_BYTE1,WREG
		CALL	WR_DATA
		
		MOVLW	0X08		;Is karakterhoogte <8?
		CPFSLT	CHAR_HEIGHT
		GOTO    CB2             ;Nee
		GOTO	A_CB            ;Ja

CB2:		MOVFF	COMM_BYTE2,WREG
		CALL	WR_DATA
		
		MOVLW	0X10		;Is karakterhoogte <16?
		CPFSLT	CHAR_HEIGHT
		GOTO    CB_3            ;Nee
		GOTO	A_CB            ;Ja

CB_3:		MOVFF	COMM_BYTE3,WREG
		CALL	WR_DATA

		MOVLW	0X18		;Is karakterhoogte <24?
		CPFSLT	CHAR_HEIGHT
		GOTO    CB4             ;Nee
		GOTO	A_CB            ;Ja

CB4:            MOVFF	COMM_BYTE4,WREG
		CALL	WR_DATA

A_CB:		DECFSZ	CHAR_WIDTH	;Karakter(s) klaar?
		GOTO	NEXT_C          ;Nee
		GOTO	END_W_C         ;Ja

NEXT_C:		INCF	H_POS
		GOTO    CHAR_LOOPD

END_W_C:	RETURN

;-------------------------------------
;MAKE_COL_PAGE = Bereken kolom, pagina
;                Entry: H_POS, V_POS
;		 Exit COLOM_x, PAGE_NR
;-------------------------------------

MAKE_COL_PAGE:	MOVFF	TBLPTRH,TPH_BU	;Is nog nodig
		MOVFF	TBLPTRL,TPL_BU

		MOVFF	H_POS,COL_NR	;Moet worden omgezet

		MOVLW	HIGH COL_TABLE  ;Via een tabel
		MOVWF	TBLPTRH
		MOVLW	LOW COL_TABLE
		MOVWF	TBLPTRL

		MOVLW	0X02
		MULWF	COL_NR

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		TBLRD*+
		MOVFF	TABLAT,COL_H
		TBLRD*+
		MOVFF	TABLAT,COL_L
		
		MOVLW	0XB0
		MOVWF	PAGE_NR

		MOVFF	V_POS,TEMP	;V_POS delen door 7
		RRNCF	TEMP
		RRNCF	TEMP
		RRNCF	TEMP
		MOVLW	0X07
		ANDWF	TEMP
		MOVFF	TEMP,WREG
		ADDWF	PAGE_NR 	;Correcte page

		MOVFF	TPH_BU,TBLPTRH 	;Haal terug
		MOVFF	TPL_BU,TBLPTRL

		RETURN
		
;---------------------------------------------------------
;TEXT = Schrijf een tekst in de shadowRAM
;       Entry : H_POS, V_POS geset
;		TBLPTR wijst naar tekst
;               SHOW = 1 schrijf de shadowRAM (toon tekst)
;---------------------------------------------------------

TEXT:		LFSR	1,TXT_AREA	;Gebruik LFSR1 als pointer

F_A_LOOP:	TBLRD*+                 ;Lees uit het tekstveld
		MOVFF	TABLAT,TEMP
		MOVLW	0XFF            ;FF is het laatste karakter
		CPFSEQ	TEMP
		GOTO	WR_AREA
		GOTO	FILL_AR_END     ;Klaar

WR_AREA:        MOVFF	TEMP,POSTINC1   ;Sla karakter op
		GOTO	F_A_LOOP

FILL_AR_END:    MOVLW	0X00 		;Schrijf eindpointer, als een 0 want dan
		MOVFF	WREG,POSTINC1   ;voorkom je ellende bij teksten met oneven lengte
		
		LFSR	1,TXT_AREA

CHAR_L:		MOVLW	HIGH LETTER_TABLE
		MOVWF	TBLPTRH
		MOVLW	LOW LETTER_TABLE
		MOVWF	TBLPTRL

		MOVFF	POSTINC1,TEMP	;Lees een karakter

		MOVLW	0X00 		;Is het de eindpointer?
		CPFSEQ	TEMP
		GOTO	ITS_CH          ;Nee
		GOTO	END_TEXT        ;Ja

ITS_CH:         MOVLW	0X20 		;We beginnen op ASCII20
		BSF	STATUS,C
		SUBWF	TEMP

		MOVLW	0X26		;Tabel heeft stappen van 9 x 4 +2 =38
		MULWF	TEMP

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		CALL	WR_CHAR		;Schrijf in shadowRAM

		MOVFF	CHAR_WIDTH2,WREG;Bepaal positie volgend karakter
		ADDWF	H_POS

		INCF	H_POS         	;Ruimte is 2 pixels
		INCF	H_POS

		GOTO    CHAR_L

END_TEXT:	BTFSC	SHOW		;Tonen?
		CALL	WRITE_SHADOW    ;Ja

		RETURN
		
;-------------------------------------------------------------
;TEXT_LFS = Schrijf een tekst in de shadowRAM vanuit RAM
;           Entry : H_POS, V_POS geset
;		    LFSR0 wijst naar tekst, 1e byte is lengte
;                   SHOW = 1 schrijf de shadowRAM (toon tekst)
;-------------------------------------------------------------

TEXT_LFS:	CLRF	WREG
		CALL	SAVE_LFS

		LFSR	1,TXT_AREA	;Gebruik LFSR1 als pointer

		MOVFF	POSTINC0,LOOP_CNT;Aantal bytes

F_A_LOOPL:	MOVFF	POSTINC0,POSTINC1;Schrijf naar tekst area
		DECFSZ	LOOP_CNT
		GOTO	F_A_LOOPL

		LFSR	1,TXT_AREA

CHAR_LL:	MOVLW	HIGH LETTER_TABLE
		MOVWF	TBLPTRH
		MOVLW	LOW LETTER_TABLE
		MOVWF	TBLPTRL

		MOVFF	POSTINC1,TEMP	;Lees een karakter

		MOVLW	0X00 		;Is het de eindpointer?
		CPFSEQ	TEMP
		GOTO	ITS_CHL         ;Nee
		GOTO	END_TEXTL       ;Ja

ITS_CHL:        MOVLW	0X20 		;We beginnen op ASCII20
		BSF	STATUS,C
		SUBWF	TEMP

		MOVLW	0X26		;Tabel heeft stappen van 9 x 4 +2 =38
		MULWF	TEMP

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		CALL	WR_CHAR		;Schrijf in shadowRAM

		MOVFF	CHAR_WIDTH2,WREG;Bepaal positie volgend karakter
		ADDWF	H_POS

		INCF	H_POS         	;Ruimte is 2 pixels
		INCF	H_POS

		GOTO    CHAR_LL

END_TEXTL:	BTFSC	SHOW		;Tonen?
		CALL	WRITE_SHADOW    ;Ja

		SETF	WREG
		CALL	SAVE_LFS

		RETURN
		
;--------------------------------
;CLEAR_SHADOW = Wis de shadow RAM
;--------------------------------

CLEAR_SHADOW:	LFSR	0,B0_0		;Startpunt shadowRAM

		MOVLW	0X04            ;In 4 slagen van 256
		MOVFF	WREG,GEN_CNT

CL_4:		MOVLW	0X00            ;256 bytes te wissen
		MOVFF	WREG,LOOP_CNT

CL_256:		CLRF	POSTINC0        ;Wist u maar
		DECFSZ	LOOP_CNT
		GOTO	CL_256

		DECFSZ	GEN_CNT         ;En dat 4x
		GOTO	CL_4

		RETURN
		
;-----------------------------------------
;CLEAR_PART = Wis bovenste blok shadow RAM
;-----------------------------------------

CLEAR_PART:	LFSR	0,B0_0		;Startpunt shadowRAM

		MOVLW	0X00            ;256 bytes te wissen
		MOVFF	WREG,LOOP_CNT

CL_P256:	CLRF	POSTINC0        ;Wist u maar
		DECFSZ	LOOP_CNT
		GOTO	CL_P256

		RETURN

;------------------------------------------------------------------
;
;Timer routines
;
;------------------------------------------------------------------


;----------------------------------------------------------
;MAKE_MIN_CNT = Maak de minutenteller uit hex waarde
;               Entry: TEMP_H heeft hex waarde uren
;                      TEMP_L heeft hex waarde minuten
;               Exit : TEMP_H heeft hoge byte minutenteller
;                      TEMP_L heeft lage byte minutenteller
;----------------------------------------------------------

MAKE_MIN_CNT:	CLRF	PRODH
		CLRF	PRODL

		MOVLW	0X3C  		;Hoge byte met 60 vermenigvuldigen
		MULWF	TEMP_H
		
		BCF	STATUS,C
		MOVFF	TEMP_L,WREG	;Lage byte in W
		ADDWF	PRODL		;Tel op
		BTFSC	STATUS,C        ;Carry?
		INCF	PRODH           ;Ja

		MOVFF	PRODH,TEMP_H    ;Resultaat terug
		MOVFF	PRODL,TEMP_L

		RETURN
		
;-------------------------------------------------------
;CLOCK_SCREEN = Schrijf tekst bevestiging dat klok loopt
;-------------------------------------------------------

CLOCK_SCREEN:	BCF	INTCON,GIEH	;Geen interrupts

		CALL	ERASE_BOTH

		MOVLW	0X00    	;Vraag of klok loopt
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

		MOVLW	HIGH ASK_CL1_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_CL1_TXT
		MOVWF	TBLPTRL

		CALL	TEXT

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

		MOVLW	HIGH ASK_CL2_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_CL2_TXT
		MOVWF	TBLPTRL

		BSF	SHOW
		CALL	TEXT
		
KR_T1:		CALL	REC_SERIAL

		CALL	GPRMC

		BTFSS	DATA_VALID
		GOTO	KR_T1
		
		MOVFF	GPS_HR_HEX,PIC_HR_HEX
		MOVFF	GPS_MIN_HEX,PIC_MIN_HEX 

		MOVLW	0X30		;Toon de tijd
		MOVFF	WREG,H_POS
		MOVLW	0X28
		MOVFF	WREG,V_POS

		CALL	SHOW_TIME

		CALL	WRITE_SHADOW

CH_CONF:        CALL	FORCE_BUTTON
		BTFSC   BUT_FORCED
		GOTO	ITS_FORCED

		BTFSC	ROT_SW
		GOTO	KR_T1

ITS_FORCED:	CALL	ERASE_BOTH      ;Scherm leeg

                MOVLW	0X08		;Kondig opstarten aan
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

		MOVLW	HIGH RUNNING_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW RUNNING_TXT
		MOVWF	TBLPTRL

		BSF	SHOW
		CALL	TEXT
		BCF	SHOW

WFR:		BTFSS	ROT_SW          ;Wacht loslaten toets
		GOTO	WFR

		CALL 	DDT1
		CALL	DDT1
		CALL	DDT1

		BCF	D_E_SHOWN	;Hopelijk geen errortekst tonen

KR_T2:		CALL	REC_SERIAL

		BTG	SERV_LED

		CALL	GPRMC

		BTFSS	DATA_VALID
		GOTO	KR_T2

		CALL	CHECK_DATE

		BTFSS	DATE_WRONG
		GOTO	DATE_OK

		BSF	D_E_SHOWN 	;Errortekst tonen

                MOVLW	0X06		;Wacht op goede string met datum
		MOVFF	WREG,H_POS
		MOVLW	0X28
		MOVFF	WREG,V_POS

		MOVLW	HIGH DATE_ERR_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW DATE_ERR_TXT
		MOVWF	TBLPTRL

		BSF	SHOW

		CALL	TEXT

		GOTO	KR_T2

DATE_OK:	BTFSS	D_E_SHOWN
		GOTO 	CST

		CALL	ERASE_BOTH

		MOVLW	0X08		;Kondig opstarten aan
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

		MOVLW	HIGH RUNNING_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW RUNNING_TXT
		MOVWF	TBLPTRL

		BSF	SHOW
		CALL	TEXT
		BCF	SHOW

CST:		CALL    SYNC_TIME
		
		CALL	MAKE_MIN_CNT

                MOVFF	MIN_CNT_H,T_POINT_H;Actuele tijd
		MOVFF	MIN_CNT_L,T_POINT_L
		CALL	TIME_TO_RAM     ;Bereken windowRAM lokatie
		MOVFF	INDF2,REL_BYTE	;Haal actuele opdracht uit windowRAM
		CALL	CALC_PIC_VAL	;Bepaal aantal secondes PIC tijd
		
		CALL	WRITE_IO

END_CONFS:	RETURN

;------------------------------------------------
;SHOW_BEAM = Toon balk in max. 15 blokjes
;            Entry: BEAM_LENGTH is aantal blokjes
;------------------------------------------------

SHOW_BEAM:      MOVLW	0X08		;Fixed positie
		MOVFF	WREG,H_POS
		MOVLW	0X28
		MOVFF	WREG,V_POS

		MOVLW	0X0F            ;15 blokjes max.
		MOVFF	WREG,GEN_CNT

ERA_BEAM:	MOVLW	HIGH NO_BLOCK   ;Eerst de regel wissen
		MOVWF	TBLPTRH
		MOVLW	LOW NO_BLOCK
		MOVWF	TBLPTRL

		CALL	DIRECT_WRITE

		DECFSZ	GEN_CNT
		GOTO	ERA_BEAM
		
		MOVLW	0X00   		;Check voor einde
		CPFSEQ	BEAM_LENGTH
		GOTO	WR_B
		GOTO	END_SHOW_B

WR_B:		MOVFF	BEAM_LENGTH,GEN_CNT;Aantal blokjes

	        MOVLW	0X08
		MOVFF	WREG,H_POS
		MOVLW	0X28
		MOVFF	WREG,V_POS

WR_BEAM:	MOVLW	HIGH BLOCK 	;Schrijft u maar
		MOVWF	TBLPTRH
		MOVLW	LOW BLOCK
		MOVWF	TBLPTRL

		CALL	DIRECT_WRITE

		DECFSZ	GEN_CNT
		GOTO	WR_BEAM

END_SHOW_B:	RETURN

;-------------------------------------
;STATUS_SCREEN = Toon status uitgangen
;-------------------------------------

STATUS_SCREEN:	CLRF	WREG
		CALL	SAVE_LFS
		
		BTFSS	ROT_SW      	;Schakelaar?
		GOTO	INPUT_SCREEN

		BCF	SHOW

		MOVLW	0X00           ;Datum
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

		CALL	SHOW_DATE

		MOVLW	0X58 		;Tijd
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

		CALL	SHOW_TIME

		MOVLW	HIGH T_LINE
		MOVWF	TBLPTRH
		MOVLW	LOW T_LINE
		MOVWF	TBLPTRL

                MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X20
		MOVFF   WREG,V_POS

		CALL	TEXT
		
		BTFSS	ROT_SW      	;Schakelaar?
		GOTO	INPUT_SCREEN

		LFSR	0,RAM_TEXT
		
		MOVLW	0X12
		MOVFF	WREG,POSTINC0	;18 karakters gaan schrijven
		
		MOVFF	MODE_T1B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH1_NOT_ACTL	;Timer 1 niet actief
		GOTO	CH1_ACTL        ;Timer 1 wel actief
		
CH1_NOT_ACTL	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_1		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,0    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS1_ON          ;Aan
		GOTO	LS1_OFF         ;Uit

LS1_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH2L

LS1_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH2L

NA_1:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH2L            ;Volgende

CH1_ACTL:	BTFSC	REL_STATUS,0	;Relais aan of uit?
		GOTO    CH1_ONL         ;Aan
		GOTO    CH1_OFFL        ;Uit

CH1_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH2L

CH1_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH2L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T2B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH2_NOT_ACTL	;Timer 2 niet actief
		GOTO	CH2_ACTL        ;Timer 2 wel actief

CH2_NOT_ACTL:	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_2		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,1    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS2_ON          ;Aan
		GOTO	LS2_OFF         ;Uit

LS2_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH3L

LS2_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH3L

NA_2:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH3L            ;Volgende

CH2_ACTL:	BTFSC	REL_STATUS,1	;Relais aan of uit?
		GOTO    CH2_ONL         ;Aan
		GOTO    CH2_OFFL        ;Uit

CH2_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH3L

CH2_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH3L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T3B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH3_NOT_ACTL	;Timer 3 niet actief
		GOTO	CH3_ACTL        ;Timer 3 wel actief

CH3_NOT_ACTL	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_3		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,2    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS3_ON          ;Aan
		GOTO	LS3_OFF         ;Uit

LS3_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH4L

LS3_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH4L

NA_3:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH4L            ;Volgende

CH3_ACTL:	BTFSC	REL_STATUS,2	;Relais aan of uit?
		GOTO    CH3_ONL         ;Aan
		GOTO    CH3_OFFL        ;Uit

CH3_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH4L

CH3_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH4L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T4B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH4_NOT_ACTL	;Timer 4 niet actief
		GOTO	CH4_ACTL        ;Timer 4 wel actief

CH4_NOT_ACTL	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_4		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,3    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS4_ON          ;Aan
		GOTO	LS4_OFF         ;Uit

LS4_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH5L

LS4_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH5L

NA_4:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH5L            ;Volgende

CH4_ACTL:	BTFSC	REL_STATUS,3	;Relais aan of uit?
		GOTO    CH4_ONL         ;Aan
		GOTO    CH4_OFFL        ;Uit

CH4_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH5L

CH4_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH5L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T5B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH5_NOT_ACTL	;Timer 5 niet actief
		GOTO	CH5_ACTL        ;Timer 5 wel actief

CH5_NOT_ACTL	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_5		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,4    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS5_ON          ;Aan
		GOTO	LS5_OFF         ;Uit

LS5_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH6L

LS5_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH6L

NA_5:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH6L            ;Volgende

CH5_ACTL:	BTFSC	REL_STATUS,4	;Relais aan of uit?
		GOTO    CH5_ONL         ;Aan
		GOTO    CH5_OFFL        ;Uit

CH5_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH6L

CH5_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH6L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T6B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH6_NOT_ACTL	;Timer 6 niet actief
		GOTO	CH6_ACTL        ;Timer 6 wel actief

CH6_NOT_ACTL	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_6		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,5    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS6_ON          ;Aan
		GOTO	LS6_OFF         ;Uit

LS6_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH7L

LS6_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH7L

NA_6:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH7L            ;Volgende

CH6_ACTL:	BTFSC	REL_STATUS,5	;Relais aan of uit?
		GOTO    CH6_ONL         ;Aan
		GOTO    CH6_OFFL        ;Uit

CH6_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH7L

CH6_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH7L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T7B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH7_NOT_ACTL	;Timer 7 niet actief
		GOTO	CH7_ACTL        ;Timer 7 wel actief

CH7_NOT_ACTL	BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_7		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,6    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS7_ON          ;Aan
		GOTO	LS7_OFF         ;Uit

LS7_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH8L

LS7_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CH8L

NA_7:		MOVLW	0X5F		;Kruisje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CH8L            ;Volgende

CH7_ACTL:	BTFSC	REL_STATUS,6	;Relais aan of uit?
		GOTO    CH7_ONL         ;Aan
		GOTO    CH7_OFFL        ;Uit

CH7_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CH8L

CH7_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0

CH8L:		MOVLW	0X20         	;Spatie
		MOVFF	WREG,POSTINC0

		MOVFF	MODE_T8B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH8_NOT_ACTL	;Timer 8 niet actief
		GOTO	CH8_ACTL        ;Timer 8 wel actief

CH8_NOT_ACTL    BTFSS	TEMP,4		;Geen mode, wel lichtsensor?
		GOTO	NA_8		;Nee, uitgeschakeld

		BTFSC	REL_STATUS,7    ;Ja, lichtsensor geprogrammeerd
		GOTO	LS8_ON          ;Aan
		GOTO	LS8_OFF         ;Uit

LS8_ON:		MOVLW	0X61            ;Letter a voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CHEL

LS8_OFF:        MOVLW	0X75            ;Letter u voor uit
		MOVFF	WREG,POSTINC0
		GOTO	CHEL

NA_8:		MOVLW	0X5F		;Streepje voor niet actief
		MOVFF	WREG,POSTINC0
		GOTO	CHEL            ;Einde

CH8_ACTL:	BTFSC	REL_STATUS,7	;Relais aan of uit?
		GOTO    CH8_ONL         ;Aan
		GOTO    CH8_OFFL        ;Uit

CH8_ONL:	MOVLW	0X31            ;1 voor aan
		MOVFF	WREG,POSTINC0
		GOTO	CHEL

CH8_OFFL:	MOVLW	0X30            ;0 voor uit
		MOVFF	WREG,POSTINC0
		
CHEL:		MOVLW	0X00		;Endpointer
		MOVFF	WREG,POSTINC0
		MOVFF	WREG,POSTINC0

		MOVLW	0X10
		MOVFF	WREG,H_POS
		MOVLW	0X30
		MOVFF   WREG,V_POS
		
		BSF	SHOW
		LFSR	0,RAM_TEXT
		CALL	TEXT_LFS
		
		BTFSS	ROT_SW      	;Schakelaar?
		GOTO	INPUT_SCREEN

END_SS:		SETF	WREG
		CALL	SAVE_LFS
		
		RETURN

;------------------------------------------
;INPUT_SCREEN = Ga de instellingen opvragen
;------------------------------------------

INPUT_SCREEN:   BCF	INTCON,GIEH	;Geen interrupts meer

		CLRF	LATB		;Alles uit

		CALL	ERASE_BOTH

		MOVLW	0X01
		CALL	L_TIMER

CBS1:		CALL	BUILT_SCR1	;Bouw eerste scherm op

		MOVLW	0X00
		MOVFF	WREG,S_POS
		MOVFF	WREG,LOWER_LIM

IS1_LOOP:	BCF	SCREEN_FLAG	;Kies scherm 1
		CALL	U_S_ERASE	;Wis de underscores

		CALL	POS_TO_VAR	;Vertaal positie in variabelen

		MOVLW	0X14  		;Lopen we uit scherm 1?
		CPFSEQ	S_POS
		GOTO	KL1             ;Nog niet
		GOTO	CBS2            ;Yip

KL1:		CALL	WR_U_S		;Schrijf de underscore

		GOTO	ROT_QUEST

CBS2:           CALL	BUILT_SCR2	;Bouw eerste scherm op

		MOVLW	0X00     	;Begin regel 0
		MOVFF	WREG,S_POS
		SETF	LOWER_LIM

IS2_LOOP:	BSF	SCREEN_FLAG	;Kies scherm 2
		CALL	U_S_ERASE	;Wis de underscores

		CALL	POS_TO_VAR	;Vertaal positie in variabelen

		MOVLW	0XFF		;Positie maar nog eentje terug
		CPFSEQ	S_POS           ;is naar scherm 1
		GOTO	KL2
		
                CALL	BUILT_SCR1	;Bouw eerste scherm op

		MOVLW	0X13		;Start op hoogste lokatie
		MOVFF	WREG,S_POS
		CLRF	LOWER_LIM
		GOTO	IS1_LOOP

KL2:		CALL	WR_U_S		;Schrijf de underscore

		GOTO	ROT_QUEST
		
END_INPUT:      CALL	BOUNCE_DELAY  	;Klaar, hang jezelf op

		CALL	ERASE_BOTH

                MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

		MOVLW	HIGH STORE_TXT1
		MOVWF	TBLPTRH
		MOVLW	LOW STORE_TXT1
		MOVWF	TBLPTRL

		BSF	SHOW
		CALL	TEXT

		MOVLW	0X02
		CALL	L_TIMER

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X20
		MOVFF	WREG,V_POS

		MOVLW	HIGH STORE_TXT2
		MOVWF	TBLPTRH
		MOVLW	LOW STORE_TXT2
		MOVWF	TBLPTRL

		BSF	SHOW
		CALL	TEXT

		CALL	MAKE_M_CNT	;D-variabelen terug naar minutenteller
		
		CALL	FILL_EEPROM	;Schrijf weg

WFE_I:		GOTO	WFE_I


;POS_TO_VAR = Hulproutine INPUT_SCREEN, lees variabelen uit tabel
;----------------------------------------------------------------

POS_TO_VAR:	MOVLW	0XFF 		;Overgang van pagina 2 naar 1
		CPFSEQ	S_POS           ;vergt geen vertaling
		GOTO	CONT_PTV
		GOTO	END_PTV

CONT_PTV:	BTFSC	SCREEN_FLAG	;Pagina 1 of 2?
		GOTO	VAR_S2          ;Ga voor de tweede

		MOVLW	HIGH MAIN_TABLE1;De eerste
		MOVWF	TBLPTRH
		MOVLW	LOW MAIN_TABLE1
		MOVWF	TBLPTRL
		GOTO	R_TD

VAR_S2:         MOVLW	HIGH MAIN_TABLE2;De tweede
		MOVWF	TBLPTRH
		MOVLW	LOW MAIN_TABLE2
		MOVWF	TBLPTRL

R_TD:           MOVLW	0X06            ;Tabel maakt stappen van 6
		MULWF	S_POS
		BCF	STATUS,C
		MOVFF	PRODL,WREG	;Tel resultaat op bij TBLPTR
		ADDWF   TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH
		
		TBLRD*+             	;Vul de pointer naar de variabele
		MOVFF	TABLAT,FSR2H    ;en de positie
		TBLRD*+
		MOVFF	TABLAT,FSR2L
		TBLRD*+
		MOVFF	TABLAT,H_POS
		TBLRD*+
		MOVFF	TABLAT,V_POS
		TBLRD*+
		MOVFF	TABLAT,MAX_VAL
		TBLRD*+
		MOVFF	TABLAT,STEP
		
END_PTV:	RETURN
		
;ROT_QUEST = Aanroep vanuit INPUT_SCREEN, keert terug in schermroutine
;---------------------------------------------------------------------

ROT_QUEST:      MOVLW	0X01
		MOVFF	WREG,SW_DELAY

IN_S_0:         CALL	PROC_DSS	;Roep de draaischakelaar

		BTFSC	ROT_SW          ;Gedrukt?
		GOTO	IN_S_1          ;Nee
		GOTO	IN_S_PR         ;Ja

IN_S_1:		BTFSS	DS_CHANGED      ;Gedraaid?
		GOTO	IN_S_0          ;Nee

		BTFSS	DS_CU		;Bepaal de richting
		GOTO	IN_S_DOWN
		GOTO	IN_S_UP

IN_S_UP:        BCF	DS_CHANGED 	;Omhoog

		MOVLW	0X14            ;Check bovenwaarde
		CPFSEQ	S_POS
		GOTO	INC_IN_S        ;Niet te hoog
		GOTO	IN_S_0          ;Staat op max

INC_IN_S:	INCF	S_POS
		GOTO    EFF_POS

IN_S_DOWN:      BCF	DS_CHANGED 	;Omlaag

		MOVFF	LOWER_LIM,WREG  ;Check onderwaarde
		CPFSEQ	S_POS
		GOTO	SUB_IN_S        ;Niet te hoog
		GOTO	IN_S_0          ;Staat op min

SUB_IN_S:	DECF	S_POS
		GOTO    EFF_POS

IN_S_PR:        BTFSS	ROT_SW
		GOTO	IN_S_PR

		CALL	BOUNCE_DELAY

		GOTO	EFF_POS_PRESS
		
EFF_POS:        BTFSS	SCREEN_FLAG
		GOTO	IS1_LOOP
		GOTO	IS2_LOOP

EFF_POS_PRESS:  BTFSS	SCREEN_FLAG	;Scherm 2?
		GOTO    NORM_EPP        ;Nee

		MOVLW	0X14            ;Ja, positie 21?
		CPFSEQ	S_POS
		GOTO	NORM_EPP        ;Nee
		
		GOTO	END_INPUT 	;Afsluiten

NORM_EPP:	CALL	POS_TO_VAR

		MOVLW	0X0D
		SUBWF	V_POS

		MOVFF	H_POS,HP_STORE	;Save positie
		MOVFF	V_POS,VP_STORE
		MOVFF	INDF2,A_TIME	;Variabele

		CALL	WRITE_UD

		MOVFF	HP_STORE,H_POS	;Positie terug
		MOVFF	VP_STORE,V_POS

		CALL	ASK_TIME 	;Waarde opvragen

		MOVFF	A_TIME,INDF2    ;Nieuwe waarde naar variabele

                CALL	WRITE_LR        ;Pijltje weer links rechts
                
                INCF	S_POS		;Kies volgend veld

		GOTO	EFF_POS         ;En verder

;-------------------------------------------------
;ASK_TIME = Vraag om een in te stellen tijd
;           Entry: H_POS,V_POS
;                  A_TIME uit EEPROM vanuit caller
;                  MAX_VAL is maximum waarde
;-------------------------------------------------

ASK_TIME:	MOVFF	H_POS,TEMP_H	;Positie bewaren
		MOVFF	V_POS,TEMP_L
		
AT_LOOP:	MOVFF	TEMP_H,H_POS	;Haal positie
		MOVFF	TEMP_L,V_POS

		MOVLW	HIGH ASK_T_TABLE;Tabel met de wisseltijden
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_T_TABLE
		MOVWF	TBLPTRL

		MOVLW	0X04 		;Stappen van 4
		MULWF	A_TIME

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		BSF	SHOW
		CALL	TEXT 		;Laat maar zien
		
		MOVLW	0X01
		MOVFF	WREG,SW_DELAY
		
WFR3:		CALL	PROC_DSS	;Draaischakelaar

		BTFSC	ROT_SW          ;Gedrukt?
		GOTO	WFR4            ;Nee
		GOTO	REL_ZS          ;Ja

WFR4:		BTFSS	DS_CHANGED      ;Gedraaid?
		GOTO	WFR3            ;Nee

		BTFSS	DS_CU		;Bepaal de richting
		GOTO	T_DOWN
		GOTO	T_UP

T_UP:           BCF	DS_CHANGED 	;Omhoog

		MOVFF	MAX_VAL,WREG    ;Check bovenwaarde
		CPFSEQ	A_TIME
		GOTO	INC_ADM         ;Niet te hoog
		GOTO	WFR3            ;Staat op max
		
INC_ADM:	BCF	STATUS,C
		MOVFF	STEP,WREG	;Stapgrootte
		ADDWF	A_TIME

		GOTO    AT_LOOP

T_DOWN:         BCF	DS_CHANGED 	;Omlaag

		MOVLW	0X00            ;Check onderwaarde
		CPFSEQ	A_TIME          ;Fixed op 00
		GOTO	SUB_ADM         ;Niet te hoog
		GOTO	WFR3            ;Staat op min

SUB_ADM:	MOVFF	STEP,WREG
		SUBWF	A_TIME
		GOTO    AT_LOOP
		
REL_ZS:         BTFSS	ROT_SW
		GOTO	REL_ZS

		CALL	BOUNCE_DELAY
		
		RETURN
		
;--------------------------------------------------
;ASK_BRIGHT = Vraag om een in te stellen helderheid
;             Entry: H_POS,V_POS
;                  BRIGHT uit EEPROM vanuit caller
;                  MAX_VAL is maximum waarde
;		   STEP = Stapgrootte
;-------------------------------------------------

ASK_BRIGHT:	MOVFF	H_POS,TEMP_H	;Positie bewaren
		MOVFF	V_POS,TEMP_L

AT_LOOPB:	MOVFF	TEMP_H,H_POS	;Haal positie
		MOVFF	TEMP_L,V_POS

		MOVLW	HIGH ASK_T_TABLE;Tabel met de wisseltijden
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_T_TABLE
		MOVWF	TBLPTRL

		MOVLW	0X04 		;Stappen van 4
		MULWF	BRIGHT

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		BSF	SHOW
		CALL	TEXT 		;Laat maar zien

		MOVLW	0X01
		MOVFF	WREG,SW_DELAY

WFR3B:		CALL	PROC_DSS	;Draaischakelaar

		BTFSC	ROT_SW          ;Gedrukt?
		GOTO	WFR4B           ;Nee
		GOTO	REL_ZSB         ;Ja

WFR4B:		BTFSS	DS_CHANGED      ;Gedraaid?
		GOTO	WFR3B           ;Nee

		BTFSS	DS_CU		;Bepaal de richting
		GOTO	T_DOWNB
		GOTO	T_UPB

T_UPB:          BCF	DS_CHANGED 	;Omhoog

		MOVFF	MAX_VAL,WREG    ;Check bovenwaarde
		CPFSEQ	BRIGHT
		GOTO	INC_ADMB        ;Niet te hoog
		GOTO	WFR3B           ;Staat op max

INC_ADMB:	BCF	STATUS,C
		MOVFF	STEP,WREG	;Stapgrootte
		ADDWF	BRIGHT

		CALL	AB_SET

		GOTO    AT_LOOPB

T_DOWNB:        BCF	DS_CHANGED 	;Omlaag

		MOVLW	0X00            ;Check onderwaarde
		CPFSEQ	BRIGHT          ;Fixed op 00
		GOTO	SUB_ADMB        ;Niet te hoog
		GOTO	WFR3B           ;Staat op min

SUB_ADMB:	MOVFF	STEP,WREG
		SUBWF	BRIGHT
		
		CALL	AB_SET

		GOTO    AT_LOOPB

REL_ZSB:        BTFSS	ROT_SW
		GOTO	REL_ZSB
		
		CALL	BOUNCE_DELAY

		RETURN
		
;AB_SET = Hulproutine ASK_BRIGHT
;-------------------------------

AB_SET:		MOVFF	BRIGHT,TEMP
		SWAPF	TEMP
		MOVLW	0X81        	;Stel helderheid in
		CALL	WR_COMMAND
		MOVFF	TEMP,WREG
		CALL	WR_COMMAND
		
		RETURN
		
;-----------------------------------------------------------
;MAKE_M_CNT = Reken de D waardes (BCD) om naar minutenteller
;-----------------------------------------------------------

MAKE_M_CNT:	MOVLW	0X10         	;16 x 2 bytes converteren
		MOVFF	WREG,LOOP_CNT

		LFSR	0,T1A_HD	;Source met hex getal
		LFSR	1,T1A_H         ;Destination krijgt minutenteller
		
CONVM_LOOP:	MOVFF	POSTINC0,TEMP_H
		MOVFF	POSTINC0,TEMP_L

		MOVLW	0X3C  		;Vermenigvuldig met 60
		MULWF	TEMP_H

		MOVFF	TEMP_L,WREG
		ADDWF	PRODL
		BTFSC	STATUS,C
		INCF	PRODH

		MOVFF	PRODH,POSTINC1
		MOVFF	PRODL,POSTINC1
		
		DECFSZ	LOOP_CNT
		GOTO	CONVM_LOOP
		
		RETURN

		
;WRITE_UD = Hulproutine INPUT_SCREEN, schijf up/down pijltje
;-----------------------------------------------------------
		
WRITE_UD:       MOVLW	0X6F            ;Wis lokatie
		MOVFF	WREG,H_POS
		MOVLW	0X37
		MOVFF	WREG,V_POS

		MOVLW	HIGH ARROW_E
		MOVWF	TBLPTRH
		MOVLW	LOW ARROW_E
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		MOVLW	0X70            ;Schrijf pijltje UD
		MOVFF	WREG,H_POS
		MOVLW	0X36
		MOVFF	WREG,V_POS

		MOVLW	HIGH ARROW_UD
		MOVWF	TBLPTRH
		MOVLW	LOW ARROW_UD
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW
		
		RETURN
		
;WRITE_LR = Hulproutine INPUT_SCREEN, schijf up/down pijltje
;-----------------------------------------------------------

WRITE_LR:       MOVLW	0X70            ;Wis lokatie
		MOVFF	WREG,H_POS
		MOVLW	0X36
		MOVFF	WREG,V_POS

		MOVLW	HIGH ARROW_E
		MOVWF	TBLPTRH
		MOVLW	LOW ARROW_E
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		MOVLW	0X6F            ;Schrijf pijltje UD
		MOVFF	WREG,H_POS
		MOVLW	0X37
		MOVFF	WREG,V_POS

		MOVLW	HIGH ARROW_LR
		MOVWF	TBLPTRH
		MOVLW	LOW ARROW_LR
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW

		RETURN
		
;BUILT_SCR1 = Hulproutine INPUT_SCREEN, bouw scherm 1
;----------------------------------------------------

BUILT_SCR1:	CALL	ERASE_BOTH	;Wis

		BCF	SHOW		;Geen directe display

		LFSR	2,T1A_HD	;Wijst naar variabele
		MOVLW	0X00      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T1,GEN_MODE

                MOVLW	HIGH T1_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T1_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn

		LFSR	2,T2A_HD	;Wijst naar variabele
		MOVLW	0X0D      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T2,GEN_MODE

                MOVLW	HIGH T2_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T2_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn
		
		LFSR	2,T3A_HD	;Wijst naar variabele
		MOVLW	0X1A      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T3,GEN_MODE

                MOVLW	HIGH T3_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T3_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn
		
		LFSR	2,T4A_HD	;Wijst naar variabele
		MOVLW	0X27      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T4,GEN_MODE

                MOVLW	HIGH T4_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T4_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn

		MOVLW	0X6F
		MOVFF	WREG,H_POS
		MOVLW	0X37
		MOVFF	WREG,V_POS

		MOVLW	HIGH ARROW_LR
		MOVWF	TBLPTRH
		MOVLW	LOW ARROW_LR
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW
		
		BCF	SHOW

		RETURN
		
;BUILT_SCR2 = Hulproutine INPUT_SCREEN, bouw scherm 2
;----------------------------------------------------

BUILT_SCR2:	CALL	ERASE_BOTH	;Wis

		BCF	SHOW		;Geen directe display

		LFSR	2,T5A_HD	;Wijst naar variabele
		MOVLW	0X00      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T5,GEN_MODE

                MOVLW	HIGH T5_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T5_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn

		LFSR	2,T6A_HD	;Wijst naar variabele
		MOVLW	0X0D      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T6,GEN_MODE

                MOVLW	HIGH T6_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T6_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn

		LFSR	2,T7A_HD	;Wijst naar variabele
		MOVLW	0X1A      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T7,GEN_MODE

                MOVLW	HIGH T7_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T7_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn

		LFSR	2,T8A_HD	;Wijst naar variabele
		MOVLW	0X27      	;Verticale positie
                MOVFF	WREG,V_POS
                MOVFF	MODE_T8,GEN_MODE

                MOVLW	HIGH T8_TXT     ;Specifieke text
		MOVWF	TBLPTRH
		MOVLW	LOW T8_TXT
		MOVWF	TBLPTRL

		CALL	WR_LINE		;Schrijf 1 lijn
		
		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X34
		MOVFF	WREG,V_POS

		MOVLW	HIGH OK_TXT	;Schrijf ok veld
		MOVWF	TBLPTRH
		MOVLW	LOW OK_TXT
		MOVWF	TBLPTRL

		CALL	TEXT

                MOVLW	0X12          	;Schrijf keuzeblokje
		MOVFF	WREG,H_POS
		MOVLW	0X34
		MOVFF	WREG,V_POS

		MOVLW	HIGH CHOICE_N
		MOVWF	TBLPTRH
		MOVLW	LOW CHOICE_N
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		MOVLW	0X6F 		;Schrijf pijltje LR
		MOVFF	WREG,H_POS
		MOVLW	0X37
		MOVFF	WREG,V_POS

		MOVLW	HIGH ARROW_LR
		MOVWF	TBLPTRH
		MOVLW	LOW ARROW_LR
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW
		
		BCF	SHOW

		RETURN

;Hulproutine BUILT_SCRx, schrijf 1 lijn
;--------------------------------------

WR_LINE:	MOVFF	H_POS1,H_POS 	;Tekst voor timer

                CALL	TEXT

                MOVFF	H_POS2,H_POS	;Hoge tijdveld aan

		MOVFF	POSTINC2,A_TIME
		CALL	CONV_TXT

		CALL	TEXT

                MOVFF	H_POS3,H_POS	;Dubbele punt

		MOVLW	HIGH INTER_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW INTER_TXT
		MOVWF	TBLPTRL

		CALL	TEXT

                MOVFF	H_POS4,H_POS	;Lage tijdveld aan

		MOVFF	POSTINC2,A_TIME
		CALL	CONV_TXT

		CALL	TEXT

		MOVFF	H_POS5,H_POS	;Hoge tijdveld uit

		MOVFF	POSTINC2,A_TIME
		CALL	CONV_TXT

		CALL	TEXT

                MOVFF	H_POS6,H_POS	;Dubbele punt

		MOVLW	HIGH INTER_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW INTER_TXT
		MOVWF	TBLPTRL

		CALL	TEXT

                MOVFF	H_POS7,H_POS	;Lage tijdveld uit

		MOVFF	POSTINC2,A_TIME
		CALL	CONV_TXT
		
		CALL	TEXT

		MOVFF	H_POS8,H_POS 	;Mode tekst

		MOVLW	HIGH MODE_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW MODE_TXT
		MOVWF	TBLPTRL

		CALL	TEXT
		
		MOVFF	H_POS9,H_POS	;Mode getal

		MOVFF	GEN_MODE,A_TIME
		CALL	CONV_TXT

		CALL	TEXT

		RETURN

;CONV_TXT = Hulproutine INPUT_SCREEN
;-----------------------------------

CONV_TXT:	MOVLW	HIGH ASK_T_TABLE;Tabel met de wisseltijden
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_T_TABLE
		MOVWF	TBLPTRL

		MOVLW	0X04 		;Stappen van 4
		MULWF	A_TIME

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		RETURN

;U_S_ERASE= Hulproutine INPUT_SCREEN, wis alle underscore lokaties
;------------------------------------------------------------------

U_S_ERASE:	MOVLW	0X0D		;Regel 1
		MOVFF	WREG,V_POS
		CALL	WR_U_S_LINE

                MOVLW	0X1A            ;Regel2
		MOVFF	WREG,V_POS
		CALL	WR_U_S_LINE

		MOVLW	0X27            ;Regel 3
		MOVFF	WREG,V_POS
		CALL	WR_U_S_LINE

		MOVLW	0X34            ;Regel 4
		MOVFF	WREG,V_POS
		CALL	WR_U_S_LINE
		
		BTFSS   SCREEN_FLAG	;Scherm 2?
		GOTO	E_USE           ;Nee

		MOVLW	0X12          	;Ja, ook keuzeblokje wissen
		MOVFF	WREG,H_POS
		MOVLW	0X34
		MOVFF	WREG,V_POS

		MOVLW	HIGH CHOICE_N
		MOVWF	TBLPTRH
		MOVLW	LOW CHOICE_N
		MOVWF	TBLPTRL

		CALL	WR_CHAR
		
		CALL	WRITE_SHADOW

E_USE:		RETURN

WR_U_S_LINE:	MOVFF	H_POS2,H_POS	;Positie hoge byte aan timer
		CALL	SET_U_S
		CALL	WR_CHAR

                MOVFF	H_POS4,H_POS    ;Positie lage byte aan timer
		CALL	SET_U_S
		CALL	WR_CHAR

		MOVFF	H_POS5,H_POS    ;Positie hoge byte uit timer
		CALL	SET_U_S
		CALL	WR_CHAR

		MOVFF	H_POS7,H_POS    ;Positie lage byte uit timer
		CALL	SET_U_S
		CALL	WR_CHAR

                MOVFF	H_POS9,H_POS	;Positie mode
		CALL	SET_U_S
		CALL	WR_CHAR
		
		RETURN

SET_U_S:	MOVLW	HIGH USCORE_E
		MOVWF	TBLPTRH
		MOVLW	LOW USCORE_E
		MOVWF	TBLPTRL

		RETURN
		
;WR_U_S = Hulproutine INPUT_SCREEN, schrijf de underscore
;--------------------------------------------------------

WR_U_S:         BTFSS	SCREEN_FLAG 	;Scherm 2?
		GOTO    WR_U_S_N        ;Nee

		MOVLW	0X14            ;Hoogste positie scherm 2?
		CPFSEQ	S_POS
		GOTO	WR_U_S_N        ;Nee

		GOTO	WR_CH           ;Ja

WR_U_S_N:	MOVLW	HIGH USCORE
		MOVWF	TBLPTRH
		MOVLW	LOW USCORE
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW

		GOTO	END_WRUS

WR_CH:		MOVLW	0X12          	;Schrijf keuzeblokje
		MOVFF	WREG,H_POS
		MOVLW	0X34
		MOVFF	WREG,V_POS

		MOVLW	HIGH CHOICE_Y
		MOVWF	TBLPTRH
		MOVLW	LOW CHOICE_Y
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW
		
END_WRUS:	RETURN

;------------------------------------------
;SHOW_TIME = Toon de tijd
;            Entry: positie moet goed staan
;------------------------------------------

SHOW_TIME:	BCF	SHOW

		MOVLW	HIGH ASK_T_TABLE;Tabel met ASCII waarden
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_T_TABLE
		MOVWF	TBLPTRL

		MOVFF	PIC_HR_HEX,TEMP	;Eerst het uur
		MOVLW	0X04
		MULWF	TEMP

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		CALL	TEXT

                MOVLW	HIGH INTER_TXT	;Dubbele punt
		MOVWF	TBLPTRH
		MOVLW	LOW INTER_TXT
		MOVWF	TBLPTRL

		CALL	TEXT

		MOVLW	0X06 		;H_POS corrigeren
		SUBWF	H_POS

                MOVLW	HIGH ASK_T_TABLE
		MOVWF	TBLPTRH
		MOVLW	LOW ASK_T_TABLE
		MOVWF	TBLPTRL

		MOVFF	PIC_MIN_HEX,TEMP;De minuut
		MOVLW	0X04
		MULWF	TEMP

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		CALL	TEXT
		
		RETURN 

;------------------------------------------------
;CHECK_DATE = Check de datum
;             Exit: flag DATE_WRONG 0=goed 1=fout
;------------------------------------------------

CHECK_DATE:     BSF	DATE_WRONG

		MOVLW	HIGH GOOD_VAL_TBL;Tabel met geldige waarden
		MOVWF	TBLPTRH
		MOVLW	LOW GOOD_VAL_TBL
		MOVWF	TBLPTRL

		MOVLW	0X1F 		;Voor de dagen 31 x checken
		MOVFF	WREG,LOOP_CNT

DAY_LOOPS:	TBLRD*+               	;Lees een waarde
		MOVLW	0XFF            ;FF is einde tabel dus fout
		CPFSEQ	TABLAT
		GOTO 	CH_DAY		;Tabelwaarde gelezen
		GOTO	DATE_PROBLEM
CH_DAY:		MOVFF	TABLAT,WREG     ;Haal uit tabel
		CPFSEQ	DAY_BCD         ;Indien gelijk is dag geldig
		GOTO    DAY_C
		GOTO	MONTH_CHK	;Ga maand testen
DAY_C:		DECFSZ	LOOP_CNT
		GOTO	DAY_LOOPS
		GOTO	DATE_PROBLEM

MONTH_CHK:	MOVLW	HIGH GOOD_VAL_TBL;Tabel met geldige waarden
		MOVWF	TBLPTRH
		MOVLW	LOW GOOD_VAL_TBL
		MOVWF	TBLPTRL

		MOVLW	0X0C		;Voor de maand 12 x checken
		MOVFF	WREG,LOOP_CNT

MONTH_LOOPS:	TBLRD*+               	;Lees een waarde
		MOVLW	0XFF            ;FF is einde tabel dus fout
		CPFSEQ	TABLAT
		GOTO 	CH_MONTH	;Tabelwaarde gelezen
		GOTO	DATE_PROBLEM
CH_MONTH:	MOVFF	TABLAT,WREG     ;Haal uit tabel
		CPFSEQ	MONTH_BCD       ;Indien gelijk is maand geldig
		GOTO    MONTH_C
		GOTO	YEAR_CHK	;Ga jaar testen
MONTH_C:	DECFSZ	LOOP_CNT
		GOTO	MONTH_LOOPS
		GOTO	DATE_PROBLEM

YEAR_CHK:       MOVLW	HIGH GOOD_Y_TBL;Tabel met geldige waarden
		MOVWF	TBLPTRH
		MOVLW	LOW GOOD_Y_TBL
		MOVWF	TBLPTRL

		MOVLW	0X0B		;Voor de jaren 11 x checken
		MOVFF	WREG,LOOP_CNT

YEAR_LOOP:	TBLRD*+               	;Lees een waarde
		MOVLW	0XFF            ;FF is einde tabel dus fout
		CPFSEQ	TABLAT
		GOTO 	CH_YEAR		;Tabelwaarde gelezen
		GOTO	DATE_PROBLEM
CH_YEAR:	MOVFF	TABLAT,WREG     ;Haal uit tabel
		CPFSEQ	YEAR_BCD        ;Indien gelijk is jaar geldig
		GOTO    YEAR_C
		GOTO	SD_NP		;Ga door
YEAR_C:		DECFSZ	LOOP_CNT
		GOTO	YEAR_LOOP
		GOTO	DATE_PROBLEM
		
SD_NP:		BCF	DATE_WRONG	;Datum correct
		GOTO	END_YEAR_C

DATE_PROBLEM:	BSF	DATE_WRONG

END_YEAR_C:	RETURN
		
;------------------------------------------
;SHOW_DATE = Toon de datum
;            Entry: positie moet goed staan
;------------------------------------------

SHOW_DATE:	MOVLW	HIGH BCD_TABLE;Tabel met ASCII waarden
		MOVWF	TBLPTRH
		MOVLW	LOW BCD_TABLE
		MOVWF	TBLPTRL

		MOVFF	DAY_BCD,TEMP 	;Eerst de dag
		MOVLW	0X04
		MULWF	TEMP

		BCF	STATUS,C
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH

		CALL	TEXT

                MOVLW	HIGH MONTH_TABLE;Tabel met ASCII waarden
		MOVWF	TBLPTRH         ;met de maanden
		MOVLW	LOW MONTH_TABLE
		MOVWF	TBLPTRL
		
		MOVFF	MONTH_BCD,TEMP	;TEMP heeft BCD waarde maand

S_D_READ:	TBLRD*+                 ;Lees tabel
		MOVFF	TABLAT,TEMP2
		MOVLW	0XFE            ;FE is groot probleem
		CPFSEQ	TEMP2
		GOTO    K_S_D
		GOTO	SD_ERROR	;Oeps

K_S_D:		MOVFF	TEMP2,WREG	;Vergelijk met maand
		CPFSEQ	TEMP            ;Bingo
		GOTO	S_D_READ        ;Doorzoeken
		
		TBLRD*+

		MOVLW	0X03
		ADDWF	H_POS

		CALL	TEXT

		MOVLW	HIGH YEAR_TABLE;Tabel met ASCII waarden
		MOVWF	TBLPTRH         ;met de jaren
		MOVLW	LOW YEAR_TABLE
		MOVWF	TBLPTRL

		MOVFF	YEAR_BCD,TEMP	;TEMP heeft BCD waarde maand

S_D_READ1:	TBLRD*+                 ;Lees tabel
		MOVFF	TABLAT,TEMP2
		MOVLW	0XFE            ;FE is groot probleem
		CPFSEQ	TEMP2
		GOTO    K_S_D1
		GOTO	SD_ERROR	;Oeps

K_S_D1:		MOVFF	TEMP2,WREG	;Vergelijk met jaar
		CPFSEQ	TEMP            ;Bingo
		GOTO	S_D_READ1       ;Doorzoeken
		
		TBLRD*+

		MOVLW	0X03 		;Mei is enige maand met spatie
		ADDWF	H_POS

		MOVLW	0X05
		CPFSEQ	MONTH_BCD
		GOTO	SH_D

		MOVLW	0X05    	;Pas H_POS aan naar links
		SUBWF	H_POS

SH_D:		CALL	TEXT

		GOTO	END_S_D
		
SD_ERROR:	MOVLW	0X08
		MOVFF	WREG,ERROR_CODE
		CALL	ERROR_DISP

END_S_D:	RETURN

;disp_dump = zet tijd en datum op hulpdisplay
;--------------------------------------------

disp_dump:	movlw	0x0d
		movff	wreg,dd_nr

                movff	pic_hr_bcd,dd_0
                movff	pic_min_bcd,dd_1
                movff	pic_sec_bcd,dd_2
                movlw	0x00
                movff	wreg,dd_3

		movff	gps_hr_hex,dd_4
		movff	gps_min_hex,dd_5
		movff	gps_sec_hex,dd_6
		movlw	0x00
                movff	wreg,dd_7

		movff	rel_byte,dd_8

		movff	min_cnt_h,dd_9
		movff	min_cnt_l,dd_10

		movff	gps_err_cnt,dd_11

		movff	wd_timer_rs232h,dd_12

		call	do_dump

		return

;--------------------------------------
;WRITE_IO = Bepaal stand uitgangsrelais
;--------------------------------------

WRITE_IO:      	BTFSC	S_U_FLAG	;Startup periode afwachten
		GOTO	END_IO

		MOVFF	MODE_T1B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH1_NOT_ACT	;Timer 1 niet actief
		GOTO	CH1_ACT         ;Timer 1 wel actief

CH1_NOT_ACT:    BCF	REL_BYTE,0	;Evt. window negeren
		GOTO	CH_LS_1		;Staat uit maar wellicht de lichtsensor

CH1_ACT:	BTFSS	REL_BYTE,0	;Is er een geldig window?
		GOTO	IO_2		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_1		;Nee

		GOTO	CH_LS_1		;Lichtsensor overrule?

CH_B2_1:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_1		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_1         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_1	;Weekend: geen aktie

CH_B3_1:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_1		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_1	;Doordeweek: geen aktie

CH_LS_1:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS1         ;Yes
		GOTO	IO_2

ACT_LS1:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_1        ;Ja
		GOTO	PH_ON_1         ;Nee

PH_OFF_1:	BCF	REL_BYTE,0	;Relais uit
		GOTO	IO_2

PH_ON_1:	BSF	REL_BYTE,0      ;Relais aan

IO_2:           MOVFF	MODE_T2B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH2_NOT_ACT	;Timer 2 niet actief
		GOTO	CH2_ACT         ;Timer 2 wel actief

CH2_NOT_ACT:    BCF	REL_BYTE,1	;Evt. window negeren
		GOTO	CH_LS_2		;Staat uit maar wellicht de lichtsensor

CH2_ACT:	BTFSS	REL_BYTE,1	;Is er een geldig window?
		GOTO	IO_3		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_2		;Nee

		GOTO	CH_LS_2		;Lichtsensor overrule?

CH_B2_2:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_2		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_2         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_2	;Weekend: geen aktie

CH_B3_2:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_2		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_2	;Doordeweek: geen aktie

CH_LS_2:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS2         ;Yes
		GOTO	IO_3

ACT_LS2:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_2        ;Ja
		GOTO	PH_ON_2         ;Nee

PH_OFF_2:	BCF	REL_BYTE,1	;Relais uit
		GOTO	IO_3

PH_ON_2:	BSF	REL_BYTE,1      ;Relais aan

IO_3:           MOVFF	MODE_T3B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH3_NOT_ACT	;Timer 3 niet actief
		GOTO	CH3_ACT         ;Timer 3 wel actief

CH3_NOT_ACT:    BCF	REL_BYTE,2	;Evt. window negeren
		GOTO	CH_LS_3		;Staat uit maar wellicht de lichtsensor

CH3_ACT:	BTFSS	REL_BYTE,2	;Is er een geldig window?
		GOTO	IO_4		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_3		;Nee

		GOTO	CH_LS_3		;Lichtsensor overrule?

CH_B2_3:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_3		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_3         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_3	;Weekend: geen aktie

CH_B3_3:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_3		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_3	;Doordeweek: geen aktie

CH_LS_3:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS3         ;Yes
		GOTO	IO_4

ACT_LS3:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_3        ;Ja
		GOTO	PH_ON_3         ;Nee

PH_OFF_3:	BCF	REL_BYTE,2	;Relais uit
		GOTO	IO_4

PH_ON_3:	BSF	REL_BYTE,2      ;Relais aan

IO_4:           MOVFF	MODE_T4B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH4_NOT_ACT	;Timer 4 niet actief
		GOTO	CH4_ACT         ;Timer 4 wel actief

CH4_NOT_ACT:    BCF	REL_BYTE,3	;Evt. window negeren
		GOTO	CH_LS_4		;Staat uit maar wellicht de lichtsensor

CH4_ACT:	BTFSS	REL_BYTE,3	;Is er een geldig window?
		GOTO	IO_5		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_4		;Nee

		GOTO	CH_LS_4		;Lichtsensor overrule?

CH_B2_4:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_4		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_4         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_4	;Weekend: geen aktie

CH_B3_4:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_4		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_4	;Doordeweek: geen aktie

CH_LS_4:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS4         ;Yes
		GOTO	IO_5

ACT_LS4:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_4        ;Ja
		GOTO	PH_ON_4         ;Nee

PH_OFF_4:	BCF	REL_BYTE,3	;Relais uit
		GOTO	IO_5

PH_ON_4:	BSF	REL_BYTE,3      ;Relais aan

IO_5:           MOVFF	MODE_T5B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH5_NOT_ACT	;Timer 5 niet actief
		GOTO	CH5_ACT         ;Timer 5 wel actief

CH5_NOT_ACT:    BCF	REL_BYTE,4	;Evt. window negeren
		GOTO	CH_LS_5		;Staat uit maar wellicht de lichtsensor

CH5_ACT:	BTFSS	REL_BYTE,4	;Is er een geldig window?
		GOTO	IO_6		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_5		;Nee

		GOTO	CH_LS_5		;Lichtsensor overrule?

CH_B2_5:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_5		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_5         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_5	;Weekend: geen aktie

CH_B3_5:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_5		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_5	;Doordeweek: geen aktie

CH_LS_5:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS5         ;Yes
		GOTO	IO_6

ACT_LS5:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_5        ;Ja
		GOTO	PH_ON_5         ;Nee

PH_OFF_5:	BCF	REL_BYTE,4	;Relais uit
		GOTO	IO_6

PH_ON_5:	BSF	REL_BYTE,4      ;Relais aan

IO_6:           MOVFF	MODE_T6B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH6_NOT_ACT	;Timer 6 niet actief
		GOTO	CH6_ACT         ;Timer 6 wel actief

CH6_NOT_ACT:    BCF	REL_BYTE,5	;Evt. window negeren
		GOTO	CH_LS_6		;Staat uit maar wellicht de lichtsensor

CH6_ACT:	BTFSS	REL_BYTE,5	;Is er een geldig window?
		GOTO	IO_7		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_6		;Nee

		GOTO	CH_LS_6		;Lichtsensor overrule?

CH_B2_6:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_6		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_6         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_6	;Weekend: geen aktie

CH_B3_6:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_6		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_6	;Doordeweek: geen aktie

CH_LS_6:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS6         ;Yes
		GOTO	IO_7

ACT_LS6:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_6        ;Ja
		GOTO	PH_ON_6         ;Nee

PH_OFF_6:	BCF	REL_BYTE,5	;Relais uit
		GOTO	IO_7

PH_ON_6:	BSF	REL_BYTE,5      ;Relais aan

IO_7:           MOVFF	MODE_T7B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH7_NOT_ACT	;Timer 7 niet actief
		GOTO	CH7_ACT         ;Timer 7 wel actief

CH7_NOT_ACT:    BCF	REL_BYTE,6	;Evt. window negeren
		GOTO	CH_LS_7		;Staat uit maar wellicht de lichtsensor

CH7_ACT:	BTFSS	REL_BYTE,6	;Is er een geldig window?
		GOTO	IO_8		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_7		;Nee

		GOTO	CH_LS_7		;Lichtsensor overrule?

CH_B2_7:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_7		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_7         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_7	;Weekend: geen aktie

CH_B3_7:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_7		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_7	;Doordeweek: geen aktie

CH_LS_7:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS7         ;Yes
		GOTO	IO_8

ACT_LS7:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_7        ;Ja
		GOTO	PH_ON_7         ;Nee

PH_OFF_7:	BCF	REL_BYTE,6	;Relais uit
		GOTO	IO_8

PH_ON_7:	BSF	REL_BYTE,6      ;Relais aan

IO_8:           MOVFF	MODE_T8B,TEMP   ;Mag niet direct
		BTFSS	TEMP,0		;Test de aan/uit bit
		GOTO	CH8_NOT_ACT	;Timer 8 niet actief
		GOTO	CH8_ACT         ;Timer 8 wel actief

CH8_NOT_ACT:    BCF	REL_BYTE,7	;Evt. window negeren
		GOTO	CH_LS_8		;Staat uit maar wellicht de lichtsensor

CH8_ACT:	BTFSS	REL_BYTE,7	;Is er een geldig window?
		GOTO	E_WR_IO		;Nee

		BTFSS	TEMP,1		;Is 7 dagen mode actief
		GOTO	CH_B2_8		;Nee

		GOTO	CH_LS_8		;Lichtsensor overrule?

CH_B2_8:        BTFSS	TEMP,2		;Is 5 dagen mode actief?
		GOTO	CH_B3_8		;Nee

		BTFSS	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_8         ;Nee, dus test de lichtsensor
		GOTO	PH_OFF_8	;Weekend: geen aktie

CH_B3_8:	BTFSS	TEMP,3		;Is de weekend mode actief
		GOTO	WR_IO_ERROR     ;Oeps, geen mode dus probleem

		BTFSC	SPEC_DAY	;Is het weekend?
		GOTO	CH_LS_8		;Ja, dus test de lichtsensor
		GOTO	PH_OFF_8	;Doordeweek: geen aktie

CH_LS_8:       	BTFSC	TEMP,4		;Lichtsensor bit aan?
		GOTO	ACT_LS8         ;Yes
		GOTO	E_WR_IO

ACT_LS8:        BTFSC	PHOTOCELL	;Fotocel actief?
		GOTO	PH_OFF_8        ;Ja
		GOTO	PH_ON_8         ;Nee

PH_OFF_8:	BCF	REL_BYTE,7	;Relais uit
		GOTO	E_WR_IO

PH_ON_8:	BSF	REL_BYTE,7      ;Relais aan
		GOTO	E_WR_IO

WR_IO_ERROR:    MOVLW	0X09
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP

E_WR_IO:	MOVFF	REL_BYTE,LATB	;Activeer de uitgangen
		MOVFF	REL_BYTE,REL_STATUS;Bestemd voor STATUS

END_IO:		RETURN

;------------------------------------------
;DAY_SELECT = Check of dag speciaal is
;             Exit: indien ja, SPEC_DAY = 1
;------------------------------------------

DAY_SELECT:     BCF	SPEC_DAY	;Begin op uit

		MOVLW	HIGH YEARS_TBL	;Wijs in tabel bestaande jaren
		MOVWF	TBLPTRH
		MOVLW	LOW YEARS_TBL
		MOVWF	TBLPTRL

D_S_LOOP:	TBLRD*+        		;Lees uit tabel
		MOVLW	0XFF            ;FF is endpointer
		CPFSEQ	TABLAT		;Gelijk aan tabel?
		GOTO    CH_R_DAY        ;Nee
		GOTO	END_DS          ;Ja, dus einde

CH_R_DAY:	MOVFF	TABLAT,WREG	;TABLAT heeft een jaar
		CPFSEQ	YEAR_BCD        ;Bestaat dat?
		GOTO    NY_FND		;Nee
		GOTO	YEAR_EXIST      ;Ja
		
NY_FND:		TBLRD*+        		;Over hex waarde
		GOTO	D_S_LOOP        ;Volgende poging
		
YEAR_EXIST:	TBLRD*+            	;Lees de hex waarde
		MOVFF	TABLAT,TEMP

		MOVLW	HIGH CAL_TBL	;Wijs in kalender
		MOVWF	TBLPTRH
		MOVLW	LOW CAL_TBL
		MOVWF	TBLPTRL
		
		MOVLW	0XC0		;Tabel heeft stappen van 192
		MULWF	TEMP
		
		BCF	STATUS,C	;Zet tabel aan begin goede jaar
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH
		
		MOVFF	TBLPTRH,TPH_BU	;Red de waarden
		MOVFF	TBLPTRL,TPL_BU

		MOVLW	HIGH MONTH_TBL	;Wijs in tabel bestaande maanden
		MOVWF	TBLPTRH
		MOVLW	LOW MONTH_TBL
		MOVWF	TBLPTRL

D_S_LOOP2:	TBLRD*+        		;Lees uit tabel
		MOVLW	0XFF            ;FF is endpointer
		CPFSEQ	TABLAT		;Gelijk aan tabel?
		GOTO    CH_R_DAY2       ;Nee
		GOTO	END_DS          ;Ja, dus einde

CH_R_DAY2:	MOVFF	TABLAT,WREG	;TABLAT heeft een maand
		CPFSEQ	MONTH_BCD       ;Bestaat dat?
		GOTO    NY_FND2		;Nee
		GOTO	MONTH_EXIST     ;Ja

NY_FND2:	TBLRD*+        		;Over hex waarde
		GOTO	D_S_LOOP2       ;Volgende poging

MONTH_EXIST:	TBLRD*+            	;Lees de hex waarde
		MOVFF	TABLAT,TEMP

		MOVLW	0X10		;16 data per maandblok
		MULWF	TEMP
		
		MOVFF	TPH_BU,TBLPTRH	;Kalendertabel terug
		MOVFF	TPL_BU,TBLPTRL

		BCF	STATUS,C        ;Tel maandenstand erbij op
		MOVFF	PRODL,WREG
		ADDWF	TBLPTRL
		MOVFF	PRODH,WREG
		ADDWFC	TBLPTRH
		
RE_DAY_LOOP:	TBLRD*+			;Lees een dag
		MOVLW	0X00            ;00 is einde oefening
		CPFSEQ	TABLAT
		GOTO    CHECK_DAY       ;Nee, dus lezen
		GOTO	END_DS

CHECK_DAY:	MOVFF	DAY_BCD,WREG	;De dag
		CPFSEQ	TABLAT
		GOTO	RE_DAY_LOOP
		
		BSF	SPEC_DAY	;Is een speciale dag

END_DS:		RETURN

;-----------------------------------------------------------------------
;
;EEPROM routines
;
;-----------------------------------------------------------------------

;----------------------------------------------------------------------
;READ_EEPROM = Lees de EEPROM data
;              Exit: Byte 0 = AAH
;		     Byte 1 .. 32 bytes timerwaardes in minutenteller
;                    Byte 33 .. 40 bytes MODE
;                    Byte 41 = GMT, byte 42 = BRIGHT
;----------------------------------------------------------------------

READ_EEPROM:    MOVLW	0X00   		;Lees adres 00
		MOVWF	EEADR
		CALL	EE_READ

		MOVLW	0XAA		;Staat er AA?
		CPFSEQ	EE_IN           ;Ja
		GOTO	IN_EEPROM	;Nee, vul initieele waardes

		GOTO	READ_BYTES

IN_EEPROM:	CALL	DISP_DEF	;Waarschuw EEPROM leeg
		CALL	INIT_EEPROM	;Vul initieele waardes

		MOVLW	0X02            ;Gedurende 2 sec.
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0XC4
		MOVFF	WREG,DELAY_LOC
		CALL	DELAY
		CALL	DELAY

READ_BYTES:	MOVLW	0X01         	;40 bytes lezen vanaf 1
		MOVFF	WREG,LOOP_CNT

		LFSR	0,T1A_H       	;Hier begint 40 bytes dataveld

EE_R_LOOP:	MOVFF	LOOP_CNT,WREG   ;Teller levert adres
		CALL	EE_READ         ;Lees de EEPROM
		MOVFF	EE_IN,POSTINC0  ;Data naar byte

		INCF	LOOP_CNT	;Teller omhoog
		MOVLW	0X29            ;40 gehad?
		CPFSEQ	LOOP_CNT
		GOTO	EE_R_LOOP       ;Nee
		
		MOVLW	0X29     	;Lees GMT veld
		CALL	EE_READ
		MOVFF	EE_IN,GMT
		
		MOVLW	0X2A        	;Helderheid OLED
		CALL	EE_READ
		MOVFF	EE_IN,BRIGHT

END_R_E:        CALL	MAKE_HM		;Vul de displayvelden

		CALL	MAKE_MODE	;Vul de modevelden

		RETURN

;---------------------------------------------
;INIT_EEPROM = Vul defaultwaardes in de EEPROM
;---------------------------------------------

INIT_EEPROM:	MOVLW	0X00    	;De herkenningsbyte
		MOVWF	EEADR
		MOVLW	0XAA
		MOVWF	EEDATA
		CALL	EE_WRITE
		
		MOVLW	0X01         	;Tel de te schrijven bytes
		MOVFF	WREG,LOOP_CNT
		
		MOVLW	HIGH EE_INIT_TBL;Rijtje met initieele waardes
		MOVWF	TBLPTRH
		MOVLW	LOW EE_INIT_TBL
		MOVWF	TBLPTRL
		
		LFSR	0,T1A_H		;Begin rijtje met tijden

EE_WR_LOOP:	MOVFF	LOOP_CNT,EEADR	;Adres uit teller
		TBLRD*+
		MOVFF	TABLAT,EEDATA   ;Data uit tabel
		MOVFF	EEDATA,POSTINC0	;Naar de waarde
		CALL	EE_WRITE        ;Schrijf maar

		INCF	LOOP_CNT	;Teller omhoog
		MOVLW	0X2B            ;Max. = 2BH
		CPFSEQ	LOOP_CNT        ;Gelijk
		GOTO	EE_WR_LOOP      ;Ongelijk, verder

		RETURN

;--------------------------------------------
;FILL_EEPROM = Schrijf EEPROM na instellingen
;--------------------------------------------

FILL_EEPROM:    MOVLW	0X01         	;Tel de te schrijven bytes
		MOVFF	WREG,LOOP_CNT

		LFSR	0,T1A_H		;Begin rijtje met tijden

EE_WR_LOOP1:	MOVFF	LOOP_CNT,EEADR	;Adres uit teller
		MOVFF	POSTINC0,EEDATA	;Waarde
		CALL	EE_WRITE        ;Schrijf maar

		INCF	LOOP_CNT	;Teller omhoog
		MOVLW	0X21            ;Max. = 21H
		CPFSEQ	LOOP_CNT        ;Gelijk
		GOTO	EE_WR_LOOP1     ;Ongelijk, verder

		MOVLW	0X21         	;Modevelden vanaf 21H
		MOVFF	WREG,LOOP_CNT

		LFSR	0,MODE_T1	;Hier staat de mode

EE_WR_LOOP2:	MOVFF	LOOP_CNT,EEADR	;Adres uit teller
		MOVFF	POSTINC0,EEDATA	;Waarde
		CALL	EE_WRITE        ;Schrijf maar

		INCF	LOOP_CNT	;Teller omhoog
		MOVLW	0X29            ;Max. = 29H
		CPFSEQ	LOOP_CNT        ;Gelijk
		GOTO	EE_WR_LOOP2     ;Ongelijk, verder

		RETURN

;-----------------------------------------
;EE_WRITE = Schrijf de EEPROM
;            Entry: EEADR EN EEDATA gevuld
;-----------------------------------------

EE_WRITE:	CLRF	EEADRH		;Alleen lage byte in gebruik

		BCF	PIR2,EEIF 	;Clear EEPROM interrupt

		BCF	EECON1,EEPGD	;EEPROM enabled
		BCF	EECON1,CFGS	;Selecteer EEPROM (geen conf. reg.)
		BSF	EECON1,WREN	;Enable schrijven

		MOVLW	0X55		;Vaste sequence
		MOVWF	EECON2
		MOVLW	0XAA
		MOVWF	EECON2

		BSF	EECON1,WR       ;Set de write bit

W_F_E2:         BTFSS	PIR2,EEIF       ;Wacht tot de EEPROM interrupt
		GOTO	W_F_E2          ;geset wordt

		BCF	PIR2,EEIF       ;Clear EEPROM interrupt

		BCF	EECON1,WREN
		CLRF	EEDATA

             	RETURN

;--------------------------------------
;EE_READ = Lees een EEPROM locatie
;          Entry: Adres in W
;          Exit : Gelezen data in EE_IN
;--------------------------------------

EE_READ:	MOVWF	EEADR		;Geef het te lezen adres

		BCF	EECON1,EEPGD	;EEPROM enabled
		BCF	EECON1,CFGS	;Selecteer EEPROM (geen conf. reg.)
		BSF	EECON1,RD       ;Set de READ flag
		
		NOP
		NOP

W_READ:		BTFSC	EECON1,RD
		GOTO	W_READ

		MOVFF	EEDATA,EE_IN    ;In de EE_IN file

END_EE_RD:	RETURN

;------------------------------
;DISP_DEF = Display EEPROM leeg
;------------------------------
		
DISP_DEF:	CALL	ERASE_BOTH

		BCF	SHOW

		MOVLW	HIGH EE1_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW EE1_TXT
		MOVWF	TBLPTRL

		MOVLW	0X01
		MOVFF   WREG,H_POS
		MOVLW	0X08
		MOVFF	WREG,V_POS

		CALL	TEXT
		
		MOVLW	HIGH EE2_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW EE2_TXT
		MOVWF	TBLPTRL


		MOVLW	0X01
		MOVFF   WREG,H_POS
		MOVLW	0X18
		MOVFF	WREG,V_POS
		
		CALL	TEXT

		MOVLW	HIGH EE3_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW EE3_TXT
		MOVWF	TBLPTRL


		MOVLW	0X01
		MOVFF   WREG,H_POS
		MOVLW	0X28
		MOVFF	WREG,V_POS
		
		BSF	SHOW
		CALL	TEXT
		BCF	SHOW
		
		RETURN
		
;-------------------------------------------------------------------------
;MAKE_HM = Zet de waarde van de minutenteller om in uren en minuten in hex
;          en schrijf ze in de D velden
;-------------------------------------------------------------------------

MAKE_HM:	MOVLW	0X10    	;16 tijden converteren
		MOVFF	WREG,LOOP_CNT

		LFSR	0,T1A_H         ;De waarde minutenteller
		LFSR	1,T1A_HD        ;De te displayen uren/minuten
		
MAKE_HM_LOOP:	MOVFF	POSTINC0,DIV_H	;Vul met de minutenteller
		MOVFF	POSTINC0,DIV_L

		CALL	DIVIDE_60       ;Zet om naar uren/minuten

		MOVFF	DRES_H,POSTINC1 ;Sla uren/minuten op
		MOVFF	DRES_L,POSTINC1

		DECFSZ	LOOP_CNT        ;En dat 16x
		GOTO	MAKE_HM_LOOP

		RETURN
		
;-----------------------------------------------------------
;MAKE_MODE = Zet de waarde in MODE_Tx om in bits in MODE_TxB
;-----------------------------------------------------------

MAKE_MODE:	LFSR	0,MODE_T1    	;Source
		LFSR	1,MODE_T1B      ;Destination

		MOVLW	0X08            ;Er zijn 8 MODE bytes
		MOVFF	WREG,LOOP_CNT
		
MM_LOOP:	MOVFF	POSTINC0,TEMP	;Haal modegetal 00 .. 70

		CALL	MM_CH           ;Zet om in bits en schrijf weg

		DECFSZ	LOOP_CNT        ;8 x uitvoeren
		GOTO	MM_LOOP
		
		RETURN

;MM_CH = Hulproutine MAKE_MODE, bepaal timer en set modebits
;-----------------------------------------------------------

MM_CH:		CLRF	TEMP2		;Gaat bits beheren

		MOVLW	0X00  		;Bepaal routine aan de hand
		CPFSEQ	TEMP            ;van het MODE getal 00,0A .. 46
		GOTO	CH_MM10
		GOTO	SET_MODE00

CH_MM10:        MOVLW	0X0A
		CPFSEQ	TEMP
		GOTO	CH_MM20
		GOTO	SET_MODE10

CH_MM20:	MOVLW	0X14
		CPFSEQ	TEMP
		GOTO	CH_MM30
		GOTO	SET_MODE20

CH_MM30:	MOVLW	0X1E
		CPFSEQ	TEMP
		GOTO	CH_MM40
		GOTO	SET_MODE30

CH_MM40:	MOVLW	0X28
		CPFSEQ	TEMP
		GOTO	CH_MM50
		GOTO	SET_MODE40

CH_MM50:	MOVLW	0X32
		CPFSEQ	TEMP
		GOTO	CH_MM60
		GOTO	SET_MODE50

CH_MM60:	MOVLW	0X3C
		CPFSEQ	TEMP
		GOTO	CH_MM70
		GOTO	SET_MODE60

CH_MM70:	MOVLW	0X46
		CPFSEQ	TEMP
		GOTO	ERROR_MM
		GOTO	SET_MODE70

SET_MODE00:     MOVFF	TEMP2,POSTINC1	;Alle bits zijn 0
		GOTO	EXIT_MM
		
SET_MODE10:	BSF	TEMP2,0		;Timer aan
		BSF	TEMP2,1         ;7 dagen
		GOTO	END_MM

SET_MODE20:	BSF	TEMP2,0 	;Timer aan
		BSF	TEMP2,2         ;5 dagen
		GOTO	END_MM

SET_MODE30:     BSF	TEMP2,0 	;Timer aan
		BSF	TEMP2,3         ;2 dagen
		GOTO	END_MM

SET_MODE40:	BSF	TEMP2,0		;Timer aan
		BSF	TEMP2,1         ;7 dagen
		BSF	TEMP2,4         ;Lichtdetectie
		GOTO	END_MM

SET_MODE50:	BSF	TEMP2,0		;Timer aan
		BSF	TEMP2,2         ;5 dagen
		BSF	TEMP2,4         ;Lichtdetectie
		GOTO	END_MM
		
SET_MODE60:	BSF	TEMP2,0		;Timer aan
		BSF	TEMP2,3         ;2 dagen
		BSF	TEMP2,4         ;Lichtdetectie
		GOTO	END_MM

SET_MODE70:	BCF	TEMP2,0		;Timer uit
		BSF	TEMP2,4         ;Lichtdetectie
		GOTO	END_MM
		
ERROR_MM:       MOVLW	0X10
		MOVFF	WREG,ERROR_CODE
		GOTO	ERROR_DISP
		
END_MM:		MOVFF	TEMP2,POSTINC1	;Schrijf de bitwaarde weg

EXIT_MM:	RETURN
		
;-----------------------------------------------------
;DIVIDE_60 = Deel door 60
;            Entry: DIV_H, DIV_L is het te delen getal
;            Exit: DRES_H = uren, DRES_L = minuten
;-----------------------------------------------------

DIVIDE_60:	CLRF	DRES_H		;Clear resultaat
		CLRF	DRES_L
		
		MOVLW	0X00		;Check hoge byte op 0
		CPFSEQ	DIV_H
		GOTO	NON_ZH          ;Was geen 0
		
		MOVLW	0X3C 		;Test lage byte op < 60
		CPFSLT	DIV_L
		GOTO	NON_ZH		;Niet < 60
		
		MOVFF	DIV_L,DRES_L	;Lage byte is aantal minuten
		GOTO	END_D60
		
NON_ZH:         BSF	STATUS,C	;Geen borrow
		MOVLW	0X3C		;W wordt 60
		SUBWF	DIV_L		;Trek 60 af van de lage byte
		BTFSC	STATUS,C	;Door de nul?
		GOTO	NO_BORROW 	;Geen borrow
		GOTO    YES_BORROW      ;Wel borrow
		
NO_BORROW:	INCF	DRES_H		;Uren omhoog
		GOTO	NON_ZH		;Volgende
		
YES_BORROW:	DECF	DIV_H		;60 lager
		MOVLW	0XFF  		;Is hij nu FF?
		CPFSEQ	DIV_H
		GOTO	NO_UZ           ;Nee
		GOTO    YES_UZ          ;Ja
		
NO_UZ:		INCF	DRES_H
		GOTO	NON_ZH		;Volgende

YES_UZ:		MOVLW	0X3C        	;Lage byte was nu te laag
		BCF	STATUS,C
		ADDWF	DIV_L           ;Er weer 60 bij
		MOVFF	DIV_L,DRES_L

END_D60:	RETURN

;--------------------------------
;CLEAR_WINDOW = Wis de window RAM
;--------------------------------

CLEAR_WINDOW:	LFSR	0,W_0		;Startpunt shadowRAM

		MOVLW	0X06            ;In 6 slagen van 256
		MOVFF	WREG,GEN_CNT

CL_6:		MOVLW	0X00            ;256 bytes te wissen
		MOVFF	WREG,LOOP_CNT

CL_256W:	CLRF	POSTINC0        ;Wist u maar
		DECFSZ	LOOP_CNT
		GOTO	CL_256W

		DECFSZ	GEN_CNT         ;En dat 4x
		GOTO	CL_6

		RETURN
		
;--------------------------------------------
;FILL_W_RAM = Vul de WindowRAM met 8 vensters
;--------------------------------------------

FILL_W_RAM:	LFSR	0,T1A_H		;Wijs naar area met tijden
		MOVLW	0X01      	;Gaat de bitlokatie bijhouden
		MOVFF	WREG,TEMP2
		
		MOVLW	0X08         	;8 x 2 bytes tellen
		MOVFF	WREG,LOOP_CNT

FWR_LOOP:	MOVFF	POSTINC0,START_H;Lees tijden in
		MOVFF	POSTINC0,START_L
		MOVFF	START_H,T_POINT_H;Bewaar starttijd
		MOVFF	START_L,T_POINT_L

                MOVFF	POSTINC0,STOP_H
		MOVFF	POSTINC0,STOP_L

		CALL	CALC_DURA	;Bereken lengte

		MOVLW	0X00   		;Duration 0 is ongeldig
		CPFSEQ	DURATION_H
		GOTO    CF_L1
		MOVLW	0X00
		CPFSEQ	DURATION_L
		GOTO    CF_L1

		GOTO    FWR_L2

CF_L1:		CALL	TIME_TO_RAM 	;Zet LFSR2 goed

CFL1_LOOP:	MOVLW	0X00		;Test op 000
		CPFSEQ	DURATION_H
		GOTO	WR_RAM
		MOVLW	0X00
		CPFSEQ	DURATION_L
		GOTO	WR_RAM
		GOTO	FWR_L2		;000, dus klaar

WR_RAM:		MOVFF	INDF2,TEMP	;Inhoud in TEMP
		MOVFF	TEMP2,WREG      ;Bitwaarde ophalen
		IORWF	TEMP            ;OR bit met inhoud RAMwindow
		MOVFF	TEMP,POSTINC2   ;Schrijf terug en verhoog

		MOVLW	0X0C 		;Dit is een absoluut adres! WindowRAM loopt
		CPFSEQ	FSR2H           ;van 700H naar C9FH
		GOTO    FWR_CALC
		MOVLW	0XA0
		CPFSEQ	FSR2L
		GOTO    FWR_CALC

		LFSR	2,W_0		;RAM terug naar startpositie

FWR_CALC:	DECF	DURATION_L	;Pointer omlaag
		MOVLW	0XFF
		CPFSEQ	DURATION_L
		GOTO	CFL1_LOOP
		DECF	DURATION_H
		GOTO	CFL1_LOOP

FWR_L2:		RLNCF	TEMP2		;Volgend bit klaarzetten
		
		DECFSZ	LOOP_CNT	;8 rondjes maken
		GOTO	FWR_LOOP

FWR_END:	RETURN

;-------------------------------------------------------------
;TIME_TO_RAM = Wijs naar lokatie in WindowRAM afhankelijk tijd
;              Entry: T_POINT_H/L
;              Exit : LFSR2 wijst in WindowRAM
;-------------------------------------------------------------

TIME_TO_RAM:	LFSR	2,W_0

		BCF	STATUS,C
		MOVFF	T_POINT_L,WREG
		ADDWF	FSR2L
		MOVFF	T_POINT_H,WREG
		ADDWFC	FSR2H

		RETURN
		
;------------------------------------------
;CALC_DURA = Bereken de duur van de periode
;             Entry: START_H/L
;		     STOP_H/L
;             Exit : DURATIONH/L
;------------------------------------------

CALC_DURA:	CLRF	DURATION_H	;Resultaat wissen
		CLRF	DURATION_L

		MOVFF	STOP_H,TEMP_H	;Tussenopslag
		MOVFF	STOP_L,TEMP_L

		BSF	STATUS,C        ;Trek af: STOP - START
		MOVFF	START_L,WREG
		SUBWF	TEMP_L
		MOVFF	START_H,WREG
		SUBWFB	TEMP_H

		MOVFF	TEMP_H,DURATION_H
		MOVFF	TEMP_L,DURATION_L

		BTFSS	DURATION_H,7	;Zijn we door de 0?
		GOTO	DUR_VALID	;Nee

		BCF	STATUS,C    	;Tel 5A0H op bij het
		MOVLW	0XA0            ;negatieve resultaat
		ADDWF	DURATION_L
		MOVLW	0X05
		ADDWFC	DURATION_H

DUR_VALID:	RETURN


;-----------------------------------------------------------------------
;
;Algemene routines
;
;-----------------------------------------------------------------------

;-------------------------------------
;ANA_5 = Meet analoge waarde in 5 bits
;        Entry : TEMP kanaalnummer
;        Exit: waarde in ANA_NEW
;-------------------------------------

ANA_5:		MOVFF	TEMP,ADCON0		;Kanaalnummer
		MOVFF	TEMP,WREG
		NOP
		NOP
		NOP
		NOP
		BSF	ADCON0,GO       ;Start de conversie

W_F_A2:		BTFSC	ADCON0,GO       ;Wacht tot de conversie bit
		GOTO	W_F_A2          ;correct verwerkt is

		CLRF	TEMP

		BTFSC	ADRESH,1	;Maak 5 bits
		BSF	TEMP,4
		BTFSC	ADRESH,0
		BSF	TEMP,3
		BTFSC	ADRESL,7
		BSF	TEMP,2
		BTFSC	ADRESL,6
		BSF	TEMP,1
		BTFSC	ADRESL,5
		BSF	TEMP,0

		MOVFF	TEMP,ANA_NEW

		RETURN

;----------------------------
;READ_PHOTO = Lees de fotocel
;----------------------------

READ_PHOTO:	MOVLW	0X01     	;Kanaal 1 in TEMP
		MOVFF	WREG,TEMP
		CALL	ANA_5           ;Meetwaarde

		CALL	CALC_AV         ;Bereken gemiddelde

		MOVLW	0X0A		;Onder 0A is donker
		CPFSLT	ANA_NEW
		GOTO	CH_LIGHT
		GOTO	ITS_DARK

CH_LIGHT:	MOVLW	0X0A            ;Boven 0A is licht
		CPFSGT	ANA_NEW
		GOTO	END_SBR
		GOTO	ITS_LIGHT

ITS_DARK:	BTFSC	PHOTOCELL	;Staat'ie al op donker?
		GOTO	SET_DARK	;Nee
		GOTO	END_SBR		;Ja

ITS_LIGHT:	BTFSS	PHOTOCELL	;Staat'ie al op licht?
		GOTO	SET_LIGHT
		GOTO	END_SBR

SET_DARK:	BCF	PHOTOCELL	;Flag op donker
		BCF	PHOTO_LED

                BTFSC	S_U_FLAG	;Pas na startup
		GOTO	END_SBR

		BSF	BLOCKED         ;Zet de blokkade aan
		MOVLW	DEF_BLOCK_H
		MOVFF	WREG,BLOCK_CNT_H
                MOVLW	DEF_BLOCK_L
		MOVFF	WREG,BLOCK_CNT_L

		GOTO	END_SBR

SET_LIGHT:	BSF	PHOTOCELL	;Flag op licht
		BSF	PHOTO_LED

                BTFSC	S_U_FLAG	;Pas na startup
		GOTO	END_SBR

                BSF	BLOCKED         ;Zet de blokkade aan
		MOVLW	DEF_BLOCK_H
		MOVFF	WREG,BLOCK_CNT_H
                MOVLW	DEF_BLOCK_L
		MOVFF	WREG,BLOCK_CNT_L
		
END_SBR:	RETURN

;--------------------------------------------------------
;CALC_AV = Bereken gemiddelde lichtwaarde over 8 metingen
;          Exit : ANA_NEW
;--------------------------------------------------------

CALC_AV:	MOVFF	AV_6,WREG       ;Schuif alle waardes 1 plaatsje
		MOVFF	WREG,AV_7
                MOVFF	AV_5,WREG
		MOVFF	WREG,AV_6
                MOVFF	AV_4,WREG
		MOVFF	WREG,AV_5
		MOVFF	AV_3,WREG
		MOVFF	WREG,AV_4
		MOVFF	AV_2,WREG
		MOVFF	WREG,AV_3
		MOVFF	AV_1,WREG
		MOVFF	WREG,AV_2
		MOVFF	AV_0,WREG
		MOVFF	WREG,AV_1

                MOVFF	ANA_NEW,WREG	;Meetwaarde op pos. 1
		MOVFF	WREG,AV_0

                CLRF	AV_TOT 		;Totaal wissen

                MOVFF	AV_0,WREG       ;Bereken totaal
                ADDWF	AV_TOT
                MOVFF	AV_1,WREG
                ADDWF	AV_TOT
                MOVFF	AV_2,WREG
                ADDWF	AV_TOT
                MOVFF	AV_3,WREG
                ADDWF	AV_TOT
                MOVFF	AV_4,WREG
                ADDWF	AV_TOT
                MOVFF	AV_5,WREG
                ADDWF	AV_TOT
                MOVFF	AV_6,WREG
                ADDWF	AV_TOT
                MOVFF	AV_7,WREG
                ADDWF	AV_TOT

                RRNCF	AV_TOT		;Deel door 8
                RRNCF	AV_TOT
                RRNCF	AV_TOT

                BCF	AV_TOT,7
                BCF	AV_TOT,6
                BCF	AV_TOT,5

                MOVFF	AV_TOT,ANA_NEW	;Gemiddelde opslaan

                RETURN

;-------------------------------------------------------
;PROC_DSS = Verwerk de draaischakelaar in veilige mode
;           Entry : SW_DELAY vertraginsfactor (1 = geen)
;-------------------------------------------------------

PROC_DSS:	CLRF	TRANS

                BSF	SW_STAT,4	;Neem aan dat er enen staan
		BSF	SW_STAT,5

		BTFSS   DS1             ;Kopieer nu de ingangsbits naar
		BCF	SW_STAT,4       ;bit 4 en 5 van SW_STAT
		BTFSS	DS2
		BCF	SW_STAT,5

		MOVLW	0XC0 		;Maskeer de oude bits niet
		ANDWF	SW_STAT,0
		MOVFF	WREG,TEMP       ;Bewaar SW_STAT even
		MOVLW	0X30            ;Maskeer nu de nieuwe bits niet
		ANDWF	SW_STAT,0
		RLNCF	WREG            ;Twee naar links
		RLNCF	WREG
		SUBWF	TEMP,0          ;Vergelijk met oude SW_STAT
		BNZ	CH_DET          ;Niet gelijk dus actie
		GOTO	END_PROC_DSS    ;Gelijk

CH_DET:         BTFSC	SW_STAT,4
		BSF	TRANS,0
                BTFSC	SW_STAT,6
		BSF	TRANS,1
		BTFSC	SW_STAT,5
		BSF	TRANS,2
		BTFSC	SW_STAT,7
		BSF	TRANS,3

ANA_STAT:	MOVLW	0X03     	;We moeten de status hebben
		ANDWF   SW_STAT,0
		MOVFF	WREG,TEMP       ;Staat nu in TEMP

		MOVLW	0X00            ;Spring naar 1 van de 3 statusroutines
		SUBWF	TEMP,0
		BZ	DO_STAT_0

		MOVLW	0X01
		SUBWF	TEMP,0
		BZ	DO_STAT_1

		MOVLW	0X02
		SUBWF	TEMP,0
		BZ	DO_STAT_2

		GOTO	END_PROC_DSS	;Mag'ie nooit komen

DO_STAT_0:      MOVLW	0X01            ;TRANS waardes 01H en 0DH staan
		SUBWF	TRANS,0         ;voor een stap
		BNZ     DO_T13          ;Geen geldige stap
		GOTO	DO_DIR          ;Geldig; zet status op 1

DO_T13:         MOVLW	0X0D
		SUBWF	TRANS,0
		BNZ	END_S0          ;Niet geldig, kappen

DO_DIR:		BSF	SW_STAT,0       ;Status op 1
		BCF	SW_STAT,1

END_S0:		GOTO	END_PROC_DSS

DO_STAT_1:      MOVLW	0X02		;02H en 0EH staan voor ongeldig
		SUBWF	TRANS,0
		BNZ	DO_T14          ;Test de 0EH
		GOTO	DO_MST0         ;Was 02H dus status terug

DO_T14:		MOVLW	0X0E
		SUBWF	TRANS,0
		BNZ	DO_T7          ;Geen ongeldige waardes

DO_MST0:	BCF	SW_STAT,0      ;Zet status terug op 0
		BCF	SW_STAT,1
		GOTO	END_PROC_DSS

DO_T7:		MOVLW	0X07		;07H is correct
		SUBWF	TRANS,0
		BNZ	DO_T11          ;Was'ie niet
		GOTO	DO_ST2

DO_T11:		MOVLW	0X0B       	;Ook 0BH kan ermee door
		SUBWF	TRANS,0
		BNZ	END_PROC_DSS    ;Rest niet, dus bye bye

DO_ST2:		BCF	SW_STAT,0    	;Status mag naar 2
		BSF	SW_STAT,1
		GOTO	END_PROC_DSS

DO_STAT_2:      MOVLW	0X07		;07H brengt de status terug op 1
		SUBWF	TRANS,0
		BNZ	DO_T11A		;Hij is geen 7
		GOTO	DO_MST1

DO_T11A:	MOVLW	0X0B 		;Ook 0BH is verkeerd
		SUBWF	TRANS,0
		BNZ	DO_T2

DO_MST1:	BSF	SW_STAT,0	;Status wordt weer 1
		BCF	SW_STAT,1
		GOTO	END_PROC_DSS

DO_T2:		MOVLW	0X02		;Bij 02H is alles goed
		SUBWF	TRANS,0
		BNZ	DO_T14A
		GOTO 	DO_CNT

DO_T14A:	MOVLW	0X0E        	;Ook 0EH wordt goedgekeurd
		SUBWF	TRANS,0
		BNZ	END_PROC_DSS

DO_CNT:		BCF	SW_STAT,0	;Volgende keer weer status 0
		BCF	SW_STAT,1

		BTFSC	SW_STAT,7	;Test de richtingsbit
		GOTO	DO_UP1          ;Een 1 is optellen

		INCF	SW_CNT_D   	;Verhoog de down teller
		MOVFF	SW_DELAY,WREG   ;Vergelijk met de delay
		CPFSEQ	SW_CNT_D        ;Gelijk
		GOTO	END_PROC_DSS    ;Ongelijk
		CLRF	SW_CNT_D
		BCF	DS_CU      	;Flag voor down
		BSF	DS_CD
		BSF	DS_CHANGED
		GOTO	END_PROC_DSS

DO_UP1:         INCF	SW_CNT_U   	;Verhoog de up teller
		MOVFF	SW_DELAY,WREG   ;Vergelijk met de delay
		CPFSEQ	SW_CNT_U        ;Gelijk
		GOTO	END_PROC_DSS    ;Ongelijk
		CLRF	SW_CNT_U
		BSF	DS_CU      	;Flag voor down
		BCF	DS_CD
		BSF	DS_CHANGED

END_PROC_DSS:	BSF	SW_STAT,6	;Update bit 6 en 7
		BSF	SW_STAT,7

		BTFSS	SW_STAT,4       ;Bit 6 komt van bit 4
		BCF	SW_STAT,6

		BTFSS	SW_STAT,5       ;Bit 7 komt van bit 5
		BCF	SW_STAT,7

		RETURN
		
;--------------------------------------------------------
;SET_UNIT = Stel evt. helderheid OLED in
;           Entry: komt alleen hier bij drukknop na reset
;--------------------------------------------------------

SET_UNIT:	BTFSC	ROT_SW
		GOTO	END_SU

		CALL	ERASE_BOTH

		MOVLW	HIGH REL_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW REL_TXT
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS
		
		BSF	SHOW

		CALL	TEXT

WFRSU:		BTFSS	ROT_SW
		GOTO	WFRSU
		
		CALL	BOUNCE_DELAY
		
		CALL	ERASE_BOTH

BR_LOOP:        MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

		MOVLW	HIGH BRIGHT_TXT1
		MOVWF	TBLPTRH
		MOVLW	LOW BRIGHT_TXT1
		MOVWF	TBLPTRL
		
		CALL	TEXT

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

		MOVLW	HIGH BRIGHT_TXT3
		MOVWF	TBLPTRH
		MOVLW	LOW BRIGHT_TXT3
		MOVWF	TBLPTRL

		CALL	TEXT

                MOVLW	0X28
		MOVFF	WREG,H_POS
		MOVLW	0X28
		MOVFF	WREG,V_POS

		MOVLW	HIGH WT
		MOVWF	TBLPTRH
		MOVLW	LOW WT
		MOVWF	TBLPTRL

		CALL	WR_CHAR

		CALL	WRITE_SHADOW

		MOVLW	0X5C
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS
		MOVLW	0X01
		MOVFF	WREG,STEP
		MOVLW	0X0F
		MOVFF	WREG,MAX_VAL

		CALL	ASK_BRIGHT

		BTFSC	ROT_SW
		GOTO	SET_GMT

		GOTO    BR_LOOP

SET_GMT:	BTFSS	ROT_SW
		GOTO	SET_GMT

		CALL	BOUNCE_DELAY

		CALL	ERASE_BOTH

		MOVLW	HIGH TIMEZ_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW TIMEZ_TXT
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

		BSF	SHOW

		CALL	TEXT

		MOVLW	HIGH GMT_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW GMT_TXT
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

		BSF	SHOW

		CALL	TEXT

		MOVLW	0X60
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS
		MOVLW	0X01
		MOVFF	WREG,STEP
		MOVFF	GMT,A_TIME
		MOVLW	0X02
		MOVFF	WREG,MAX_VAL

		CALL	ASK_TIME
		
		CALL	BOUNCE_DELAY
		
		CALL	ERASE_BOTH
		
		MOVLW	HIGH SAVE_TXT0
		MOVWF	TBLPTRH
		MOVLW	LOW SAVE_TXT0
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

                BSF	SHOW

		CALL	TEXT

                MOVLW	HIGH SAVE_TXT1
		MOVWF	TBLPTRH
		MOVLW	LOW SAVE_TXT1
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X10
		MOVFF	WREG,V_POS

                BSF	SHOW

		CALL	TEXT

                MOVLW	HIGH SAVE_TXT2
		MOVWF	TBLPTRH
		MOVLW	LOW SAVE_TXT2
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X20
		MOVFF	WREG,V_POS

                BSF	SHOW

		CALL	TEXT

		MOVLW	0X29
		MOVFF	WREG,EEADR
		MOVFF	GMT,EEDATA
		CALL	EE_WRITE

		MOVLW	0X2A
		MOVFF	WREG,EEADR
		MOVFF	BRIGHT,EEDATA
		CALL	EE_WRITE
		
		MOVLW	0X03
		CALL	L_TIMER

END_SU:		RETURN

;------------------------------------
;INIT_408 = Inintialiseer de GPS unit
;------------------------------------

INIT_408:       MOVLW	HIGH DISA_GGA	;Disable GGA
		MOVWF	TBLPTRH
		MOVLW	LOW DISA_GGA
		MOVWF	TBLPTRL

		LFSR	0,SEND_SER_BUF

		CALL	F_S_BUF

		MOVLW	HIGH DISA_GLL	;Disable GLL
		MOVWF	TBLPTRH
		MOVLW	LOW DISA_GLL
		MOVWF	TBLPTRL

		LFSR	0,SEND_SER_BUF

		CALL	F_S_BUF

		MOVLW	HIGH DISA_GSA	;Disable GSA
		MOVWF	TBLPTRH
		MOVLW	LOW DISA_GSA
		MOVWF	TBLPTRL

		LFSR	0,SEND_SER_BUF

		CALL	F_S_BUF

		MOVLW	HIGH DISA_GSV	;Disable GGV
		MOVWF	TBLPTRH
		MOVLW	LOW DISA_GSV
		MOVWF	TBLPTRL

		LFSR	0,SEND_SER_BUF

		CALL	F_S_BUF

		MOVLW	HIGH DISA_VTG	;Disable VTG
		MOVWF	TBLPTRH
		MOVLW	LOW DISA_VTG
		MOVWF	TBLPTRL

		LFSR	0,SEND_SER_BUF

		CALL	F_S_BUF

		MOVLW	HIGH ENA_RMC	;Enable RMC
		MOVWF	TBLPTRH
		MOVLW	LOW ENA_RMC
		MOVWF	TBLPTRL

		LFSR	0,SEND_SER_BUF

		CALL	F_S_BUF

		RETURN

;F_S_BUF = Hulproutine INIT_408
;------------------------------

F_S_BUF:	TBLRD*+			;Lees byte uit tabel
		MOVFF	TABLAT,TEMP
		MOVFF	TEMP,POSTINC0

		MOVLW	0XFF            ;Was dat een FF?
		CPFSEQ	TEMP            ;Ja
		GOTO	F_S_BUF

  		CALL	SEND_SERIAL

		MOVLW	0X01 		;Wacht 4 msec.
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0X04
		MOVFF	WREG,DELAY_LOC
		CALL	DELAY

		RETURN
		
;----------------------------------------------
;ERROR_DISP = Schrijf een error in directe mode
;             Entry: Error in ERROR_CODE
;Let op : routine hangt zichzelf op
;----------------------------------------------

ERROR_DISP:	CALL	ERASE_BOTH

		MOVLW	HIGH D_ERROR
		MOVWF	TBLPTRH
		MOVLW	LOW D_ERROR
		MOVWF	TBLPTRL

		MOVLW	0X18
		MOVFF	WREG,H_POS
		MOVLW	0X12
		MOVFF	WREG,V_POS

		CALL	DIRECT_WRITE

		MOVLW	0X58
		MOVFF	WREG,H_POS
		MOVLW	0X12
		MOVFF	WREG,V_POS

		MOVFF	ERROR_CODE,TEMP
		CALL    MAKE_TBL

		CALL	DIRECT_WRITE
		
WFE:		GOTO	WFE

;--------------------------------------------------
;MAKE_TBL = Pas de TBLPTR aan aan de waarde in TEMP
;           Startpunt is D_0
;           Entry: Waarde in TEMP
;--------------------------------------------------

MAKE_TBL:       MOVLW	HIGH D0		;Startpunt digits 0 .. 0C
		MOVWF	TBLPTRH         ;0 .. 9 cijfers
		MOVLW	LOW D0	        ;0A = spatie
		MOVWF	TBLPTRL         ;0B = minus
					;0C = error
		MOVLW	0X36		;Stapgrootte = 54
		MULWF	TEMP

		BCF	STATUS,C  	;Carry weg
		MOVFF	PRODL,WREG      ;Haal lage resultaat
		ADDWF	TBLPTRL         ;Tel op
		BTFSC	STATUS,C        ;Carry?
		INCF	TBLPTRH         ;Ja
		BCF	STATUS,C
		MOVFF	PRODH,WREG      ;Tel hoge resultaat op
		ADDWF	TBLPTRH

		RETURN
		
;--------------------------
;WELCOME = Toon opstartfase
;--------------------------

WELCOME:        CALL	ERASE_BOTH

		MOVLW	HIGH UNIT_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW UNIT_TXT
		MOVWF	TBLPTRL

		MOVLW	0X00
		MOVFF	WREG,H_POS
		MOVLW	0X00
		MOVFF	WREG,V_POS

		CALL	TEXT

                MOVLW	HIGH START_TXT
		MOVWF	TBLPTRH
		MOVLW	LOW START_TXT
		MOVWF	TBLPTRL

		MOVLW	0X0C
		MOVFF	WREG,H_POS
		MOVLW	0X20
		MOVFF	WREG,V_POS

		BSF	SHOW
		CALL	TEXT
		
		RETURN

;-----------------------------
;SHOW_ALIVE = Toon levensteken
;-----------------------------

SHOW_ALIVE:     MOVLW	0X08		;Ga 8 x knipperen
		MOVFF	WREG,GEN_CNT

		MOVLW	0X02
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0X08
		MOVFF	WREG,DELAY_LOC

FLASH_SERV:	BCF	SERV_LED

		CALL	DELAY

		BSF	SERV_LED

		CALL	DELAY

		DECFSZ	GEN_CNT
		GOTO	FLASH_SERV

                BCF	SERV_LED

		RETURN
		
;-----------------------------------------------
;SAVE_LFS = Push of pop LFSR registers en TBLPTR
;           Entry: WREG 0 is push, WREG 1 is pop
;-----------------------------------------------

SAVE_LFS:	BTFSC	WREG,0
		GOTO	DO_S_POP
		
		MOVFF   FSR0H,FSR0H_BU
		MOVFF   FSR0L,FSR0L_BU
		MOVFF   FSR1H,FSR1H_BU
		MOVFF   FSR1L,FSR1L_BU
		MOVFF   FSR2H,FSR2H_BU
		MOVFF   FSR2L,FSR2L_BU

		MOVFF	TBLPTRH,TPH_BU
		MOVFF	TBLPTRL,TPL_BU
		
		GOTO	END_SL
		
DO_S_POP:       MOVFF   FSR0H_BU,FSR0H
		MOVFF   FSR0L_BU,FSR0L
		MOVFF   FSR1H_BU,FSR1H
		MOVFF   FSR1L_BU,FSR1L
		MOVFF   FSR2H_BU,FSR2H
		MOVFF   FSR2L_BU,FSR2L

		MOVFF	TPH_BU,TBLPTRH
		MOVFF	TPL_BU,TBLPTRL
		
END_SL:		RETURN

;------------------------
;S_U = Start-up procedure
;------------------------

S_U:		BTFSS	S_U_FLAG	;Start-up actief?
		GOTO	END_S_U         ;Nee
		
		MOVLW	0X00   		;Test timer op 0
		CPFSEQ	S_U_TIMER       ;Ja
		GOTO	ADJ_S_U         ;Nee

		BCF	S_U_FLAG
		
		CALL	ERASE_BOTH

		GOTO	END_S_U

ADJ_S_U:	DECF	S_U_TIMER

		MOVFF	S_U_TIMER,BEAM_LENGTH
		CALL	SHOW_BEAM

END_S_U:	RETURN

;------------------------------------------------------------------
;FORCE_BUTTON = Override de drukknop bij ingave tijd na een periode
;------------------------------------------------------------------

FORCE_BUTTON:	INCF	FORCE_TMRL
		MOVLW	0XB8
		CPFSEQ	FORCE_TMRL
		GOTO	END_FB

		BSF	BUT_FORCED

		CLRF	FORCE_TMRL

END_FB:		RETURN

;-----------------------------------------------------
;do_dump = stuur 16 bytes naar de buitenwereld
;          entry: dd_nr 0.. 3 = aantal te zenden bytes
;                 dd_nr 7 =  0 ontvanger begint op 0
;                            1 ontvanger telt door
;                 dd_0 t/ dd_15 = de data
;          uitvoering: iic op c3/c4
;-----------------------------------------------------

do_dump:	bcf	sspcon1,sspen

		bcf	trisc,3		;lijnen als uitgang
		bcf	trisc,4
		bcf	latc,3		;beiden op 0
		bcf	latc,4

		movlw	0x00     	;syncbyte 0
		call	send_dump

		nop
		nop

		movlw	0xaa  		;syncbyte 1
		call	send_dump

		nop
		nop

                movlw	0x03            ;effu pausu
                call	dts

                movlw	0x55            ;syncbyte 2 is 55h als de ontvanger
		btfsc	dd_nr,0x07      ;de data moet opslaan vanaf adres 0
		movlw	0x5a            ;syncbyte 2 is 5a als de ontvanger
		call	send_dump       ;moet doortellen bij het opslaan

		nop
		nop

		movlw	0x7f            ;strip bit 7
		andwf	dd_nr           ;resteert aantal te zenden bytes
		movff	dd_nr,loop_cnt  ;zet in de teller
		movff	dd_nr,wreg
		call	send_dump	;zendt het aantal

		nop
		nop

		lfsr	2,dd_0 		;begin op dd_0

bytes_to_dump:	movff	postinc2,wreg   ;zenden maar
		call	send_dump

		nop
		nop

		decfsz  loop_cnt
		goto	bytes_to_dump

		bsf	trisc,3
		bsf	trisc,4
		bsf	latc,3
		bsf	latc,4

		bsf	sspcon1,sspen

		movlw	0x0a
		call	dts

		return

;send_dump = hulproutine voor do_dump
;------------------------------------

send_dump:	movff	wreg,temp	;de te zenden byte

		bsf	dump_data       ;set de data
		btfss	temp,7
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

                bsf	dump_data       ;set de data
		btfss	temp,6
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

		bsf	dump_data       ;set de data
		btfss	temp,5
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

		bsf	dump_data       ;set de data
		btfss	temp,4
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

		bsf	dump_data       ;set de data
		btfss	temp,3
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

		bsf	dump_data       ;set de data
		btfss	temp,2
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

		bsf	dump_data       ;set de data
		btfss	temp,1
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag
                
		bsf	dump_data       ;set de data
		btfss	temp,0
		bcf	dump_data
		nop
		nop
		nop
		nop
		bsf	dump_clock      ;clock hoog
		nop
		nop
		nop
		nop
		bcf	dump_clock      ;clock laag

		return

;--------------------------------------------
;INIT = Initialiseer controller en variabelen
;--------------------------------------------

INIT:           BSF	RCON,POR        ;Reset POR

		BCF	INTCON,GIEH	;Disable general interrupt high priority

		MOVLW	0X00  		;Oscillator op external 11.0592 MHz
		MOVWF	OSCCON
		
		BSF	ADCON0,ADON 	;1 lijn analoog
		MOVLW	0X00
		MOVFF	WREG,ADCON1
		MOVLW	0X01
		MOVFF	WREG,ANSELA	;A0 analoog
		MOVLW	0X00
		MOVFF	WREG,ANSELB
		MOVFF	WREG,ANSELC
		MOVFF	WREG,ANSELD
		MOVFF	WREG,ANSELE

		BSF	TRISA,0 	;Ingang fotocel

		BCF	TRISA,2		;ServiceLED
                BCF	TRISA,3		;Fotocel LED
		BCF	TRISA,4         ;Testpunt
		BSF	TRISA,5		;Jumper

		BSF	TRISA,6         ;Kristal
                BSF	TRISA,7

		CLRF	TRISB		;B is uitgang naar relais
		CLRF	PORTB

		CLRF	TRISC
		BSF	TRISC,7		;RS232 in

		CLRF	TRISD
		
		BSF	TRISE,0
		BSF	TRISE,1
		BSF	TRISE,2

                CLRF	FLAGS
                CLRF	FLAGS2
                CLRF	FLAGS3
                CLRF	FLAGS4

                BSF	S_U_FLAG

		CLRF	GEN_CNT
		CLRF	LOOP_CNT
		CLRF	TEMP
		CLRF	TEMP2
		CLRF	TEMP_H
		CLRF	TEMP_L
		
		CLRF	WREG_LOC
		CLRF	TEMP_LOC

		CLRF	DELAY_LOC
		CLRF	DELAY_TYPE
		CLRF	DELAY_H
		CLRF	DELAY_L
		CLRF	CNT_DELAY_H
		CLRF	CNT_DELAY_L
		CLRF	MAIN_DELAY
		CLRF	DTS_TIMER
		CLRF	L_CNT

		CLRF	WD_TIMER_RS232H
		CLRF	WD_TIMER_RS232L
		
		CLRF	CHAR_CNT

		CLRF	INT_CNT

		CLRF	PIC_SEC_HEX
		CLRF	PIC_MIN_HEX
		CLRF	PIC_HR_HEX
		
		CLRF	GPS_STATUS
		
		CLRF	HR_10_ASC
		CLRF	HR_1_ASC
		CLRF	MIN_10_ASC
		CLRF	MIN_1_ASC
		CLRF	SEC_10_ASC
		CLRF	SEC_1_ASC

		CLRF	DAY_10_ASC
		CLRF	DAY_1_ASC
		CLRF	MONTH_10_ASC
		CLRF	MONTH_1_ASC
		CLRF	YEAR_10_ASC
		CLRF	YEAR_1_ASC
		
		CLRF	HR_BCD
		CLRF	MIN_BCD
		CLRF	SEC_BCD

		CLRF	DAY_BCD
		CLRF	MONTH_BCD
		CLRF	YEAR_BCD

		CLRF	GPS_HR_HEX
		CLRF	GPS_MIN_HEX
		CLRF	GPS_SEC_HEX

		CLRF	G_HR_HEX
		CLRF	G_MIN_HEX
		CLRF	G_SEC_HEX

		CLRF	G_VAL_U
		CLRF	G_VAL_H
		CLRF	G_VAL_L

                CLRF	PIC_HR_HEX
                CLRF	PIC_MIN_HEX
		CLRF	PIC_SEC_HEX
		
		CLRF	PIC_HR_BCD
		CLRF	PIC_MIN_BCD
		CLRF	PIC_SEC_BCD
		
		CLRF	P_HR_HEX
		CLRF	P_MIN_HEX
		CLRF	P_SEC_HEX
		
		CLRF	P_VAL_U
		CLRF	P_VAL_H
		CLRF	P_VAL_L

		CLRF	SUM
		CLRF	CS_H
		CLRF	CS_L
		
		CLRF	OFFSET_T
		
		CLRF	TOT_HRS_H
                CLRF	TOT_HRS_L

		CLRF	LIMD_HRS_H
		CLRF	LIMD_HRS_L
		
		CLRF	LIMU_HRS_H
		CLRF	LIMU_HRS_L
		
		CLRF	ADJUST_CNT

		CLRF	GPS_ERR_CNT

		CLRF	SUB1_U
		CLRF	SUB1_H
		CLRF	SUB1_L
		
		CLRF	SUB2_U
		CLRF	SUB2_H
		CLRF	SUB2_L
		
		CLRF	SUBRES_U
		CLRF	SUBRES_H
		CLRF	SUBRES_L
		
		CLRF	MIN_CNT_H
		CLRF	MIN_CNT_L

		SETF	OLD_MIN
		
		CLRF	STATUS_CNT
		
		CLRF	EE_IN
		
		CLRF	DIV_H
		CLRF	DIV_L

		CLRF	DRES_H
		CLRF	DRES_L
		
		CLRF	A_TIME
		
		CLRF	ERROR_CODE
		
		CLRF	S_POS
		
		CLRF	SW_STAT
		CLRF	TRANS
		CLRF	SW_CNT_U
		CLRF	SW_CNT_D
		CLRF	SW_DELAY
		
		CLRF	LOWER_LIM
		
		CLRF	GEN_MODE
		
		CLRF	HP_STORE
		CLRF	VP_STORE
		
		CLRF	MAX_VAL
		
		CLRF	STEP
		
		CLRF	START_H
		CLRF	START_L
		
		CLRF	STOP_H
		CLRF	STOP_L

		CLRF	T_POINT_H
		CLRF	T_POINT_L
		
		CLRF	DURATION_H
		CLRF	DURATION_L
		
		CLRF	REL_BYTE

		LFSR	0,SEC_0_BUF
		MOVLW	0X20
		MOVFF	WREG,LOOP_CNT
ERA0:		CLRF	POSTINC0
		DECFSZ	LOOP_CNT
		GOTO	ERA0
		
		CLRF	CHAR_WIDTH
		CLRF	CHAR_HEIGHT
		CLRF	CHAR_WIDTH2
		
		CLRF	PAGE_NR
		CLRF	COL_NR
		CLRF	COL_H
		CLRF	COL_L

		CLRF	H_POS
		CLRF	V_POS

		CLRF	ADR_LFH
		CLRF	ADR_LFL

                CLRF	COMM_BYTE
		CLRF	COMM_BYTE1
                CLRF	COMM_BYTE2
                CLRF	COMM_BYTE3
                CLRF	COMM_BYTE4
                CLRF	COMM_BYTE5

                CLRF	OFFSET

		CLRF	TYPE_BYTE
		CLRF	TYPE_BYTE1
		CLRF	TYPE_BYTE2
		CLRF	TYPE_BYTE3
		CLRF	TYPE_BYTE4
		CLRF	TYPE_BYTE5
		CLRF	SEQ_BYTE

		CLRF	CONT_BYTE
		
		CLRF	TPH_BU
		CLRF	TPL_BU

		CLRF	ANA_OLD
		CLRF	ANA_NEW
		
		CLRF	REL_STATUS
		
		CLRF	AV_TOT
		
		CLRF	BLOCK_CNT_H
		CLRF	BLOCK_CNT_L
		
		MOVLW	0X0F
		MOVFF	WREG,S_U_TIMER
		
		CLRF	L3_CNT
		
		CLRF	BEAM_LENGTH

		CLRF	FORCE_TMRL
		
		CLRF	GMT

		LFSR	0,T1A_H
		MOVLW	0XFF
		MOVFF	WREG,LOOP_CNT
ERA1:		MOVLW	0X00
		MOVFF	WREG,POSTINC0
                DECFSZ	LOOP_CNT
		GOTO	ERA1

		LFSR	0,TXT_AREA
		MOVLW	0X20
		MOVFF	WREG,LOOP_CNT
ERA2:		CLRF	POSTINC0
		DECFSZ	LOOP_CNT
		GOTO	ERA2

		LFSR	0,REC_SER_BUF
		MOVLW	0X40
		MOVFF	WREG,LOOP_CNT
ERA3:		CLRF	POSTINC0
		DECFSZ	LOOP_CNT
		GOTO	ERA3

		LFSR	0,SEND_SER_BUF
		MOVLW	0X80
		MOVFF	WREG,LOOP_CNT
ERA4:		CLRF	POSTINC0
		DECFSZ	LOOP_CNT
		GOTO	ERA4
		
		MOVLW	HIGH H_POS_TBL	;Vul RAM locaties voor
		MOVWF	TBLPTRH         ;de horizontale posities
		MOVLW	LOW H_POS_TBL
		MOVWF	TBLPTRL
		LFSR	0,H_POS1
		MOVLW	0X0A
		MOVFF	WREG,LOOP_CNT
F_H_P:		TBLRD*+
		MOVFF	TABLAT,POSTINC0
		DECFSZ	LOOP_CNT
		GOTO	F_H_P
		
		MOVLW	0X1F
		MOVFF	WREG,AV_0		;Nodig voor berekening gem. lichtwaarde
                MOVFF	WREG,AV_1
                MOVFF	WREG,AV_2
                MOVFF	WREG,AV_3
                MOVFF	WREG,AV_4
                MOVFF	WREG,AV_5
                MOVFF	WREG,AV_6
                MOVFF	WREG,AV_7

		CALL	CLEAR_WINDOW	;Wis de windowRAM

                BSF	ADCON0,ADON	;Enable A/D metingen
                MOVLW	0X0D		;2 kanalen
		MOVWF	ADCON1
		MOVLW	0X96
		MOVWF	ADCON2

		MOVLW	0X83		;Timer 0 aan
		MOVWF	T0CON
		BSF	INTCON2,TMR0IP	;Timer 0 high priority
		BSF	INTCON,TMR0IE	;Enable timer 0 interrupt
		BCF	INTCON,INT0IF

		MOVLW	0X20 		;8 bit asynchroon zenden
		MOVWF	TXSTA
		MOVLW	0X90		;Idem voor ontvangen
		MOVWF	RCSTA
		MOVLW	0X00
		MOVFF	WREG,BAUDCON
		MOVLW	0X47 		;Baudrate 9600 Bd
		MOVWF	SPBRG

                BCF	EECON1,EEPGD	;Prepareer de EEPROM
		BCF     EECON1,CFGS

		RETURN 

;---------------------
;Start hoofdprogramma
;---------------------

MAIN:  		CALL	INIT 		;Alles initialiseren

		CALL	SHOW_ALIVE	;Knipper serviceLED

                CALL	READ_EEPROM 	;Lees mode en timers
                
                MOVLW	0X01   		;Rust voor opstart OLED
                CALL	L_TIMER

		CALL	RESET_OLED      ;OLED opstarten

                CALL	ERASE_BOTH

		MOVLW	0X02          	;200 msec. wachttijd
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0X28
		MOVFF	WREG,DELAY_LOC
		CALL	DELAY

		CALL	SET_UNIT	;Stel evt. helderheid en fotocel in

                CALL	FILL_W_RAM	;Vul de window RAM

		CALL	WELCOME		;Toon de welkomtekst

		CALL	DDT1

            	CALL	INIT_408        ;Enable alleen RMC

		BSF	NO_GPS		;Geen GPS

		BCF	FIRST_GPS	;Geen eerste GPS string geweest

                BCF	RCSTA,CREN 	;Reset evt. overrun
		BSF	RCSTA,CREN

		BSF	SERV_LED	;Flag geen RS232

WE_RS:		BTFSS	RS_IN		;Is er RS232?
		GOTO	WE_RS           ;Nee

		MOVLW	0X02 		;Ja, dus wacht initieel 200 msec.
		MOVFF	WREG,DELAY_TYPE
		MOVLW	0X28
		MOVFF	WREG,DELAY_LOC
		CALL	DELAY

		BCF	SERV_LED        ;Flag dat er RS232 is

C_R_S:		CALL	REC_SERIAL	;Lees een de 1e string
                
		BTFSC	RS232_ERR       ;Error?
		GOTO	C_R_S           ;Ja, dus wacht op volgende string

		CALL	GPRMC           ;Test geldigheid string
		BTFSS	DATA_VALID      ;Geldig?
		GOTO	C_R_S           ;Helaas

		BCF	NO_GPS		;Er is GPS geweest

		BSF	FIRST_GPS       ;Dus aan de eerste keer is voldaan

		MOVLW	0X03   		;3 sec. wachten
		CALL	L_TIMER

		CALL	ERASE_BOTH  	;Wissen

		CALL	CLOCK_SCREEN    ;Ga wachten tot klok correct is
		
		BSF	INTCON,GIEH     ;Interrupts aan

MAIN_LOOP:	CALL	REC_SERIAL      ;Is er een string?

		BTFSC	RS232_ERR       ;Geldig?
		GOTO    ML_INTERRUPTED	;Fout gezien

		GOTO    ML_CONT         ;String is compleet

ML_INTERRUPTED:	MOVLW	GPS_ERR_MAX	;Haal max aantal fouten
		CPFSEQ	GPS_ERR_CNT     ;Is dat bereikt?
		GOTO	CONT_ERROR	;Nog niet
		BSF	NO_GPS 		;Flag ontbreken GPS signaal
		BSF	SERV_LED	;ServiceLED aan ten teken fout
		GOTO	MAIN_LOOP       ;Ja, dus blijf wachten

CONT_ERROR:	INCF	GPS_ERR_CNT	;GPS ontvangst fout verhogen
		BCF	GPS_STABLE      ;Opeenvolgingsfout
		GOTO	MAIN_LOOP	;Wacht op string

ML_CONT:	BTFSS	INT		;Sync op interrupt
		GOTO	ML_CONT
		BCF	INT

		CALL	GPRMC 		;Test de inhoud van de string
		BTFSS	DATA_VALID      ;Is die goed?
		GOTO    STR_REJECT      ;Nee
		GOTO    STRING_OK       ;Ja hoor

STR_REJECT:     MOVLW	GPS_ERR_MAX	;Haal max aantal fouten
		CPFSEQ	GPS_ERR_CNT     ;Is dat bereikt?
		GOTO	CONT_ERROR	;Nog niet
		BSF	NO_GPS		;Flag fouten in GPS signaal
		BSF	SERV_LED	;ServiceLED aan ten teken fout
		GOTO	MAIN_LOOP       ;Ja, dus blijf wachten

STRING_OK:	BCF	NO_GPS     	;Er is GPS

		CLRF	GPS_ERR_CNT     ;Foutenteller mag op 0

		CALL	TEST_STABILITY	;Is de GPS betrouwbaar?

		BTFSC	GPS_STABLE	;Indien nee, skip
		CALL	CORRECTION	;Moet er bijgesteld worden?

		CALL	S_U		;Roep opstartdelay
		BTG	SERV_LED 	;Toon goede string ontvangen

		BTFSC	STATUS_ACT	;Statusscherm aan?
		CALL	STATUS_SCREEN   ;Ja

		BTFSS	ERA_STAT        ;Statusscherm wissen?
		GOTO	C_ML            ;Nee

		CLRF	WREG
		CALL	SAVE_LFS
		BCF	ERA_STAT        ;Wissen maar!
		CALL	ERASE_BOTH
		SETF	WREG
		CALL	SAVE_LFS

C_ML:		GOTO	MAIN_LOOP

		END


